###########################################################################
#
#
# Name:        cam_test.tcl
#
# Description: 
#    This script defines the eDiag test template
#                        
###########################################################################


# Specify any dependency on other packages
package require Tcl

# Specify the package this module provides
package provide _e4_tests 1.0

namespace eval _e4_tests {
    
    # Export the package
    namespace export e4_tests   
}

# Give a short-name for the regression test.
interp alias {} cam_test {}  "_e4_tests::cam_test_main"


#Globals:

#Registers information in 4 arrays
global regsName
global regsOffset
global regsSize
global regsType
global regsResetVal


# -------------------------------------------------------
# The description of the test usage
# ------------------------------------------------------
proc _e4_tests::cam_test_usage {} {
    puts " cam_test - checks CSAM BIST mechanism"
    puts "Usage: cam_test"    
}

# -------------------------------------------------------
# The test main proc
# -------------------------------------------------------
proc _e4_tests::cam_test_main { } { 
    
    global rdfFile
    global regsName
    global regsOffset

    #Parse the appropriate RDF file 
    parse_rdf

    puts "cam_test - Checks CAM BIST"
    
    # Initialization 
    cam_test_init
    set map_tbl_sz [expr ([reg_count -n "IGU_REG_MAPPING_MEMORY"]*4)]
    set buf_map [hmem palloc $map_tbl_sz "MAPPING table"]

    #IGU CAM save
    save_map_table $buf_map $map_tbl_sz

    # Test procedures/commands
    set returnVal [cam_test_test] 
       
    # Cleanup before exiting test
    restore_map_table $buf_map $map_tbl_sz
    mcp run
    reset_chip
    
    if {$returnVal != "passed"} {
        puts "***********************************************************"
        puts "**             TEST FAILED !!!                             "
        puts "***********************************************************"
    } else {
        # Verify idle_chk passes after this test
        #driver load
        #if {[idleChk -noWarn] == 0} {        
            puts "***********************************************************"
            puts "**             TEST PASSED !!!                             "
            puts "***********************************************************"
        #} else {        
        #    puts "Idle_chk after cam_test failed !!!"
        #    set returnVal "failed"
        #}        
        #driver unload    
    }
    
    # Test should return passed/failed
    return $returnVal
}


# -------------------------------------------------------
# The test init proc
# -------------------------------------------------------
proc _e4_tests::cam_test_init { } {
           
    # reset the chip and halt the MCP before testing
    mcp halt
    reset_chip 1         

}

# -------------------------------------------------------
# Adds value to and returns the result
# -------------------------------------------------------
proc _e4_tests::add_to_addr {addr val} {
    set lo32Negative [expr [lindex $addr 0]<0]
    set lo32Result [expr ([lindex $addr 0] + $val)]
    set hi32Result [lindex $addr 1]
    if {$lo32Negative && $lo32Result >= 0} {
	set hi32Result [expr $hi32Result + 1]
    }
    return [list $lo32Result $hi32Result]
}

# -------------------------------------------------------
# IGU CAM save
# -------------------------------------------------------
proc _e4_tests::save_map_table {buf_map map_tbl_sz} {
    set offset [get_start_offset [reg_offset -n "IGU_REG_MAPPING_MEMORY"]]
    puts "IGU Mapping CAM save"
    for {set i 0} {$i < $map_tbl_sz} {incr i 4} {    
        hmem write [add_to_addr $buf_map $i] [reg read [expr ($offset + $i)]]
    }
}

# -------------------------------------------------------
# IGU CAM restore
# -------------------------------------------------------
proc _e4_tests::restore_map_table {buf_map map_tbl_sz} {

    #Disable CAM BIST 
    if {![is_e4_K2]} {
        reg_write -n IGU_REG_CAM_BIST_EN -v 0x0
    }

    set offset [get_start_offset [reg_offset -n "IGU_REG_MAPPING_MEMORY"]]
    puts "IGU Mapping CAM restore"
    for {set i 0} {$i < $map_tbl_sz} {incr i 4} {
	set val [hmem read [add_to_addr $buf_map $i]]
	set addr [expr {$offset+$i}]
        reg write $addr $val
    }
    
    hmem free $buf_map
}

# -------------------------------------------------------
# CAM BIST enable
# -------------------------------------------------------
proc _e4_tests::cam_test_bist_en { } {
    
    #BIST enable
    #device 1

    #*** Engine 0 ***
    # TCFC
    reg_write -n       TCFC_REG_CID_CAM_BIST_EN    -v 0x1 
    reg_write -n       TCFC_REG_STRING_CAM_BIST_EN -v 0x1 
    
    # CCFC
    reg_write -n       CCFC_REG_CID_CAM_BIST_EN    -v 0x1 
    reg_write -n       CCFC_REG_STRING_CAM_BIST_EN -v 0x1 
    
    # IGU
    #reg_write -n       IGU_REG_CAM_BIST_EN    -v 0x1 
    
    # XSEM
    reg_write -n      XSEM_REG_FAST_MEMORY_CAM_BIST_EN -v 0x1 
    # YSEM
    reg_write -n      YSEM_REG_FAST_MEMORY_CAM_BIST_EN -v 0x1 
    # PSEM
    reg_write -n      PSEM_REG_FAST_MEMORY_CAM_BIST_EN -v 0x1 
    # TSEM
    reg_write -n      TSEM_REG_FAST_MEMORY_CAM_BIST_EN -v 0x1 
    # MSEM
    reg_write -n      MSEM_REG_FAST_MEMORY_CAM_BIST_EN -v 0x1 
    # USEM
    reg_write -n      USEM_REG_FAST_MEMORY_CAM_BIST_EN -v 0x1
}


# -------------------------------------------------------
# CAM BIST status read
# -------------------------------------------------------
proc _e4_tests::cam_test_bist_read { ind } {
    
    switch $ind {
        "0"  {set cam_bist_status_addr   TCFC_REG_CID_CAM_BIST_STATUS}
        "1"  {set cam_bist_status_addr   TCFC_REG_STRING_CAM_BIST_STATUS}
        "2"  {set cam_bist_status_addr   CCFC_REG_CID_CAM_BIST_STATUS}
        "3"  {set cam_bist_status_addr   CCFC_REG_STRING_CAM_BIST_STATUS}
        #"4"  {set cam_bist_status_addr   IGU_REG_CAM_BIST_STATUS}
        "5"  {set cam_bist_status_addr   XSEM_REG_FAST_MEMORY_CAM_BIST_STATUS}
        "6"  {set cam_bist_status_addr   YSEM_REG_FAST_MEMORY_CAM_BIST_STATUS}
        "7"  {set cam_bist_status_addr   PSEM_REG_FAST_MEMORY_CAM_BIST_STATUS}
        "8"  {set cam_bist_status_addr   TSEM_REG_FAST_MEMORY_CAM_BIST_STATUS}
        "9"  {set cam_bist_status_addr   MSEM_REG_FAST_MEMORY_CAM_BIST_STATUS}
        "10" {set cam_bist_status_addr   USEM_REG_FAST_MEMORY_CAM_BIST_STATUS}
    }
    set status_val [reg_read -n $cam_bist_status_addr]
    return $status_val
}


# -------------------------------------------------------
# The test commands
# -------------------------------------------------------
proc _e4_tests::cam_test_test {  } {
    
    set cams_number 11
    
    #BIST enable
    cam_test_bist_en

    delay ms 50
    
    #BIST status read
    puts stdout "read from all CAM's bist_status register"
    for {set i 0} {$i < $cams_number} {incr i 1} {
	if {$i == 4} {
	    set i 5
	}
	set status_val [cam_test_bist_read $i]
	    puts "read BIST status of CAM $i is $status_val"
	if {$i == 1} {
	    puts stdout "TCFC String CAM BIST check is temporarily invalidated due to known BIST enable bug"
	} else {
	    if {$status_val != 0x3} {
		return "failed"
	    } else {
		puts stdout "read from CAM $i passed"		
	    }
	}
    }
	
    set returnVal "passed"      
    return $returnVal
}





