 
proc dump_mb {} {
	 set num_windows [reg read 0x5040c]
	 set first_window [reg read 0x50500]
	 set MFW_SECURITY_MODE [reg read 0x009078]
	 set MFW_SECURITY_MODE_IO [reg read 0x00907c]
	 set chip_num [reg_read -n MISCS_REG_CHIP_NUM]
	 if {[expr {$chip_num & 0xff00}] == 0x8000} {
		 set is_ah 1
	 } else {
		 set is_ah 0
	 }
	 puts "num_windows: $num_windows. first one: $first_window"
	 puts "Secure mode: $MFW_SECURITY_MODE"
	 puts "Secure mode IO: $MFW_SECURITY_MODE_IO"
	 puts "PF id addr     drv_msg    drv_param  mfw_rsp    mfw_param    union(0)  union(1) "
	 set my_format "%2d %2d 0x%06x 0x%08x 0x%08x 0x%08x 0x%08x 0x%08x 0x%08x"
	 set base_addr [expr ([reg read 0x8c20] | 0xe00000) + 8*4 ]
	 set fw_ver [reg read [expr ([reg read 0x8c20] | 0xe00000) +  8*4 + 56*16 + 36*16 + 6*4]]
	 puts "MFW ver: [format '0x%x' $fw_ver]"
	 for {set i 0} {$i < 16} {incr i} {
		 if {$is_ah} {
			 set shmem_func $i
		 } else {
			 set shmem_func [expr ($i & 1) * 8 + ($i>>1)]
		 }
		 set addr [expr $base_addr + $shmem_func*56] 

		 set drv_msg [ reg read [expr $addr + 0]]
		 set drv_param [ reg read [expr $addr + 4]]
		 set mfw_rsp [ reg read [expr $addr + 8]]
		 set mfw_param [ reg read [expr $addr + 12]]
		 set union(0) [ reg read [expr $addr + 24]]
		 set union(1) [ reg read [expr $addr + 28]]
		 puts "[format $my_format $i $shmem_func $addr $drv_msg $drv_param $mfw_rsp $mfw_param $union(0) $union(1)]"  
	 }
}

proc dump_mfw_mb {} {
	 set chip_num [reg_read -n MISCS_REG_CHIP_NUM]
	 if {[expr {$chip_num & 0xff00}] == 0x8000} {
		 set is_ah 1
	 } else {
		 set is_ah 0
	 }
        puts "Event                       PF0     PF1     PF2     PF3     PF4     PF5     PF6     PF7     PF8     PF9     PF10    PF11    PF12    PF13    PF14    PF15"
	 set events {LINK_CHANGE            
			  FLR_FW_ACK_FAILED     
			  VF_DISABLED          
			  LLDP_DATA_UPDATED
			  DCBX_REMOTE_MIB_UPDATED
			  DCBX_OPERATIONAL_MIB_UPDATE
			  ERROR_RECOVERY            
			  BW_UPDATE                 
			  S_TAG_UPDATE
			  GET_LAN_STATS             
			  GET_FCOE_STATS            
			  GET_ISCSI_STATS
			  GET_RDMA_STATS            
			  FAILURE_DETECTED         
			  TRANSCEIVER_STATE_CHANGE 
			  CRITICAL_ERROR_OCCURRED  
			  EEE_NEGOTIATION_COMPLETE  
			  GET_TLV_REQ               
			  OEM_CFG_UPDATE            
			  LLDP_RECEIVED_TLVS_UPDATED
			  GENERIC_IDC
			  XCVR_TX_FAULT             
			  XCVR_RX_LOS               
			  GET_FCOE_CAP              
			  GEN_MDUMP2          }      

	 set my_format "%-27s %-7s %-7s %-7s %-7s %-7s %-7s %-7s %-7s %-7s %-7s %-7s %-7s %-7s %-7s %-7s %-7s"
        set offsize [reg read [expr ([reg read 0x8c20] | 0xe20000) + 2*4 ]]
        set base_addr [expr (($offsize & 0xffff) << 2) | 0xe20000]
	 set base_size [expr (($offsize >> 16) << 2)]
        set sup_msg [reg read $base_addr]
        
        for {set i 0} {$i < 16} {incr i} {
       	 if {$is_ah} {
			 set shmem_func $i
		 } else {
			 set shmem_func [expr ($i & 1) * 8 + ($i>>1)]
		 }
		 set addr [expr $base_addr + $shmem_func*$base_size] 
       	
       	 for {set m 0} {$m < [expr ($sup_msg + 3) / 4]} {incr m} {
       		 set temp_msg [reg read [expr $addr + 4 + $i*4]]
			 set temp_ack [reg read [expr $addr + 4 + [expr ($sup_msg + 3) / 4] + $i*4]]
       		 #lappend msg([expr ($m * 4) + 0]) [list [expr ($temp_msg >> 0) & 0xff] [expr ($temp_ack >> 0) & 0xff]]
			 lappend msg([expr ($m * 4) + 0]) "[expr ($temp_msg >> 0) & 0xff]/[expr ($temp_ack >> 0) & 0xff]"
			 lappend msg([expr ($m * 4) + 1]) "[expr ($temp_msg >> 8) & 0xff]/[expr ($temp_ack >> 8) & 0xff]"
			 lappend msg([expr ($m * 4) + 2]) "[expr ($temp_msg >> 16) & 0xff]/[expr ($temp_ack >> 16) & 0xff]"
			 lappend msg([expr ($m * 4) + 3]) "[expr ($temp_msg >> 24) & 0xff]/[expr ($temp_ack >> 24) & 0xff]"
		 }
	 }
	 
        for {set i 0} {$i < $sup_msg} {incr i} {
		 puts "[format $my_format [lindex $events $i] [lindex $msg($i) 0] [lindex $msg($i) 1] [lindex $msg($i) 2] [lindex $msg($i) 3] [lindex $msg($i) 4] [lindex $msg($i) 5] [lindex $msg($i) 6] [lindex $msg($i) 7] [lindex $msg($i) 8] [lindex $msg($i) 9] [lindex $msg($i) 10] [lindex $msg($i) 11] [lindex $msg($i) 12] [lindex $msg($i) 13] [lindex $msg($i) 14] [lindex $msg($i) 15]]"

       	 #puts "[lindex $events $i] [lindex $msg($i) 0]"
	 }
	 
}

