proc dump_pim {} {

      set chip_num [reg_read -n MISCS_REG_CHIP_NUM]
      if {$chip_num == 0x8170} {
              puts "E5"
              set num_pages 64
              set page_size 2048
              set last_page_page_idx_mask 0x3f
              set last_page_page_offset_mask 0x3ff
              set last_page_page_offset_shift 8
      } else {
              set num_pages 16
              set page_size 4096
              set last_page_page_idx_mask 0x3c
              set last_page_page_offset_mask 0x1ff
              set last_page_page_offset_shift 6
      }

       set pim_base [reg_read -n MCP_REG_CACHE_PIM_NVRAM_BASE]
       set reg_status_base [reg_offset -n MCP_REG_CACHE_CACHE_CTRL_STATUS_0]
       if {$pim_base > 0x140000} {
               set image_base [expr $pim_base - 0x140000]
       } else {
               set image_base [expr $pim_base - 0x4000]
       }
       puts "Page#  Lock  Active  Valid  page  Offset"
       for {set i 0} {$i < $num_pages} {incr i} {
               set cache [reg read [expr $reg_status_base + ($i*4)]]
               set lock [expr $cache & 1]
               set active [expr ($cache >> 1) & 1 ]
               set valid [expr ($cache >> 2) & 1 ]
               set nvram_page_offset [expr ($cache >> 3) & 0x1ff ]
               set global_offset [format "0x%x" [expr ($page_size*$nvram_page_offset)] ]
               set fmt "%2d     %1d     %1d       %1d      %3d   0x%08x"
               puts "[format $fmt $i $lock $active $valid $nvram_page_offset $global_offset]"
       }
       set miss [reg_read -n MCP_REG_CACHE_STAT_MISS_COUNTER]
       set hit [reg_read -n MCP_REG_CACHE_STAT_HIT_COUNTER]

       set last0 [reg_read -n MCP_REG_CACHE_LAST_PAGE_0]
       set last1 [reg_read -n MCP_REG_CACHE_LAST_PAGE_1]

       set status [reg_read -n MCP_REG_CACHE_CACHE_ERROR_STATUS]

       puts "Last two pages:"
       puts "Page#  Valid   is_last   page_index  page_offset"
       set page_index0 [expr ($last0 & $last_page_page_idx_mask) >> 2]
       set page_index1 [expr ($last1 & $last_page_page_idx_mask) >> 2]
       puts "0       [expr $last0 & 1]      [expr ($last0 & 2) >> 1]         $page_index0           [expr ($last0 >> $last_page_page_offset_shift) & $last_page_page_offset_mask]"
       puts "1       [expr $last1 & 1]      [expr ($last1 & 2) >> 1]         $page_index1           [expr ($last1 >> $last_page_page_offset_shift) & $last_page_page_offset_mask]"
       puts "miss: $miss, hit: $hit, status: $status"
       set pc [reg_read -n MCP_REG_CPU_PROGRAM_COUNTER]
       puts "PC: $pc"
       return $status
}

proc catch_pim_error {count} {
        for {set i 0} {$i < $count} {incr i} {
               if {[pim_dump]} {
                       return
               }
               mcp step
        }
}

proc count_me {} {
        for {set i 0} {$i < 100} {incr i} {

                set status [reg_read -n MCP_REG_CACHE_CACHE_ERROR_STATUS]
                if {$status} {
                        puts "found $i $pc"
                        break;
                }
                set pc [reg_read -n MCP_REG_CPU_PROGRAM_COUNTER]
        }
}

proc pim_dump_iter {count} {
       set pim_base [reg read 0xe06300]

       if {$pim_base > 0x140000} {
               set image_base [expr $pim_base - 0x140000]
       } else {
               set image_base [expr $pim_base - 0x4000]
       }
       puts "Page#  Lock  Active  Valid  page  Offset"

       for {set cnt 0} {$cnt < $count} {incr cnt} {
               
               for {set i 0} {$i < 16} {incr i} {
                       set cache($cnt, $i) [reg read [expr 0xe0630c + ($i*4)]]
                       set lock($cnt, $i) [expr $cache & 1]
                       set active($cnt, $i) [expr ($cache >> 1) & 1 ]
                       set valid($cnt, $i) [expr ($cache >> 2) & 1 ]
                       set nvram_page_offset($cnt, $i) [expr ($cache >> 3) & 0x1ff ]
                       set global_offset($cnt, $i) [format "0x%x" [expr (4096*$nvram_page_offset)] ]
                       set fmt "%2d     %1d     %1d       %1d      %3d   0x%08x"
               }
               set miss($cnt) [reg_read -n MCP_REG_CACHE_STAT_MISS_COUNTER]
               set hit($cnt) [reg_read -n MCP_REG_CACHE_STAT_HIT_COUNTER]

               set last0($cnt) [reg_read -n MCP_REG_CACHE_LAST_PAGE_0]
               set last1($cnt) [reg_read -n MCP_REG_CACHE_LAST_PAGE_1]

               set status($cnt) [reg_read -n MCP_REG_CACHE_CACHE_ERROR_STATUS]

               set pc($cnt) [reg_read -n MCP_REG_CPU_PROGRAM_COUNTER]

               if {$status($cnt)} {
                       break
               }
       }
}

