
proc mtip_cfg_pcs_10g {} {

        # MTIP PCS settings (10G) 
        for {set lane 0} {$lane < 4} {incr lane} {
       	 set hs_bar [expr 0x802400 + $lane*32*4]

       	 # ETH_PCS10_50G_REG_CONTROL1
       	 reg write [expr $hs_bar + 0] 0

       	 # ETH_PCS10_50G_REG_VENDOR_PCS_MODE
       	 reg write [expr $hs_bar + 0x20040] 3

		 # ETH_PCS10_50G_REG_CONTROL2
		 reg write [expr $hs_bar + 0x1c] 0

		 # ETH_PCS10_50G_REG_VENDOR_RXLAUI_CONFIG
		 reg write [expr $hs_bar + 0x20010] 0
	 }
}

proc mtip_cfg_mac {} {
	 ## MTIP MAC
        for {set lane 0} {$lane < 4} {incr lane} {
       	 set mac_base [expr 0x800400 + $lane * 256 * 4]

       	 # Maintain PHY_TXENA, since its per SErdes lane and not PCS lane. 
		 set command_config [[reg read [expr $mac_base + 8]] & (1<<15)]

       	 set command_config [expr $command_config | 0x2853 ]

		 # ETH_MAC_REG_XIF_MODE
       	 reg write [expr $mac_base + 0x80] 1

       	 # ETH_MAC_REG_FRM_LENGTH, 9036
       	 reg write [expr $mac_base + 0x14] 9036

		 # ETH_MAC_REG_CL01_PAUSE_QUANTA
       	 reg write [expr $mac_base + 0x54] 0xffffffff
		 # ETH_MAC_REG_CL23_PAUSE_QUANTA, 0xffffffff);
		 reg write [expr $mac_base + 0x58] 0xffffffff
		 # ETH_MAC_REG_CL45_PAUSE_QUANTA, 0xffffffff);
		 reg write [expr $mac_base + 0x5c] 0xffffffff
		 # ETH_MAC_REG_CL67_PAUSE_QUANTA, 0xffffffff);
		 reg write [expr $mac_base + 0x60] 0xffffffff
		 # ETH_MAC_REG_TX_FIFO_SECTIONS
       	 reg write [expr $mac_base + 0x20] 0xe0005
		 # ETH_MAC_REG_COMMAND_CONFIG, command_config);
       	 reg write [expr $mac_base + 8] $command_config

		 # PHY_TXENA - In case of swapping, need to use the serdes lane...
		 reg write [expr $mac_base + 8] [expr $command_config | (1<<15)]
	 }
}

proc ql_cfg_cnig {port speed} {

        set cnig_cfg_addr [expr 0x218200 + $port * 4]

        # Check if there's swap 
        set cnig_cfg_data [expr 1 | ($port << 1) | (3<<6) ]

        if {$speed == 1000 || $speed = 10000} {
       	 set cnig_cfg_data [expr $cnig_cfg_data | (0 << 3)]
	 } else {
       	 if {$speed == 25000} {
			 set cnig_cfg_data [expr $cnig_cfg_data | (1 << 3)]
		 } else {
       		 if {$speed == 40000} {
				 set cnig_cfg_data [expr $cnig_cfg_data | (2 << 3)]
			 } else {
				 if {$speed == 50000} {
					 set cnig_cfg_data [expr $cnig_cfg_data | (3 << 3)]
				 }  else {
       			 	 puts "Invalid speed $speed"
				 }
			 }
		 }
	 }
        reg write $cnig_cfg_addr $cnig_cfg_data

        puts "CNIG was programmed for port $port speed $speed"
}

proc set_10g {} {
        # NWS_REG_ANEG_CFG
	 reg_write -n NWS_REG_ANEG_CFG -v 0x0

	 # NWS_REG_PHY_CTRL
	 reg_write -n NWS_REG_PHY_CTRL -v 0x11869

        # NWS_REG_LNx_CNTL
	 reg_write -n NWS_REG_LN0_CNTL  -v 0x5
	 reg_write -n NWS_REG_LN1_CNTL  -v 0x5
	 reg_write -n NWS_REG_LN2_CNTL  -v 0x5
	 reg_write -n NWS_REG_LN3_CNTL  -v 0x5

        # Common control
	 # Set CMU0 data rate to 10.3125, CMU1 data rate to 25.78125, Ref clock to 257.8125Mhz
        set val [reg_read -n NWS_REG_COMMON_CONTROL]

        # Bring all lanes to active state.
        set val [expr [$val & ~(3<<22 | 3<<24 | 3<<26 | 3<<28)]]
	 reg_write -n NWS_REG_COMMON_CONTROL -v $val

        # Remove CMU resets and powerdown
        set val [expr ($val & ~(0xf<<18)) | (3<<12)]
	 reg_write -n NWS_REG_COMMON_CONTROL -v $val

        # Wait 1 us to make sure that CMU/Lane resets are removed before POR de-assertion
        after 3

	 # Remove POR from serdes
        set val [expr $val | (1<<11)]
	 reg_write -n NWS_REG_COMMON_CONTROL -v $val

        set lanes_ok 0
        # check if lane_ok and lane_rx_valid are asserted
        for {set timeout_ms 0; $timeout_ms < 1000; incr timeout_ms} {
       	 set lane0_sts [reg_read -n NWS_REG_LN0_STATUS]
		 set lane1_sts [reg_read -n NWS_REG_LN1_STATUS]
		 set lane2_sts [reg_read -n NWS_REG_LN2_STATUS]
		 set lane3_sts [reg_read -n NWS_REG_LN3_STATUS]
       	 if {($lane0_sts & $lane1_sts & lane2_sts & lane3_sts & 3) == 3} {
       		 puts "lanes are ok, and lane rx valid"
       		 set lanes_ok 1
       		 break;
		 }
       	 after 1
	 }
        if {$lanes_ok == 0} {
       	 puts "Failed to get lanes ok. Aborting"
       	 return -1;
	 }

	 mtip_cfg_pcs_10g

        # !!! Assuming no lane swapping !!!
	 mtip_cfg_mac

        ql_cfg_cnig 0 10000
	 ql_cfg_cnig 1 10000
	 ql_cfg_cnig 2 10000
	 ql_cfg_cnig 3 10000
}

proc startup_seq {{mode "10G"}} {
	 set NWM_REG_PCS_HS0 0x840000

        set hs_pcs_mode_val 3
	 set rx_xlaui_val 0
	 set hs_pcs_ctrl2_val 0
        set hs_pcs_ctrl1_val 0
        if {$mode eq "40G"} {
       	 set hs_pcs_mode_val 0
       	 set hs_pcs_ctrl2_val 4
	 } else {if {$mode eq "50G"} {
		 set hs_pcs_mode_val 0
		 set hs_pcs_ctrl2_val 4
		 set rx_xlaui_val 0xd81
	 }}
        # MISCS_REG_RESET_PL_HV_2
        reg write 0x9150 0 
        reg w

	 set val [reg_read -n NWS_REG_COMMON_CONTROL]

        set val [expr $val & ~(0x7f << 11)]

        reg_write -n NWS_REG_COMMON_CONTROL -v $val

        reg write 0x9068 [expr 1<<12]

        ## MTIP ## 

	 # ETH_PCS10_50G_REG_CONTROL1
        reg write $NWM_REG_PCS_HS0 $hs_pcs_ctrl1_val
	 # ETH_PCS10_50G_REG_CONTROL2
        reg write 0x84001c $hs_pcs_ctrl2_val
	 # ETH_PCS10_50G_REG_VENDOR_PCS_MODE
	 reg write 0x860040 $hs_pcs_mode_val
	 # ETH_PCS10_50G_REG_VENDOR_RXLAUI_CONFIG
	 reg write 0x860010 $rx_xlaui_val
	 # ETH_RSFEC_REG_RS_FEC_VENDOR_CONTROL
        reg write 0x801600 0
}

proc yes_no {val} {
        if {$val != 0} {
       	 return "Yes"
	 }
        return "No"
}

proc phy_dump {} {
        set common_reg_list {
       	 {0x9150   "MISCS_REG_RESET_PL_HV_2"           }
		 {0x9060   "MISCS_REG_RESET_PL_HV"             }
       	 {0x700000 "NWS_REG_COMMON_CONTROL"            }
       	 {0x700004 "NWS_REG_PHY_CTRL"                  }
		 {0x700008 "NWS_REG_ANEG_CFG"                  }
		 {0x70000c "NWS_REG_COMMON_STATUS"             }
		 {0x700050 "NWS_REG_EXTERNAL_SIGNAL_DETECT"    }
		 {0x700054 "NWS_REG_EXTERNAL_LINK_ALARM_STATUS"}
		 {0x800048 "NWM_REG_PCS_SELECT:L3L2L1L0"       }
        }

        set port_reg_list {
		 {0x700010 "NWS_REG_LN_CTRL"                        0x10}
		 {0x700014 "NWS_REG_LN_STATUS"                      0x10}
		 {0x700018 "NWS_REG_LN_AN_LINK_INPUTS"              0x10}
		 {0x70001c "NWS_REG_LN_AN_LINK_OUTPUTS"             0x10}
         	 
		 {0x840000 "ETH_PCS10_50G_REG_CONTROL1"             0x40000}
		 {0x840004 "ETH_PCS10_50G_REG_STATUS1"              0x40000}
		 {0x840010 "ETH_PCS10_50G_REG_SPEED_ABILITY"        0x40000}
       	 {0x84001c "ETH_PCS10_50G_REG_CONTROL2"             0x40000}
		 {0x840020 "ETH_PCS10_50G_REG_STATUS2"              0x40000}
		 {0x860010 "ETH_PCS10_50G_REG_VENDOR_RXLAUI_CFG"    0x40000}
		 {0x860040 "ETH_PCS10_50G_REG_VENDOR_PCS_MODE"      0x40000}
         {0x802400 "ETH_PCS1G_REG_CONTROL"                  0x80}


		 {0x801600 "ETH_RSFEC_REG_RS_FEC_VENDOR_CONTROL"    0x400}
		 {0x800480 "ETH_MAC_REG_XIF_MODE"                   0x400}
		 {0x800414 "ETH_MAC_REG_FRM_LENGTH"                 0x400}
		 {0x800444 "ETH_MAC_REG_TX_IPG_LENGTH"              0x400}
		 {0x800408 "ETH_MAC_REG_COMMAND_CONFIG"             0x400}
	 }
        
	 set line [format "%-35s %-15s %s" "Common Register Name" "Addr" "Value"]
        puts "$line"
	 set line [format "%-35s %-15s %s" "--------------------" "----" "-----"]
        puts "$line"
	 for {set idx 0} {$idx < [llength $common_reg_list]} {incr idx} {
		 set item [lindex $common_reg_list $idx]
       	 set addr [lindex $item 0]
		 set name [lindex $item 1]
       	 set val [reg read $addr]
       	 set line [format "%-35s %-15s 0x%x" $name ($addr) $val]
       	 puts "$line"
	 }

        puts ""
        set offset -10s
	 set line [format "%-35s %$offset %$offset %$offset %$offset %$offset" "Register name" "Addr" "Port0" "Port1" "Port2" "Port3"]
        puts "$line"
	 set line [format "%-35s %$offset %$offset %$offset %$offset %$offset" "-------------" "----------" "----------" "----------" "----------" "----------"]
        puts "$line"
	 for {set idx 0} {$idx < [llength $port_reg_list]} {incr idx} {
		 set item [lindex $port_reg_list $idx]
       	 set addr [lindex $item 0]
		 set name [lindex $item 1]
       	 set jump [lindex $item 2]
       	 for {set port 0} {$port < 4} {incr port} {
       		 set val_arr($port) [reg read [expr $addr + ($port*$jump)]]
		 }
		 set line [format "%-35s (%s) %$offset %$offset %$offset %$offset" $name $addr $val_arr(0) $val_arr(1) $val_arr(2) $val_arr(3)]
		 puts "$line"
	 }

        puts "LinkDump"
        for {set lane 0} {$lane < 4} {incr lane} {
         set lane_status($lane) [reg read [expr 0x700014 + $lane*0x10]]
         set stec_link_up($lane) [yes_no [expr $lane_status($lane) & 1]]
                 set stec_rx_valid($lane) [yes_no [expr $lane_status($lane) & 2]]
         set mtip_link($lane) [yes_no [expr [reg read [expr 0x840004 + 0x40000*$lane]] & (1<<2)]]
	 set mtip_1g_link($lane) [yes_no [expr [reg read [expr 0x802404 + 0x80*$lane]] & (1<<2)]]
         }

         set line [format "%-40s       %$offset %$offset %$offset %$offset" "Serdes Link Up"  $stec_link_up(0) $stec_link_up(1) $stec_link_up(2) $stec_link_up(3)]
         puts "$line"
         set line [format "%-40s       %$offset %$offset %$offset %$offset" "Serdes RX Valid" $stec_rx_valid(0) $stec_rx_valid(1) $stec_rx_valid(2) $stec_rx_valid(3)]
         puts "$line"
         set line [format "%-40s       %$offset %$offset %$offset %$offset" "10G-50G PCS link"  $mtip_link(0) $mtip_link(1) $mtip_link(2) $mtip_link(3)]
	puts "$line"
         set line [format "%-40s       %$offset %$offset %$offset %$offset" "1G PCS link"  $mtip_1g_link(0) $mtip_1g_link(1) $mtip_1g_link(2) $mtip_1g_link(3)]
         puts "$line"
}


