set help_info "Display / Set gpio\n"
append help_info "Usage: gpio read \[num\]\n"
append help_info "       Display gpios I/O value/s\n"
append help_info "Usage: gpio write <num> <val>\n"
append help_info "       Set specific gpio output to 1 or 0\n"
append help_info "       <num> - 0..31\n"
append help_info "       <val> - 0..1\n"
add_help gpio $help_info

proc gpio {args} { 
    set gpio_in_reg [reg read 0x9480]
    set gpio_val_reg [reg read 0x9584]
  
    set indent 1s
    set lfmt "format \"\%-$indent \%-$indent \%-$indent\ \%-$indent \%-$indent\ \%-$indent \%-$indent \%-$indent\ \%-$indent \%-$indent\ \%-$indent \%-$indent \%-$indent\ \%-$indent \%-$indent\ \%-$indent \%-$indent \%-$indent\ \%-$indent \%-$indent\ \%-$indent \%-$indent \%-$indent\ \%-$indent \%-$indent\ \%-$indent \%-$indent \%-$indent\ \%-$indent \%-$indent\ \%-$indent \%-$indent\""
    set number_h $lfmt
    set number_l $lfmt
    set oe $lfmt
    set io $lfmt
    set cmd [lindex $args 0]

    if {$cmd == "read"} {
        for {set gpio_idx 0} {$gpio_idx < 32} {incr gpio_idx} {
            set o_val [reg read [expr 0x9484 + $gpio_idx*4]]
            set gpio_in($gpio_idx) [expr ($gpio_in_reg >> $gpio_idx) & 1]
            set gpio_i($gpio_idx) [expr ($gpio_val_reg >> $gpio_idx) & 1]
            if {$o_val == 1} {
                set gpio_o($gpio_idx) "1"
            } elseif {$o_val == 2} {
                set gpio_o($gpio_idx) "0"
            } elseif {$o_val == 2} {
                set gpio_o($gpio_idx) "?"
            }
            if {$gpio_in($gpio_idx)} {
                set gpio_f($gpio_idx) "I"
                set gpio_val($gpio_idx) $gpio_i($gpio_idx)
            } else {
                set gpio_f($gpio_idx) "O"
                set gpio_val($gpio_idx) $gpio_o($gpio_idx)
            }
            set number_l "$number_l [expr $gpio_idx % 10]"
            set oe "$oe $gpio_f($gpio_idx)"
            set io "$io $gpio_val($gpio_idx)"
        }
        set gpio_num [lindex $args 1]
        if {$gpio_num != ""} {
            if {$gpio_num > 31} {
                puts "gpio num not in range 0..31. Use \"help gpio\"."
                return
            }
            puts "$gpio_f($gpio_num)"
            return $gpio_val($gpio_num)
        } else {
            puts "gpios:"
            puts "0                   1                   2                   3"
            puts "[eval $number_l]"
            puts "[eval $oe]"
            puts "[eval $io]"
        }
    } elseif {$cmd == "write"} {
        set gpio_num [lindex $args 1]
        set gpio_val [lindex $args 2]
        if {[llength $args] < 3 } {
            puts "Not enough arguments. Use \"help gpio\"."
            return
        }
        if {$gpio_num > 31} {
            puts "gpio num not in range 0..31. Use \"help gpio\"."
            return
        }
        if {$gpio_val > 1} {
            puts "gpio val not in range 0..1. Use \"help gpio\"."
            return
        }
        reg write 0x9480 [expr [reg read 0x9480] & ~(1<<$gpio_num)]
        reg write [expr 0x9484  + (4*$gpio_num)] [expr 1<<($gpio_val^1)]
    #   puts "Setting gpio$gpio_num output val to $gpio_val"
    } else {
        puts "Invalid command $cmd. Use \"help gpio\"."
        return;
    }
}

