##############################################################################################
#  mcpro - MCP Profiler                                                                      #
#                                                                                            #
#Format: mcpro <obj> [max_times 50000] [run_profiler 1] [display_static_func_not_run 0]      #
#	obj  object file. Make sure to  provide the correct file, otherwise youll get       #
#	      bad results.                                                                    #
#	      For example: /usr/qlc/il_proj/bcm5710sw_rel/mcp/579xx/8.20.7.0/b900_obj/b900mim #
#	max_times  How many times to read MCP program counter (Default 50000)                #
#	run_profiler  To run mcp profiler (the output below)                                 #
#	display_static_func_not_run  Displays which functions are located in STATIC area, but#
#	                              didnt run (Usually NVM functions in case no nvm        #
#	                              operation took place)                                   #
##############################################################################################

proc color {foreground text} {return [exec tput setaf $foreground]$text[exec tput sgr0]}
proc putsc {c text} { puts [color $c $text] }


proc find_func {addr obj} {
	set table [exec nm $obj | sed "s/\(.*\) \(.*\) \(.*\)/\1/" | grep -i " t \[a-zA-Z\]" | sort ]

	set lst [split $table "\n"]
	set prev_func ""
	foreach item $lst {
		set hex [lindex $item 0]
		set cur_addr [expr 0x$hex]
		if {$cur_addr > $addr} {
			return $prev_func
		}
		set prev_func [lindex $item 2]

	}
}

proc mcpro {obj {max_times 50000} {run_profiler 1} {display_static_func_not_run 0}} {
	set table [exec nm $obj | sed "s/\(.*\) \(.*\) \(.*\)/\1/" | grep -i " t \[a-zA-Z\]" | sort ]
	set lst [split $table "\n"]
	set prev_addr 0
	set prev_func none
       puts "Process obj file..."
	foreach item $lst {
		set hex [lindex $item 0]
		set cur_addr [expr 0x$hex]
		set size [format 0x%x [expr $cur_addr - $prev_addr]]
		set func_size($prev_func) $size
		set prev_func [lindex $item 2]
		set prev_addr $cur_addr
	}
	
       # Collect the CPU program counter 
       set prev_per -1
	puts "Read Program Counter..."
	for {set i 0} {$i < $max_times} {incr i} {
		lappend addr_arr [reg_read -n MCP_REG_CPU_PROGRAM_COUNTER]
       	set per [expr $i * 100 / $max_times]
       	if {$per != $prev_per} {
       		#puts "$per $prev_per i=$i"
       		puts -nonewline "."
       		flush stdout
       		set prev_per $per
		}
	}
       puts "\nLook up functions..."

       # Catch the function name based on the Program Counter, and times visited 
       set list_size [llength $addr_arr]
       set disp_count 0
	foreach addr $addr_arr {
		set prev_func ""
		set prev_addr 0
			
		foreach item $lst {
			set hex [lindex $item 0]
			set cur_addr [expr 0x$hex]
			if {$cur_addr > $addr} {
				break
			}
			set prev_func [lindex $item 2]
			set prev_addr $cur_addr
		}
		if {[info exists names($prev_func)] == 0 } {
			set func_count 1;
		} else {
			set func_count [expr [lindex $names($prev_func) 2] + 1]
		}
		set size [format 0x%x [expr $cur_addr - $prev_addr]]
		set names($prev_func) [list $prev_addr $size $func_count]

		incr disp_count
		set per [expr $disp_count * 100 / $list_size]
       	if {$per != $prev_per} {
       		puts -nonewline "."
			flush stdout
       		set prev_per $per
		}
	}	

       # Mark the functions that passed in CPU
       set num_pim_func 0
    
	foreach {func value} [array get names] {
		set percent [expr double(double([lindex $names($func) 2]) * double(100) / double($max_times))]
		#puts "pecent $percent, [lindex $names($func) 2]"
		set percent_val  [format %0.2f $percent]
		if {[lindex $value 0] > 0x8028000} {
			#puts "PIM: $func percent: $percent_val"
			set area "PIM"
			incr num_pim_func
		} else {
			#puts "STA: $func percent: $percent_val"
			set area "STATIC"
		}
		set params [list $percent_val $area $func]
		lappend profiler $params
	}

       puts ""
	if {$run_profiler == 1} {
		set srtprofiler [lsort -real -index 0 $profiler]
		puts "MCP Profiler:"
		puts "Function Name                             Location  Percentage"
		puts "----------------------------------------  --------  ----------"
		foreach item $srtprofiler {
			set percent 	[lindex $item 0]
			set area 	[lindex $item 1]
			set func 	[lindex $item 2]
			#puts "$func $area $percent"
			set line [format "%-40s  %-8s  %s%%" $func $area $percent]
       		if {$area == "PIM"} {
				putsc 1 "$line"
			} else {
       			if {$percent > 5} {
					putsc 3 "$line"
				} else {
					putsc 2 "$line"
				}
			}
			
		}
	}
       if {$display_static_func_not_run == 1} {
		puts "\nFunctions in Static which didn't run:"
		puts "Function Name                             Address   Size "
		puts "----------------------------------------  --------  ----------"
		foreach item $lst {
			set hex [lindex $item 0]
			set cur_addr [expr 0x$hex]
			set func [lindex $item 2]
       		if {$cur_addr > 0x8028000} {
				break;
			}
			if {[info exists names($func)] == 0} {
				set line [format "%-40s  0x%x  0x%x" $func $cur_addr $func_size($func)]
				puts "$line"
				#puts "Func $func \([format 0x%x $cur_addr]\) size:[format 0x%x $func_size($func)] was not used"
			}
		}
	}

       puts "Number of PIM functions ran: $num_pim_func"
	return $num_pim_func
}

