global OFFSIZE_OFFSET_SHIFT


proc offsize_offset {offsize} {
	set OFFSIZE_OFFSET_SHIFT	0		
	set OFFSIZE_OFFSET_MASK 0x0000ffff
	set OFFSIZE_SIZE_SHIFT	16
	set OFFSIZE_SIZE_MASK	0xffff0000
	return [expr (((($offsize & $OFFSIZE_OFFSET_MASK) >> $OFFSIZE_OFFSET_SHIFT) << 2))]
}

proc offsize_size {offsize} {
	set OFFSIZE_OFFSET_SHIFT	0		
	set OFFSIZE_OFFSET_MASK 0x0000ffff
	set OFFSIZE_SIZE_SHIFT	16
	set OFFSIZE_SIZE_MASK	0xffff0000
	return [expr (((($offsize & $OFFSIZE_SIZE_MASK) >> $OFFSIZE_SIZE_SHIFT) << 2))]
}


proc calc_nvm_cfg {func} {
	set DEV_MODE "BIGBEAR"
	set CR0_reg 0x8c80
	set NVM_TYPE_NVM_CFG 10

	global nvm_offset_array
	global nvm_cfg_func_size 
	set board_desc_offset [expr [reg read $CR0_reg]]
	set nvm_cfg_offset [expr $board_desc_offset - 0xe20000]
	
	if {$DEV_MODE == "BIGBEAR"} {
		set pf [expr (($func & 1) * 8) + ($func >> 1)]
	} else {
		set pf $func
	}

	# Get nvm config file address 
	set nvm_cfg_base 0
	set DIR_ADDR_OFFSET 0x34
	set dir_addr [nvm read $DIR_ADDR_OFFSET]
	for {set x [expr $dir_addr + 12]} {$x < [expr $dir_addr + 12 + 16*30]} {incr x 16} {

		set type [expr ([nvm read $x] & 0xFFE00000) >> 21]
		if {$type == $NVM_TYPE_NVM_CFG} {
			set nvm_cfg_base [nvm read [expr $x+4]]
			break;
		}
	}

	if {$nvm_cfg_base == 0} {
		puts "NVM CFG file wasn't found!!! Aborting"
		return -1
	}

	puts "func = $func shmem_func = $pf" 

	# Calculate field offsets in nvram
	set glob_offset [expr [offsize_offset [reg read [expr $board_desc_offset + 0]]] - $nvm_cfg_offset + $nvm_cfg_base]
	#puts "glob_offset: $glob_offset"
	set path_offset [expr [offsize_offset [reg read [expr $board_desc_offset + 4]]] - $nvm_cfg_offset + $nvm_cfg_base]
	#puts "path_offset = $path_offset"
	set port_offset [expr [offsize_offset [reg read [expr $board_desc_offset + 8]]] - $nvm_cfg_offset + $nvm_cfg_base]
	set func_offsize [reg read [expr $board_desc_offset + 12]]
	set func_offset [expr [expr [offsize_offset $func_offsize] + ($pf * [offsize_size $func_offsize])] - $nvm_cfg_offset + $nvm_cfg_base]

	#set nvm_cfg_func_size [offsize_size [reg read [expr $board_desc_offset + 12]]]
	set max_path_offset [expr $glob_offset + 0]
	set max_port_offset [expr $glob_offset + 4]
	set pci_id_offset [expr $func_offset + 0]
	set pci_sub_id_offset [expr $func_offset + 4]
	set pf_initvf_offset [expr $func_offset + 8]
	set initvf_offset [expr $func_offset + 12]
	set vf_offset_offset [expr $func_offset + 16]
 
	array set nvm_offset_array [list \
		{max_path} $max_path_offset \
		{max_port} $max_port_offset \
		{pci_id} $pci_id_offset \
		{pci_sub_id} $pci_sub_id_offset \
		{pf_initvf} $pf_initvf_offset \
		{initvf} $initvf_offset \
		{vf_offset} $vf_offset_offset ]
}

proc existence {variable} {
    upvar $variable testVar
    if { [info exists testVar] } {
	return 1
    } else {
	return 0
    }
}

global mcp_debug_result
proc check_res {status_addr exp_seq} {
	global mcp_debug_result
	set val [reg read $status_addr]
	#puts "Read [hex $val] from addr [hex $status_addr], seq = $exp_seq"
	if {[expr $val & 0xffff] == $exp_seq} {
		set mcp_debug_result [expr $val >> 16]
		return
	}
	set mcp_debug_result -1
}

proc nvm_set {func field val} {
	global mcp_debug_result
	global nvm_offset_array

	calc_nvm_cfg $func
	if {![existence nvm_offset_array($field)]} {
		puts "Error: Format is \"nvm_set <func> <field> <value>\""
		puts "<field>: [array names nvm_offset_array]"
		return;
	}
	puts "$field offset $nvm_offset_array($field)"
	set addr $nvm_offset_array($field)
	puts "Writing $val to field $field at address $addr"
	nvm write $addr $val
}

proc disp_nvm_field {field} {
	global nvm_offset_array
	set addr $nvm_offset_array($field)
	set val [nvm read $addr]
	puts "Read $val from field $field at address $addr"
	return $val
}
proc nvm_get {func { field "" }} {
	global mcp_debug_result
	global nvm_offset_array
	calc_nvm_cfg $func
	
	if {$field == ""} {
		foreach field [array names nvm_offset_array] {
			disp_nvm_field $field
		}
	} else {
		if {![existence nvm_offset_array($field)]} {
			puts "Error: Format is \"nvm_get <func> [field]\""
			puts "<field>: [array names nvm_offset_array]"
			return;
		}
		return [disp_nvm_field $field]
	}
}

proc nvm_set_example {} {
	nvm_set 0 max_path 2
	nvm_set 0 max_port 2
	for {set pf_num 0} {$pf_num < 8} {incr pf_num 2} {
	
		nvm_set $pf_num pci_id 0x14e41629
		nvm_set $pf_num pci_sub_id 0x162914e4

		# pf_initvf - First VF_NUM for PF is encoded in this register (in octects - 0..15 -> 0,8,16,24...)
		nvm_set $pf_num pf_initvf [expr $pf_num /2]

		# initvf - how many VFs for this PF (15-0  INITIALVF 31-16  TOTALVF)
		nvm_set $pf_num initvf 0x00080008

		nvm_set $pf_num vf_offset 0x16300008
	}
}


