###########################################################################
#
# Copyright(c) 2017 Cavium Corporation, all rights reserved
# Proprietary and Confidential Information.
#
# Module Description:
# This TCL script for debugging NCSI Passthrough path
#
# Written by Boaz Bar-On, QLogic, sep 2017
#
###########################################################################

# Specify any dependency on other packages
package require Tcl


proc usage {} {
   puts  "Get passthrough information\n"
   puts  "==== script options ===="
   puts  "-dump:    To get all the passtrough registers"
   puts  "-stat:    To get all the passtrough packets statistics"
   puts  "-btb:     To get all the  BTB packets statistics"
   puts  "-cache:   To get ncsi cache flags"
   puts  "-conf \[ch\] \[bmc_mac_addr\] \[path: nw2bmc|os2bmc\]: To set passthrough path"
   puts "Note: If no path is mentioned, both paths are configured."
}

proc passth_help {} {
   set help_info     "Get passthrough information\n"
   append help_info  "==== script options ====\n"
   append help_info  "-dump:    To get all the passtrough registers\n"
   append help_info  "-stat:    To get all the passtrough packets statistic\n"
   append help_info  "-btb: To get all the  BTB packets statistic\n"
   append help_info  "-cache:   To get ncsi cache flags\n"
   append help_info  "-conf \[ch\] \[bmc_mac_addr\] \[path: nw2bmc|os2bmc\]: To set passthrough path.\n"
   append help_info  "Note: If no path is mentioned, both paths are configured.\n"
   add_help passth $help_info
}

proc filter_feilds {filter} {
   append bcast_type "    " "ARP =  [expr  ($filter >> 13) & 0x1]," ;#Mask bit for forwarding packets with Ethertype of 0x0806 and Bcast address to MCP. */
   append bcast_type "" " DHCP =  [expr  ($filter >> 28) & 0x1],"   ;#Mask bit for forwarding packets with DHCP UDP/TCP ports 67/68/546/547/647/847 to MCP. */
   append bcast_type "" " NTBS =  [expr  ($filter >> 20) & 0x1]"    ;#Mask bit for forwarding packets with NetBIOS TCP destination port 137/138/139 to MCP
   puts "$bcast_type\n"
}

proc filters {} {
   puts "====== BC and MC filters ======"
   puts "-->  nw2bmc " 
   puts "NIG_REG_RX_LLH_NCSI_MCP_MASK     =  [reg_read -n NIG_REG_RX_LLH_NCSI_MCP_MASK]"
   filter_feilds [reg_read -n NIG_REG_RX_LLH_NCSI_MCP_MASK]
   puts "NIG_REG_RX_LLH_NCSI_MCP_MASK_2   =  [reg_read -n NIG_REG_RX_LLH_NCSI_MCP_MASK_2]\n"
   
   puts "-->  os2bmc" 
   puts "NIG_REG_TX_LLH_NCSI_MCP_MASK     =  [reg_read -n NIG_REG_TX_LLH_NCSI_MCP_MASK]"
   filter_feilds [reg_read -n NIG_REG_TX_LLH_NCSI_MCP_MASK]
   puts "NIG_REG_TX_LLH_NCSI_MCP_MASK_2   =  [reg_read -n NIG_REG_TX_LLH_NCSI_MCP_MASK_2]\n"
   
   puts "NIG_REG_TX_LLH_NCSI_NTWK_MASK    =  [reg_read -n NIG_REG_TX_LLH_NCSI_NTWK_MASK]"
   filter_feilds [reg_read -n NIG_REG_TX_LLH_NCSI_NTWK_MASK]
   puts "NIG_REG_TX_LLH_NCSI_NTWK_MASK_2  =  [reg_read -n NIG_REG_TX_LLH_NCSI_NTWK_MASK_2]\n"
}

proc vlan {} {
   puts "====== VLAN ======"
   puts "NIG_REG_RX_LLH_NCSI_MCP_MASK_IVLAN =  [reg_read -n NIG_REG_RX_LLH_NCSI_MCP_MASK_IVLAN]"
   puts "NIG_REG_TX_LLH_NCSI_MCP_MASK_IVLAN =  [reg_read -n NIG_REG_TX_LLH_NCSI_MCP_MASK_IVLAN]"
   puts "NIG_REG_TX_LLH_NCSI_NTWK_MASK_IVLAN =  [reg_read -n NIG_REG_TX_LLH_NCSI_NTWK_MASK_IVLAN]"
   puts "vlan_id0 =  [reg_read -n NIG_REG_LLH_VLAN_ID_0]"
   puts "vlan_id1 =  [reg_read -n NIG_REG_LLH_VLAN_ID_1]"
   puts "vlan_id2 =  [reg_read -n NIG_REG_LLH_VLAN_ID_2]\n"
}

proc mac_addr {} {
   puts "====== MAC ADDRESS ======"
   set num_ports 4

   set NCSI_REG_BMC_MAC_ADDR_LO_0   [reg_read -n NCSI_REG_BMC_MAC_ADDR_LO_0]
   set NCSI_REG_BMC_MAC_ADDR_HI_0   [reg_read -n NCSI_REG_BMC_MAC_ADDR_HI_0]
   set NCSI_REG_BMC_MAC_ADDR_LO_1   [reg_read -n NCSI_REG_BMC_MAC_ADDR_LO_1]
   set NCSI_REG_BMC_MAC_ADDR_HI_1   [reg_read -n NCSI_REG_BMC_MAC_ADDR_HI_1]
   set NCSI_REG_BMC_MAC_ADDR_LO_2   [reg_read -n NCSI_REG_BMC_MAC_ADDR_LO_2]
   set NCSI_REG_BMC_MAC_ADDR_HI_2   [reg_read -n NCSI_REG_BMC_MAC_ADDR_HI_2]
   set NCSI_REG_BMC_MAC_ADDR_LO_3   [reg_read -n NCSI_REG_BMC_MAC_ADDR_LO_3]
   set NCSI_REG_BMC_MAC_ADDR_HI_3   [reg_read -n NCSI_REG_BMC_MAC_ADDR_HI_3]

   puts "NCSI_REG_BMC_MAC_ADDR_LO_0  =  $NCSI_REG_BMC_MAC_ADDR_LO_0"
   puts "NCSI_REG_BMC_MAC_ADDR_HI_0  =  $NCSI_REG_BMC_MAC_ADDR_HI_0"
   puts "NCSI_REG_BMC_MAC_ADDR_LO_1  =  $NCSI_REG_BMC_MAC_ADDR_LO_1"
   puts "NCSI_REG_BMC_MAC_ADDR_HI_1  =  $NCSI_REG_BMC_MAC_ADDR_HI_1"
   puts "NCSI_REG_BMC_MAC_ADDR_LO_2  =  $NCSI_REG_BMC_MAC_ADDR_LO_2"
   puts "NCSI_REG_BMC_MAC_ADDR_HI_2  =  $NCSI_REG_BMC_MAC_ADDR_HI_2"
   puts "NCSI_REG_BMC_MAC_ADDR_LO_3  =  $NCSI_REG_BMC_MAC_ADDR_LO_3"
   puts "NCSI_REG_BMC_MAC_ADDR_HI_3  =  $NCSI_REG_BMC_MAC_ADDR_HI_3"
    
   set NCSI_REG_BMC_MAC_VALID_FLAG_0 [reg_read -n NCSI_REG_BMC_MAC_VALID_FLAG_0]
   set NCSI_REG_BMC_MAC_VALID_FLAG_1 [reg_read -n NCSI_REG_BMC_MAC_VALID_FLAG_1]
   set NCSI_REG_BMC_MAC_VALID_FLAG_2 [reg_read -n NCSI_REG_BMC_MAC_VALID_FLAG_2]
   set NCSI_REG_BMC_MAC_VALID_FLAG_3 [reg_read -n NCSI_REG_BMC_MAC_VALID_FLAG_3]
    
   puts "NCSI_REG_BMC_MAC_VALID_FLAG_0 =  $NCSI_REG_BMC_MAC_VALID_FLAG_0"
   puts "NCSI_REG_BMC_MAC_VALID_FLAG_1 =  $NCSI_REG_BMC_MAC_VALID_FLAG_1"
   puts "NCSI_REG_BMC_MAC_VALID_FLAG_2 =  $NCSI_REG_BMC_MAC_VALID_FLAG_2"
   puts "NCSI_REG_BMC_MAC_VALID_FLAG_3 =  $NCSI_REG_BMC_MAC_VALID_FLAG_3"
   puts "\n --> Destination MAC address"
   for {set i 0} {$i < 5} {incr i} {
      set mac_x_0  [reg read [expr  0x501088 + 8 * $i]]
      puts "NIG_REG_LLH_DEST_MAC_${i}_0 =  $mac_x_0"
      set mac_x_1 [reg read [expr  0x50108c + 8 * $i]]
      puts "NIG_REG_LLH_DEST_MAC_${i}_1 =  $mac_x_1"
   }
   puts ""
   puts "====== NCSI CONFIGURATION ======"
   set ncsi_config [reg_read -n NCSI_REG_CONFIG]
   puts "NCSI_REG_CONFIG =  $ncsi_config"
}

proc passth_dump {} {
   filters
   vlan
   mac_addr
}
proc passth_tc {} {
   puts "NW_HOST 2 BMC (TC_0) =  [reg_read -n NIG_REG_STAT_TX_PACKET_TC_0]"
   puts "NW_HOST 2 MCP (TC_1) =  [reg_read -n NIG_REG_STAT_TX_PACKET_TC_1]"
   puts "MCP 2 BMC     (TC_2) =  [reg_read -n NIG_REG_STAT_TX_PACKET_TC_2]"
   puts "MCP 2 NW_HOST (TC_3) =  [reg_read -n NIG_REG_STAT_TX_PACKET_TC_3]"
   puts "BMC 2 MCP     (TC_4) =  [reg_read -n NIG_REG_STAT_TX_PACKET_TC_4]"
   puts "BMC 2 NW_HOST (TC_5) =  [reg_read -n NIG_REG_STAT_TX_PACKET_TC_5]"
   puts " (TC_6) =  [reg_read -n NIG_REG_STAT_TX_PACKET_TC_6]"
   puts " (TC_7) =  [reg_read -n NIG_REG_STAT_TX_PACKET_TC_7]"
}
global PREV_RX_BMB_PKT_ERR
global PREV_RX_BMB_JUMBO_FRAME
global PREV_NIG_REG_STAT_RX_BMB_PACKET

set PREV_RX_BMB_PKT_ERR 0
set PREV_RX_BMB_JUMBO_FRAME 0
set PREV_SENT_TO_BMC_BY_MFW 0
set PREV_HANDLED_BY_MFW 0
for {set i 0} {$i < 4} {incr i} {
    set PREV_NIG_REG_STAT_RX_BMB_PACKET($i) 0       
}
proc set_device {gport} {
   set base_dev [expr $::current(DEV) - $::current(PCI_FUNC)]
   set rest_devices [expr $::current(TOTAL_DEV) - $base_dev + 1]
   
   set new_device [expr $base_dev + $gport]
   #puts "set device $new_device"
   device $new_device -no_display 
   return $rest_devices
}

proc passth_stat {} {
   global PREV_RX_BMB_PKT_ERR
   global PREV_RX_BMB_JUMBO_FRAME
   global PREV_NIG_REG_STAT_RX_BMB_PACKET
   global PREV_SENT_TO_BMC_BY_MFW
   global PREV_HANDLED_BY_MFW

   puts "\n===============       UMAC regs       ==============="
   puts "=====Tx statistic  UMAC <-- NCSI / HOST"  
   puts "MSTAT_REG_TX_STAT_GTPKT_LO (ALL PKTS)     =  [reg_read -n MSTAT_REG_TX_STAT_GTPKT_LO]"
   puts "MSTAT_REG_TX_STAT_GTERR_LO (BAD PKTS)     =  [reg_read -n MSTAT_REG_TX_STAT_GTERR_LO]"
   puts "MSTAT_REG_TX_STAT_GTXPOK_LO (GOOD PKTS)   =  [reg_read -n MSTAT_REG_TX_STAT_GTXPOK_LO]"
   puts "    Unicast packets    =  [reg_read -n MSTAT_REG_TX_STAT_GTUCA_LO]"
   puts "    Multicsat packets  =  [reg_read -n MSTAT_REG_TX_STAT_GTMCA_LO]"
   puts "    Broadcast packets  =  [reg_read -n MSTAT_REG_TX_STAT_GTBCA_LO]"
   
   puts "===== Rx statistic  UMAC --> NCSI / HOST"  
   puts "MSTAT_REG_RX_STAT_GRPKT_LO (ALL PKTS)     =  [reg_read -n MSTAT_REG_RX_STAT_GRPKT_LO]"
   puts "MSTAT_REG_RX_STAT_GRFLR_LO (BAD PKTS)     =  [reg_read -n MSTAT_REG_RX_STAT_GRFLR_LO]"
   puts "MSTAT_REG_RX_STAT_GRPOK_LO (GOOD PKTS)    =  [reg_read -n MSTAT_REG_RX_STAT_GRPOK_LO]"
   puts "    Unicast packets    =  [reg_read -n MSTAT_REG_RX_STAT_GRUCA_LO]"
   puts "    Multicsat packets  =  [reg_read -n MSTAT_REG_RX_STAT_GRMCA_LO]"
   puts "    Broadcast packets  =  [reg_read -n MSTAT_REG_RX_STAT_GRBCA_LO]"
   puts "    Undersized Packets(<64 bytes)     =  [reg_read -n MSTAT_REG_RX_STAT_GRUND_LO]"
   puts "    Jumbo packets     (>1523 bytes)   =  [format 0x%x [ expr [reg_read -n MSTAT_REG_RX_STAT_GR2047_LO] + [reg_read -n MSTAT_REG_RX_STAT_GR4095_LO] + [reg_read -n MSTAT_REG_RX_STAT_GR9216_LO]+ [reg_read -n MSTAT_REG_RX_STAT_GR16383_LO]]]"

   puts "\n===============       NCSI regs       ==============="
   set NCSI_REG_STAT_NUM_PACKETS_TO_NETWORK   [reg_read -n NCSI_REG_STAT_NUM_PACKETS_TO_NETWORK]
   set NCSI_REG_STAT_NUM_PACKETS_TO_HOST      [reg_read -n NCSI_REG_STAT_NUM_PACKETS_TO_HOST]
   set NCSI_REG_STAT_NUM_PACKETS_TO_MCP      [reg_read -n NCSI_REG_STAT_NUM_PACKETS_TO_MCP]
   set NCSI_REG_STAT_PKT_DROP_SA_MISMATCH      [reg_read -n NCSI_REG_STAT_PKT_DROP_SA_MISMATCH]
   set NCSI_REG_STAT_PKT_DROP_BMB_FULL        [reg_read -n NCSI_REG_STAT_PKT_DROP_BMB_FULL]
   puts "NCSI_REG_STAT_NUM_PACKETS_TO_NETWORK      =  $NCSI_REG_STAT_NUM_PACKETS_TO_NETWORK"
   puts "NCSI_REG_STAT_NUM_PACKETS_TO_HOST         =  $NCSI_REG_STAT_NUM_PACKETS_TO_HOST"
   puts "NCSI_REG_STAT_NUM_PACKETS_TO_MCP          =  $NCSI_REG_STAT_NUM_PACKETS_TO_MCP"
   puts "NCSI_REG_STAT_PKT_DROP_SA_MISMATCH        =  $NCSI_REG_STAT_PKT_DROP_SA_MISMATCH"
   puts "NCSI_REG_STAT_PKT_DROP_BMB_FULL           =  $NCSI_REG_STAT_PKT_DROP_BMB_FULL"
    
   puts "\n===============       NIG regs        ==============="
    #Read the statistic of all the ports
   set port_mode [reg_read -n MCP_REG_PORT_MODE]
   if {$port_mode == 2} {
         set num_ports 4
    } else {
            if {$port_mode == 1} {
                    set num_ports 2
            } else {
                    set num_ports 1
            }
    }
    set curr_port_index   [expr {$::current(DEV)}]

	
    puts "=====Tx statistics  NIG --> NW / HOST"
    for {set port_index 0} {$port_index < $num_ports } {incr port_index} {
		set rest_devices [set_device $port_index]
		#puts "BOAZ $rest_devices"
        #set dev   [expr $port_index +1]
        #device $dev
        #Calculate address of port statistics buffers
        set GRCBASE_MCP 0xe20000
        set PUBLIC_PORT 4
        set public_base [expr [reg_read -n MISC_REG_SHARED_MEM_ADDR] | $GRCBASE_MCP]
        set port_offsize [reg read [expr $public_base +  4 + (4 * ($PUBLIC_PORT))]]
        set port_offset [expr ($port_offsize & 0xffff) << 2]
        set port_size [expr (($port_offsize & 0xffff0000) >> 16 ) << 2]
        set PUBLIC_GLOBAL 2
        set glob_offsize [reg read [expr $public_base +  4 + (4 * ($PUBLIC_GLOBAL))]]
        set glob_offset [expr ($glob_offsize & 0xffff) << 2]
        set RX_BMB_PKT_OFFSET 0xc84
        set RX_BMB_PKT_ERR_OFFSET 0x38
        set RX_BMB_JUMBO_FRAME_OFFSET 0x3c
        set SENT_TO_BMC 0x40
        set HANDLED_BY_MFW 0x44
        set bmb_error_status_cnt_addr [ expr $GRCBASE_MCP + $glob_offset + $RX_BMB_PKT_ERR_OFFSET]
        set bmb_jumbo_frame_cnt_addr  [ expr $GRCBASE_MCP + $glob_offset + $RX_BMB_JUMBO_FRAME_OFFSET]
        set sent_to_bmc_cnt_addr      [ expr $GRCBASE_MCP + $glob_offset + $SENT_TO_BMC]
        set handled_by_mfw_cnt_addr  [ expr $GRCBASE_MCP + $glob_offset + $HANDLED_BY_MFW]
        set UPDATED_RX_BMB_PKT_ERR      0
        set UPDATED_RX_BMB_JUMBO_FRAME   0
        set UPDATED_SENT_TO_BMC_BY_MFW   0
        set UPDATED_HANDLED_BY_MFW       0                                                                                      
        append NIG_REG_STAT_TX_H2BMB_PACKET             "  " [format 0x%-4x [reg_read -n NIG_REG_STAT_TX_H2BMB_PACKET]          ]
        append NIG_REG_STAT_TX_H2BMB_PACKET_TRUNCATE    "  " [format 0x%-4x [reg_read -n NIG_REG_STAT_TX_H2BMB_PACKET_TRUNCATE] ]
        append NIG_REG_STAT_TX_H2BMB_PACKET_DISCARD     "  " [format 0x%-4x [reg_read -n NIG_REG_STAT_TX_H2BMB_PACKET_DISCARD]  ]
        append NIG_REG_STAT_TX_BMB_PACKET               "  " [format 0x%-4x [reg_read -n NIG_REG_STAT_TX_BMB_PACKET]            ]
        append NIG_REG_STAT_LB_BMB_PACKET               "  " [format 0x%-4x [reg_read -n NIG_REG_STAT_LB_BMB_PACKET]            ]
    

        #set port_index   [expr {$::current(DEV) - 1}] 
        set nig_reg_stat_rx_bmb_packet_addr [expr $GRCBASE_MCP + $port_offset + $port_size*$port_index + $RX_BMB_PKT_OFFSET]
        set UPDATED_RX_BMB_PKT_ADDR_$port_index   $nig_reg_stat_rx_bmb_packet_addr

        # The following three counters show the diff from the previous read
        set RX_BMB_PKT_ERR                          [reg read $bmb_error_status_cnt_addr]
        set RX_BMB_JUMBO_FRAME                      [reg read $bmb_jumbo_frame_cnt_addr]
        set SENT_TO_BMC_BY_MFW                      [reg read $sent_to_bmc_cnt_addr]
        set HANDLED_BY_MFW                          [reg read $handled_by_mfw_cnt_addr]
        set NIG_REG_STAT_RX_BMB_PACKET($port_index) [reg read $nig_reg_stat_rx_bmb_packet_addr]
        set UPDATED_RX_BMB_PKT_ERR                          [expr $UPDATED_RX_BMB_PKT_ERR    + $RX_BMB_PKT_ERR - $PREV_RX_BMB_PKT_ERR]
        set UPDATED_RX_BMB_JUMBO_FRAME                      [expr $UPDATED_RX_BMB_JUMBO_FRAME+ $RX_BMB_JUMBO_FRAME - $PREV_RX_BMB_JUMBO_FRAME]
        set UPDATED_SENT_TO_BMC_BY_MFW                      [expr $UPDATED_SENT_TO_BMC_BY_MFW+ $SENT_TO_BMC_BY_MFW - $PREV_SENT_TO_BMC_BY_MFW]
        set UPDATED_HANDLED_BY_MFW                          [expr $UPDATED_HANDLED_BY_MFW    + $HANDLED_BY_MFW - $PREV_HANDLED_BY_MFW]
        set UPDATED_NIG_REG_STAT_RX_BMB_PACKET($port_index) [expr $NIG_REG_STAT_RX_BMB_PACKET($port_index) - $PREV_NIG_REG_STAT_RX_BMB_PACKET($port_index)]
        append UPDATED_RX_BMB_PKT_LIST "  " "[format 0x%-4x $UPDATED_NIG_REG_STAT_RX_BMB_PACKET($port_index)]"
        # In case the diff is zero, perhaps its the only MFW, so read directly from NIG register
        if {$UPDATED_NIG_REG_STAT_RX_BMB_PACKET($port_index) == 0} {
                set UPDATED_NIG_REG_STAT_RX_BMB_PACKET($port_index) [reg_read -n NIG_REG_STAT_RX_BMB_PACKET]
        }
        
        append  NIG_REG_STAT_RX_BMB_PACKET_TRUNCATE "  " [format 0x%-4x [reg_read -n NIG_REG_STAT_RX_BMB_PACKET_TRUNCATE]]
        append  NIG_REG_STAT_RX_BMB_PACKET_DISCARD "  " [format 0x%-4x [reg_read -n NIG_REG_STAT_RX_BMB_PACKET_DISCARD]]

        set PREV_RX_BMB_PKT_ERR $RX_BMB_PKT_ERR
        set PREV_RX_BMB_JUMBO_FRAME $RX_BMB_JUMBO_FRAME
        set PREV_SENT_TO_BMC_BY_MFW $SENT_TO_BMC_BY_MFW
        set PREV_HANDLED_BY_MFW $HANDLED_BY_MFW
        set PREV_NIG_REG_STAT_RX_BMB_PACKET($port_index) $NIG_REG_STAT_RX_BMB_PACKET($port_index)
    }
   device $curr_port_index
   #print all the port statistic
   puts "NIG_REG_MNG_TO_MCP                        =  [reg_read -n NIG_REG_MNG_TO_MCP]"
   puts "=====Tx statistics  NIG --> NW / HOST"
   puts "NIG_REG_STAT_TX_H2BMB_PACKET (HOST -> BMB)=$NIG_REG_STAT_TX_H2BMB_PACKET"
   puts "NIG_REG_STAT_TX_H2BMB_PACKET_TRUNCATE     =$NIG_REG_STAT_TX_H2BMB_PACKET_TRUNCATE"
   puts "NIG_REG_STAT_TX_H2BMB_PACKET_DISCARD      =$NIG_REG_STAT_TX_H2BMB_PACKET_DISCARD"
   puts "NIG_REG_STAT_TX_BMB_PACKET (BMB -> NW)    =$NIG_REG_STAT_TX_BMB_PACKET"
   puts "NIG_REG_STAT_LB_BMB_PACKET (BMB -> HOST)  =$NIG_REG_STAT_LB_BMB_PACKET"
   
   puts "\n=====Rx statistics  NIG <-- NW / HOST"
   puts "NIG_REG_STAT_RX_BMB_PACKET([format 0x%x $UPDATED_RX_BMB_PKT_ADDR_0]/[format 0x%x [expr $UPDATED_RX_BMB_PKT_ADDR_1 - $UPDATED_RX_BMB_PKT_ADDR_0]])=$UPDATED_RX_BMB_PKT_LIST"
   puts "NIG_REG_STAT_RX_BMB_PACKET_TRUNCATE       =$NIG_REG_STAT_RX_BMB_PACKET_TRUNCATE"
   puts "NIG_REG_STAT_RX_BMB_PACKET_DISCARD        =$NIG_REG_STAT_RX_BMB_PACKET_DISCARD"
   puts "RX_BMB_PKT_ERR (global [format 0x%x $bmb_error_status_cnt_addr])          =  [format 0x%x $UPDATED_RX_BMB_PKT_ERR]"
   puts "RX_BMB_JUMBO_FRAME (global [format 0x%x $bmb_jumbo_frame_cnt_addr])      =  [format 0x%x $UPDATED_RX_BMB_JUMBO_FRAME]"
   puts "SENT_TO_BMC_BY_MFW                        =  [format 0x%x $UPDATED_SENT_TO_BMC_BY_MFW]"
   puts "HANDLED_BY_MFW                            =  [format 0x%x $UPDATED_HANDLED_BY_MFW]"

   if {[is_e5]} {
      puts "\n===============       MAC regs        ==============="
      set port_index   [expr {$::current(DEV) - 1}]
      if {$port_index == 2} {
         set port_index  1
      }
      if {$port_index == 1} {
         set port_index  2
      }
      puts "=====Tx statistics  mac --> NW"         
      puts "NWM_REG_MAC_MIB_TX_ACTIVE_FRAMESTXD_31_0_I$port_index          (ALL PKTS)    =  [reg_read -n NWM_REG_MAC_MIB_TX_ACTIVE_FRAMESTXD_31_0_I$port_index]"
      puts "NWM_REG_MAC_MIB_TX_ACTIVE_FRAMESTXDBADFCS_31_0_I$port_index    (BAD PKTS)    =  [reg_read -n NWM_REG_MAC_MIB_TX_ACTIVE_FRAMESTXDBADFCS_31_0_I$port_index]"
      puts "NWM_REG_MAC_MIB_TX_ACTIVE_FRAMESTXDOK_31_0_I$port_index        (GOOD PKTS)   =  [reg_read -n NWM_REG_MAC_MIB_TX_ACTIVE_FRAMESTXDOK_31_0_I$port_index]"
      puts "    Unicast packets    =  [reg_read -n NWM_REG_MAC_MIB_TX_ACTIVE_UNICASTFRAMESTXDOK_31_0_I$port_index]"
      puts "    Multicsat packets  =  [reg_read -n NWM_REG_MAC_MIB_TX_ACTIVE_MULTICASTFRAMESTXDOK_31_0_I$port_index]"
      puts "    Broadcast packets  =  [reg_read -n NWM_REG_MAC_MIB_TX_ACTIVE_BROADCASTFRAMESTXDOK_31_0_I$port_index]"

      puts "=====Rx statistics  mac <-- NW"
      puts "NWM_REG_MAC_MIB_RX_ACTIVE_FRAMESRXD_31_0_I$port_index          (ALL PKTS)  =  [reg_read -n NWM_REG_MAC_MIB_RX_ACTIVE_FRAMESRXD_31_0_I$port_index]"
      puts "NWM_REG_MAC_MIB_RX_ACTIVE_FRAMESRXDBADFCS_31_0_I$port_index    (BAD PKTS)  =  [reg_read -n NWM_REG_MAC_MIB_RX_ACTIVE_FRAMESRXDBADFCS_31_0_I$port_index]"
      puts "NWM_REG_MAC_MIB_RX_ACTIVE_FRAMESRXDOK_31_0_I$port_index        (GOOD PKTS) =  [reg_read -n NWM_REG_MAC_MIB_RX_ACTIVE_FRAMESRXDOK_31_0_I$port_index]"
      puts "    Unicast packets    =  [reg_read -n NWM_REG_MAC_MIB_RX_ACTIVE_UNICASTFRAMESRXDOK_31_0_I$port_index]"
      puts "    Multicsat packets  =  [reg_read -n NWM_REG_MAC_MIB_RX_ACTIVE_MULTICASTFRAMESRXDOK_31_0_I$port_index]"
      puts "    Broadcast packets  =  [reg_read -n NWM_REG_MAC_MIB_RX_ACTIVE_BROADCASTFRAMESRXDOK_31_0_I$port_index]"
   }
    
   puts "===============       BMB regs        ==============="
   set BMB_PKTS_TO_BMC_OCCUPANCY_0               [reg_read -n BMB_REG_TC_OCCUPANCY_0]
   set BMB_PKTS_TO_MCP_OCCUPANCY_1               [reg_read -n BMB_REG_TC_OCCUPANCY_1]
   set BMB_PKTS_MCP_to_BMC_OCCUPANCY_2           [reg_read -n BMB_REG_TC_OCCUPANCY_2]
   set BMB_PKTS_FROM_MCP_OCCUPANCY_3             [reg_read -n BMB_REG_TC_OCCUPANCY_3]
   set BMB_PKTS_BMC_to_MCP_OCCUPANCY_4           [reg_read -n BMB_REG_TC_OCCUPANCY_4]
   set BMB_PKTS_FROM_BMC_OCCUPANCY_5             [reg_read -n BMB_REG_TC_OCCUPANCY_5] 
                                                  
   puts "BMB_PKTS_TO_BMC_OCCUPANCY_0               =  $BMB_PKTS_TO_BMC_OCCUPANCY_0     "
   puts "BMB_PKTS_TO_MCP_OCCUPANCY_1               =  $BMB_PKTS_TO_MCP_OCCUPANCY_1     "
   puts "BMB_PKTS_MCP_to_BMC_OCCUPANCY_2           =  $BMB_PKTS_MCP_to_BMC_OCCUPANCY_2 "
   puts "BMB_PKTS_FROM_MCP_OCCUPANCY_3             =  $BMB_PKTS_FROM_MCP_OCCUPANCY_3   "
   puts "BMB_PKTS_BMC_to_MCP_OCCUPANCY_4           =  $BMB_PKTS_BMC_to_MCP_OCCUPANCY_4 "
   puts "BMB_PKTS_FROM_BMC_OCCUPANCY_5             =  $BMB_PKTS_FROM_BMC_OCCUPANCY_5   "
} 
proc passth_cache {} {
      set NCSI_REG_SA_CACHE_VALID_FLAG_0          [reg_read -n NCSI_REG_SA_CACHE_VALID_FLAG_0]
      set NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_0      [reg_read -n NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_0 ]
      puts "NCSI_REG_SA_CACHE_VALID_FLAG_0        =  $NCSI_REG_SA_CACHE_VALID_FLAG_0"
      puts "NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_0    =  $NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_0"

      set NCSI_REG_SA_CACHE_VALID_FLAG_1          [reg_read -n NCSI_REG_SA_CACHE_VALID_FLAG_1]
      set NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_1      [reg_read -n NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_1 ]
      puts "NCSI_REG_SA_CACHE_VALID_FLAG_1        =  $NCSI_REG_SA_CACHE_VALID_FLAG_1"
      puts "NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_1    =  $NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_1"
    
      set NCSI_REG_SA_CACHE_VALID_FLAG_2          [reg_read -n NCSI_REG_SA_CACHE_VALID_FLAG_2]
      set NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_2      [reg_read -n NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_2]
      puts "NCSI_REG_SA_CACHE_VALID_FLAG_2        =  $NCSI_REG_SA_CACHE_VALID_FLAG_2"
      puts "NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_2    =  $NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_2"
    
      set NCSI_REG_SA_CACHE_VALID_FLAG_3          [reg_read -n NCSI_REG_SA_CACHE_VALID_FLAG_3]
      set NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_3      [reg_read -n NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_3]
      puts "NCSI_REG_SA_CACHE_VALID_FLAG_3        =  $NCSI_REG_SA_CACHE_VALID_FLAG_3"
      puts "NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_3    =  $NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_3"
    
      set NCSI_REG_SA_CACHE_VALID_FLAG_4          [reg_read -n NCSI_REG_SA_CACHE_VALID_FLAG_4]
      set NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_4      [reg_read -n NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_4]
      puts "NCSI_REG_SA_CACHE_VALID_FLAG_4        =  $NCSI_REG_SA_CACHE_VALID_FLAG_4"
      puts "NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_4    =  $NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_4"
    
      set NCSI_REG_SA_CACHE_VALID_FLAG_5          [reg_read -n NCSI_REG_SA_CACHE_VALID_FLAG_5]
      set NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_5      [reg_read -n NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_5]
      puts "NCSI_REG_SA_CACHE_VALID_FLAG_5          =  $NCSI_REG_SA_CACHE_VALID_FLAG_5"
      puts "NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_5    =  $NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_5"
    
      set NCSI_REG_SA_CACHE_VALID_FLAG_6          [reg_read -n NCSI_REG_SA_CACHE_VALID_FLAG_6]
      set NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_6      [reg_read -n NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_6]
      puts "NCSI_REG_SA_CACHE_VALID_FLAG_6        =  $NCSI_REG_SA_CACHE_VALID_FLAG_6"
      puts "NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_6    =  $NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_6"
    
      set NCSI_REG_SA_CACHE_VALID_FLAG_7          [reg_read -n NCSI_REG_SA_CACHE_VALID_FLAG_7]
      set NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_7      [reg_read -n NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_7]
      puts "NCSI_REG_SA_CACHE_VALID_FLAG_7        =  $NCSI_REG_SA_CACHE_VALID_FLAG_7"
      puts "NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_7    =  $NCSI_REG_SA_CACHE_REPLACEMENT_FLAG_7"
    
      for {set i 0} {$i < 8} {incr i} {
         set mac_x_0  [reg read [expr  0x4038c + 8 * $i]]
     puts "NCSI_REG_SA_CACHE_MAC_ADDR_LO_${i} =  $mac_x_0"
         
     set mac_x_1 [reg read [expr  0x40390 + 8 * $i]]
     puts "NCSI_REG_SA_CACHE_MAC_ADDR_HI_${i} =  $mac_x_1"
         
     set mac_x_1 [reg read [expr  0x403cc + 4 * $i]]
     puts "NCSI_REG_SA_CACHE_DEST_HOST_FUNC_${i} =  $mac_x_1"
      }
}

proc passthrough {} {
    passth_help
    }
    
proc pass {} {
    passth_help
    }

proc passth_conf_nw2bmc {ch bmc_mac_addr} {
   puts "To configure passthrough need to use serial" 
   reg_write -n NIG_REG_RX_LLH_NCSI_MCP_MASK -v 0xf050202b
   reg_write -n NIG_REG_RX_LLH_NCSI_MCP_MASK_2 -v 0x7
   # ====== VLAN ======
   reg_write -n NIG_REG_RX_LLH_NCSI_MCP_MASK_IVLAN   -v 0x3

   set_mac_addr $ch $bmc_mac_addr
   reg_write -n NCSI_REG_CONFIG -v 0x126c
}

proc passth_conf_os2bmc {ch bmc_mac_addr} {

   reg_write -n NIG_REG_TX_LLH_NCSI_MCP_MASK -v 0xf050202b
   reg_write -n NIG_REG_TX_LLH_NCSI_MCP_MASK_2 -v 0x7
   reg_write -n NIG_REG_TX_LLH_NCSI_NTWK_MASK -v 0xf050202b
   reg_write -n NIG_REG_TX_LLH_NCSI_NTWK_MASK_2 -v 0x7
   # ====== VLAN ======
   reg_write -n NIG_REG_TX_LLH_NCSI_MCP_MASK_IVLAN   -v 0x3
   reg_write -n NIG_REG_TX_LLH_NCSI_NTWK_MASK_IVLAN -v 0x3
   set_mac_addr $ch $bmc_mac_addr
   reg_write -n NCSI_REG_CONFIG -v 0x126c
}


proc set_mac_addr {ch bmc_mac_addr} {

    puts "ch =  $ch mac_addr =  $bmc_mac_addr"
    set low_bits  0x[format %X [expr $bmc_mac_addr & 0xffffffff]]
    set high_bits 0x[format %X [expr $bmc_mac_addr >> 32]]
    # ====== MAC ADDRESS ======
    reg_write -n NCSI_REG_BMC_MAC_ADDR_LO_$ch -v $low_bits
    reg_write -n NCSI_REG_BMC_MAC_ADDR_HI_$ch -v $high_bits
    reg_write -n NCSI_REG_BMC_MAC_VALID_FLAG_$ch -v  0x1
    #--> Destination MAC address
    reg_write -n NIG_REG_LLH_DEST_MAC_0_0 -v $low_bits
    reg_write -n NIG_REG_LLH_DEST_MAC_0_1 -v $low_bits
    # reg write [expr    0x501088 + 8 * $ch]  $low_bits
    # reg write [expr    0x50108c + 8 * $ch]  $high_bits
}

proc passth_conf {{ch} {bmc_mac_addr} {path}} {
    
    if {$path == "nw2bmc"} {
        passth_conf_nw2bmc $ch $bmc_mac_addr
    } elseif {$path == "os2bmc"} {
        passth_conf_os2bmc $ch $bmc_mac_addr
    } else {
        passth_conf_os2bmc $ch $bmc_mac_addr
        passth_conf_nw2bmc $ch $bmc_mac_addr
    }
}

proc passth {args} {
    set ch 0
    set bmc_mac_addr 0
    set path ""
   
   if {[llength $args] == 0} {
      usage
   }

   if {[lindex $args 0] == "-conf"} {
       if {[llength $args] > 1} {
           set ch [lindex $args 1]
       }
        
       if {[llength $args] > 2} {
            set bmc_mac_addr 0x[format %X [lindex $args 2]]
       }

       if {[llength $args] > 3} {
           set path  [lindex $args 3]
       }
   }
    
   # Parse the options.
   for {set i 0} {$i < [llength $args]} {incr i} {        
      switch -- [lindex $args $i] {
         "-dump"      { incr i; passth_dump }
         "-stat"      { incr i; passth_stat }
         "-btb"        { incr i;passth_btb }
         "-cache"     { incr i; passth_cache }
         "-conf"      { incr i; passth_conf $ch $bmc_mac_addr $path; return 0 }
         "-help"      { usage; return -1 }
         default      { usage; return -1 }
      }
   }
}

passth_help
