###########################################
#                                         #
#          GENERIC EFUSE access           #
#                                         #
###########################################
# Check with Dharma - do we need soft or fuse
global GEN_EFUSE_DIRECT_BANK
set GEN_EFUSE_DIRECT_BANK 1
global local_efuse
global test_local_efuse 
set test_local_efuse 1
set local_efuse(x) 0

proc clear_efuse {} {
	 global array local_efuse
        for {set i 0} {$i < 512} {incr i} {
       	 set local_efuse($i) 0
		 unset local_efuse($i) 
	 }
}

proc blow_gen_efuse_bit {bit} {
	 global array local_efuse
	 global test_local_efuse 

	 global GEN_EFUSE_DIRECT_BANK
  
        # Check with Dharma - do we need soft or fuse
        
        if {$test_local_efuse} {
		 set local_efuse($bit) 1
	 } else {
              set PRELOAD_START_BIT 580
	       set EFUSE_BANK_SIZE 128
	       # Set the PRELOAD BANK. Each bank size is 128bits. The preload bank starts at bit 580 (means bank 0xb)
	       reg_write -n FUS_BNK_DAT -v [expr ($PRELOAD_START_BIT + $bit)/$EFUSE_BANK_SIZE)]
	       reg_write -n FUS_PROG -v [expr (($bit % $EFUSE_BANK_SIZE) << 4) | (1<<12) | ($GEN_EFUSE_DIRECT_BANK << 15)]

	       # Wait for operation to complete
	       while {1} {
		       set prg [reg_read -n FUS_PROG]
		       if {[expr $prg & (1<<12)] == 0} {
			       break;
		       }
	       }
	 }
}

proc blow_gen_efuse_value {bit value len} {

        for {set i 0} {$i < $len} {incr i} {
       	 set fuse_volt [expr $value & 1]
       	 if {$fuse_volt} {
			 blow_gen_efuse_bit $bit
		 }
       	 incr bit
       	 set value [expr $value >> 1]
	 }
        # Return the updated bit offset
        return $bit
}

proc read_gen_efuse_bit {bit} {
	 global GEN_EFUSE_DIRECT_BANK
	 global local_efuse
	 global test_local_efuse 

        if {$test_local_efuse} {
       	 set val [info exists local_efuse($bit)]
	 } else {
		 set PRELOAD_START_BIT 580
		 set EFUSE_BANK_SIZE 128
		 # Set the PRELOAD BANK. Each bank size is 128bits. The preload bank starts at bit 580 (means bank 0xb)
		 reg_write -n FUS_BNK_DAT -v [expr ($PRELOAD_START_BIT + $bit)/$EFUSE_BANK_SIZE)]
		 reg_write -n FUS_RCMD -v [expr (($bit % $EFUSE_BANK_SIZE) << 4) | (1<<12) | ($GEN_EFUSE_DIRECT_BANK << 15)]
		 # Wait for operation to complete
		 while {1} {
			 set rcmd [reg_read -n FUS_RCMD]
			 if {[expr $rcmd & (1<<12)] == 0} {
				 break;
			 }
		 }
       	 set val [expr ($rcmd >> 14) & 1]
	 }
        return $val
}

proc dump_gen_efuse {} {
        for {set i 0} {$i < 512} {incr i} {
       	 set val [read_gen_efuse_bit $i]
       	 if {$val} {
			 puts "$i: $val"
		 }
	 }
}

proc burn_efuse_to_nvm {} {
	 global local_efuse
	 global test_local_efuse 
        set gen_efuse_nvm_addr 0x7f500
        set val 0
        for {set i 0} {$i < 512} {incr i} {
       	set val [expr $val | ([read_gen_efuse_bit $i] << ($i & 0x1f))]
       	if {[expr ($i & 0x1f) == 0x1f]} {
       		nvm write $gen_efuse_nvm_addr $val
       		incr gen_efuse_nvm_addr 4
       		set val 0
		}
	 }
}

# Block              | Offset | Value1    | Opt Value2 
# -------------------+--------+-----------+--------------
# ROM_FALLBACK                 0
# GSER_ALL_LANES(1)   0x50     0x12345678  0xabcdf123
# GSER_COMMON(2)      0x20     0xf         0x4444
# PEM(3)              0x11     0x3
# PCIEEP(4)           0x12     0x5        
# GRC(5)              0xe20000 0x99
# DELAY_USEC(6)       20
#######################################################
proc program_efuse_preload {filename} {
        set PCIE_OFFSET_LEN 	12
	 set MISC_OFFSET_LEN	22
        set DELAY_VAL_LEN 	12
	 set VAL_LEN 		32
        set CMD_LEN		3
        set GSER_ALL_LANES 	1
	 set GSER_COMMON 	2
	 set PEM 		3
	 set PCIEEP		4
	 set GRC 		5
	 set DELAY_USEC 	6

	 clear_efuse
        #  Slurp up the data file
	 set fd [open $filename r]
	 set file_data [read $fd]
	 close $fd
       
        set cur_fuse_offset 1
        
        #  Process data file
        set data [split $file_data "\n\r"]
        
	 foreach line $data {
       	 set cmd 0
		 set cmd_len 0
		 set cmd_offset 0
		 set cmd_offset_len 0
		 set cmd_val1 0
		 set cmd_val1_len 0
		 set cmd_val2 0
		 set cmd_val2_len 0

       	 # Avoid comments 
		 set chars [split $line {}]
		 
		
       	 if {[lindex $chars 0] == "#"} {
       		 continue
		 }
		 
		 set req [lsearch -inline -all -not -exact [split $line " "] {} ]
		 if {[llength $req] == 0} {
       		 continue
		 }
		 
       	 set cmd_str "[lindex $req 0]"
       	 puts "cmd str : $cmd_str Req: $req"
       	
       	 switch $cmd_str {
       		 "ROM_FALLBACK" {
       			# Expected to be the first bit 
				set cmd_val1 [lindex $req 1]
       			if {$cmd_val1} {
       				puts "rollback"
					blow_gen_efuse_bit 0
				}
       			continue
			 }
			 GSER_ALL_LANES { 
				 set cmd $GSER_ALL_LANES
				 set cmd_len $CMD_LEN
				 set cmd_offset		 [lindex $req 1]
				 set cmd_offset_len	 $PCIE_OFFSET_LEN
				 set cmd_val1		 [lindex $req 2]
				 set cmd_val1_len	 $VAL_LEN
       			 if {[llength $req] == 4} {
					 set cmd_val2		 [lindex $req 3]
					 set cmd_val2_len	 $VAL_LEN
				 }
			 }
			 GSER_COMMON { 
				 set cmd $GSER_COMMON
				 set cmd_len $CMD_LEN
       			 
				 set cmd_offset		 [lindex $req 1]
				 set cmd_offset_len	 $PCIE_OFFSET_LEN
				 set cmd_val1		 [lindex $req 2]
				 set cmd_val1_len	 $VAL_LEN
       			 if {[llength $req] == 4} {
					 set cmd_val2		 [lindex $req 3]
					 set cmd_val2_len	 $VAL_LEN
				 }
			 }
			 PEM {
				 set cmd $PEM
				 set cmd_len $CMD_LEN
				 set cmd_offset		 [lindex $req 1]
				 set cmd_offset_len	 $PCIE_OFFSET_LEN
				 set cmd_val1		 [lindex $req 2]
				 set cmd_val1_len	 $VAL_LEN
			 }
			 PCIEEP {
				 set cmd $PCIEEP
				 set cmd_len $CMD_LEN
				 set cmd_offset		 [lindex $req 1]
				 set cmd_offset_len	 $PCIE_OFFSET_LEN
				 set cmd_val1		 [lindex $req 2]
				 set cmd_val1_len	 $VAL_LEN
			 }
			 GRC {
			 	 set cmd $GRC
				 set cmd_len $CMD_LEN
				 set cmd_offset		 [lindex $req 1]
				 set cmd_offset_len	 $MISC_OFFSET_LEN
				 set cmd_val1		 [lindex $req 2]
				 set cmd_val1_len	 $VAL_LEN
			 }
			 DELAY_USEC {
				  set cmd $DELAY_USEC
				  set cmd_len $CMD_LEN
       			  set cmd_val1		 [lindex $req 1]
				  set cmd_val1_len	 $DELAY_VAL_LEN
			 }
       		 default {
       			 puts "Unknown command [lindex $req 0]"
			 }
		 }
       	 
       	 # Blow the fuses:
		 puts "Cmd: $cmd_str - $cmd $cmd_len $cur_fuse_offset"
		 set cur_fuse_offset [blow_gen_efuse_value $cur_fuse_offset $cmd 	$cmd_len]

		 puts "Offset: $cmd_offset $cmd_offset_len $cur_fuse_offset"
		 set cur_fuse_offset [blow_gen_efuse_value $cur_fuse_offset $cmd_offset $cmd_offset_len]

		 puts "Val1: $cmd_val1 $cmd_val1_len $cur_fuse_offset"
		 set cur_fuse_offset [blow_gen_efuse_value $cur_fuse_offset $cmd_val1 	$cmd_val1_len]

		 puts "Val2: $cmd_val2 $cmd_val2_len $cur_fuse_offset"
		 set cur_fuse_offset [blow_gen_efuse_value $cur_fuse_offset $cmd_val2 	$cmd_val2_len]
	 }
	 burn_efuse_to_nvm
}


