namespace eval server {

set host_interactive 1
set client_interactive 1

proc host_interactive_puts {args} {
	global server::host_interactive
	if {$host_interactive} {
		puts {*}$args
	}
}

proc client_interactive_puts {args} {
	global server::client_interactive
	if {$client_interactive} {
		puts {*}$args
	}
}

# redefinition of puts
proc server_puts args {
		
	global server::client
	if {![info exist client]} {error "server_puts is only for server::Connection internal use"}
	
	eval "server::_puts $args"
	
	catch {	
		set len [llength $args]
		foreach {arg1 arg2 arg3} $args { break }
		
		if {$len == 1} {
			server::_puts $client $arg1
		} elseif {$len == 2} {
		if {![string compare [string index $arg1 0] -]} {
			server::_puts $arg1 $client $arg2
		} elseif {![string compare $arg1 stdout] \
			|| ![string compare $arg1 stderr]} {
				server::_puts $client $arg2
		} else {
			set len 0
		}
		} elseif {$len == 3} {
		if {![string compare [string index $arg1 0] -] \
			&& (![string compare $arg2 stdout] \
			|| ![string compare $arg2 stderr])} {
				server::_puts $arg1 $client $arg3
		} elseif {(![string compare $arg1 stdout] \
			|| ![string compare $arg1 stderr]) \
			&& ![string compare $arg3 nonewline]} {
				server::_puts $client $arg2 $arg3
		} else {
			set len 0
		}
		} else {
			set len 0
		}
			
		if {$len == 0} {
			# eval "server::_puts $args"
		}    
	}
	return -code 0
}

proc _prompt {sock} {
	catch {
		client_interactive_puts -nonewline $sock "$sock>"
		flush $sock
	}
	return -code 0
}

proc Server {sock addr port} {	
	global server::connection

	#protect the server from errors
	catch  {
		# Ensure that each "puts" by the server
		# results in a network transmission			
		fconfigure $sock -blocking 1 -buffering line
		
		# send some welcome message
		client_interactive_puts $sock "Welcome to TCL server" 
		client_interactive_puts $sock "TCL version [info tclversion]"
		client_interactive_puts $sock "Run by [info nameofexecutable] on host [info hostname]"
		flush $sock
		
		_prompt $sock
		
		# Record the client's information
		host_interactive_puts "Accepted $sock from $addr port $port"
		set connection($sock) [list $addr $port]
		
		# Set up a callback for when the client sends data
		# old line: fileevent $sock readable [list [namespace code Connection] $sock]
		fileevent $sock readable "[namespace code Connection] $sock"
	}
	return -code 0
}

proc CloseConnetion {sock} {	
	global server::connection
	if ([info exist connection($sock)]) {
		catch {
			fileevent $sock readable {} 
			close $sock
		}
		host_interactive_puts "Closed $sock $connection($sock)"	
		unset connection($sock)
	}
}

proc Connection {sock} {
	global server::connection
	global server::client
	
	# Check end of file or abnormal connection drop. Read line from client
	if {[eof $sock] || [catch {gets $sock line}]} {
		CloseConnetion $sock
	} else {			
		# remove BS at start of line:
		while { [regsub "^\[\\b\]+(.*)$" $line \\1 line] } {} 
		# apply BS at middle of line
		while {[regsub "^(\[^\\b\]*)\[^\\b\]\\b(.*)$" $line \\1\\2 line]} {} 
		# remove all control characters
		while {[regsub "^(\[^\[:cntrl:\]\]*)\[\[:cntrl:\]\]+(.*)$" $line \\1\\2 line]} {} 
		
		host_interactive_puts "$sock>$line"

		# redefine puts
		set client $sock
		rename ::puts _puts
		rename server_puts ::puts
		
		# evaluate input from remote client and handle errors
		if {[catch {set output [namespace eval :: $line]} result]} {
			catch {
				_puts stderr "Error: $result"
				_puts $sock "Error: $result"
			}
		} else {					
			if {[string bytelength $output]!=0} {
				catch {
					_puts stdout $output
					_puts $sock $output				
				}
			}
		}
		
		#restore puts
		rename ::puts server_puts
		rename _puts ::puts
		unset client
		
		#prompt
		_prompt $sock
		
		# If socket died, close the connection
		if {[catch {eof $sock}] || [eof $sock]} { CloseConnetion $sock}
	}
}

# Display a list of active connections
proc Connections {} {
	global server::serverSocket
	global server::serverPort
	global server::connection
	
	if {[info exist serverSocket]} {
		puts "Listening on $serverSocket port $serverPort"		
	}	
	if {[info exist connection]} {
		puts "Active connections: "
		foreach sock [array names connection] {
			puts "$sock : $connection($sock)"
		}
	}
}

# Stop the server
proc Stop {} {
	global server::serverSocket	
	global server::connection
	global server::serverRunning
		
	if {![info exist serverRunning] || $serverRunning==0} {
		error "Server is not running."
		return
	}
	
	if {[info exist connection]} {
		foreach sock [array names connection] {
			CloseConnetion $sock			
		}
		unset connection
	}
	
	if {[info exist serverSocket]} {
		close $serverSocket	
		host_interactive_puts "Server $serverSocket stopped."
		unset serverSocket
	}
	
	set serverRunning 0
	return -code 0
}

# Start the server
proc Start {port} {	
	global server::serverSocket
	global server::serverPort
	global server::serverRunning
	
	if {[info exist serverRunning] && $serverRunning!=0} {
		error "Server already started."		
	}
	set serverRunning 1
	set serverPort $port
	set serverSocket [socket -server [namespace code Server] $port]
	host_interactive_puts "Server $serverSocket started on port $port."
}

# Enter event loop until server is stopped
proc Wait {} {
	global server::serverRunning	
	vwait serverRunning
}

proc setClientInteractive {value} {
	global server::client_interactive
	set client_interactive $value
}

proc setHostInteractive {value} {
	global server::host_interactive
	set host_interactive $value
}

}
