###########################################################################
#
# Copyright(c) 2014-2017 Broadcom Corporation, all rights reserved
# Proprietary and Confidential Information.
#
##########################################################################



 ##********************************************************
 ## Name: stdout
 ##
 ## Description:
 ##
 ## Switch stdout handling by puts command from the usual
 ## behaviour to a state where stdout is redefined to be
 ## another file.
 ##
 ## Native puts command error handling is unimpaired and
 ## a dangling filehandle is never generated.
 ##
 ##
 ## Calling convention:
 ##
 ##   stdout off          - redirects stdout to nowhere
 ##
 ##   stdout off /tmp/foo - redirects stdout to /tmp/foo
 ##
 ##   stdout on           - restores normal behaviour 
 ##********************************************************

   proc stdout { switch { file "" } } {
      if { ! [ llength [ info command __puts ] ] && \
            [ string equal off $switch ] } {
         
         rename puts __puts
         
         if { [ string length $file ] } {            
            eval [ subst -nocommands {proc puts { args } {
               # print to screen               
               eval __puts \$args                             
               set fid [ open $file a+ ]
               if { [ llength \$args ] > 1 && \
                  [ lsearch \$args stdout ] == 0 } {                  
                  set args [ lreplace \$args 0 0 \$fid ]                  
               } elseif {  [ llength \$args ] == 1 } {                  
                  set args [ list \$fid \$args ]                  
               } elseif {[ lsearch \$args -nonewline] == 0} {
                   set args [ lreplace \$args 0 0 -nonewline \$fid ]                   
               } 
               if { [ catch {
                  #print to file                   
                  eval __puts \$args                  
               } err ] } {                  
                  close \$fid                  
               }
               close \$fid
            }} ]
         } else {            
            eval [ subst -nocommands {proc puts { args } {
               if {  [ llength \$args ] > 1 && \
                     [ lsearch \$args stdout ] == 0 || \
                     [ llength \$args ] == 1 } {
                     # no-op
               } else {
                  eval __puts \$args
               }
            }} ]   
         }
      } elseif { [ llength [ info command __puts ] ] && \
               [ string equal on $switch ] } {
            rename puts {}            
            rename __puts puts
      }
 }


