###########################################################################
#
# Copyright(c) 2014-2017 Cavium, Inc., all rights reserved
# Proprietary and Confidential Information.
#
#
# Name:        template.tcl
#
# Description: 
#    This script defines the eDiag test template
#
# Note: Please replace "testname" with the appropriate string
#                        
###########################################################################


# Specify any dependency on other packages
package require Tcl

# Specify the package this module provides
package provide _e4_tests 1.0

namespace eval _e4_tests {
    
    # Export the package
    namespace export e4_tests   
}

# Give a short-name for the regression test.
interp alias {} testname {}  "_e4_tests::testname_main"


#Globals:

#Registers information in 4 arrays
global regsName
global regsOffset


# -------------------------------------------------------
# The description of the test usage
# ------------------------------------------------------
proc _e4_tests::testname_usage {} {
    puts " <Testname> - <General description>"
    puts "Usage: <test command> <options>"    
    puts "Parameters: "
    puts "            -option1"
    puts "            -option2"    
    puts "Optional Parameters:"       
}

# -------------------------------------------------------
# The test main proc
# -------------------------------------------------------
proc _e4_tests::testname_main { args } { 
    
    global rdfFile
    global regsName
    global regsOffset

    #Parse the appropriate RDF file 
    parse_rdf

    # Parse the input arguments
    for {set i 0} {$i < [llength $args]} {incr i} {
        switch -- [lindex $args $i] {    
            "-option1" { incr i}
            "-option2" { incr i}
            default { testname_usage ; return 1}
        }
    }
    
    puts "<testname> - <General description>"
    
    # Initialization 
    testname_init
           
    # Test procedures/commands
    set returnVal [testname_test] 
       
    # Cleanup before exiting test
    testname_cleanup
    
    if {$returnVal != "passed"} {
        puts "***********************************************************"
        puts "**             TEST FAILED !!!                             "
        puts "***********************************************************"
    } else {
        # Verify idle_chk passes after this test
        driver load
        if {[idle_chk -noWarn] == 0} {        
            puts "***********************************************************"
            puts "**             TEST PASSED !!!                             "
            puts "***********************************************************"
        } else {        
            puts "Idle_chk after <testname> failed !!!"
            set returnVal "failed"
        }        
        driver unload    
    }
    
    # Test should return passed/failed
    return $returnVal
}


# -------------------------------------------------------
# The test init proc
# -------------------------------------------------------
proc _e4_tests::testname_init { args } {
           
    # load driver at the begining of the test
    puts "loading driver ..."
    driver load                                     
}


# -------------------------------------------------------
# The test cleanup commands
# -------------------------------------------------------
proc _e4_tests::testname_cleanup { args } {
    
    # Cleanup commands before exiting 
    # Such as: Unset variables, driver unload
    
        
    # unload the driver if running
    puts "unloading driver ..."
    driver unload
                          
    # Unset variables, arrays, specs before exiting test
    
    # Free allocated memories
}

# -------------------------------------------------------
# The test commands
# -------------------------------------------------------
proc _e4_tests::testname_test { args } {
    
    set returnVal "passed"      
    
}





