###########################################################################
#
# Copyright(c) 2014-2017 Cavium, Inc., all rights reserved
# Proprietary and Confidential Information.
#
# Module Description:
# This TCL script for debugging WoL
#
# Written by Yaniv Rosner/Alon Elhanani, QLogic, Nov 2014
#
###########################################################################

# Specify any dependency on other packages
package require Tcl


proc set_device {gport} {
   set base_dev [expr $::current(DEV) - $::current(PCI_FUNC)]
   set rest_devices [expr $::current(TOTAL_DEV) - $base_dev + 1]
   
   set new_device [expr $base_dev + $gport]
   #puts "set device $new_device"
   device $new_device -no_display      
   return $rest_devices
}

proc yesno {val} {
    if {$val} {
        return "YES"
    } else {
        return "NO "
    }
}

proc SECTION_OFFSET {offsize} {
	 return [expr ($offsize & 0x0000ffff) << 2]
}
proc SECTION_SIZE {offsize} {
	 return [expr (($offsize & 0xFFFF0000) >> 16 ) << 2]
}


proc get_link_status {gport} { 
    set GRCBASE_MCP 0xe20000
    set PUBLIC_PORT 4
    set LINK_STATUS_OFFSET 4
    set public_base [expr [reg_read -n MISC_REG_SHARED_MEM_ADDR] | $GRCBASE_MCP]
    set port_offsize [reg read [expr $public_base +  4 + (4 * ($PUBLIC_PORT))]]
    set pub_port_addr [expr $GRCBASE_MCP + [SECTION_OFFSET $port_offsize] + [SECTION_SIZE $port_offsize] * $gport]
    set link_status [reg read [expr $pub_port_addr + $LINK_STATUS_OFFSET]]
    set link_up [expr $link_status & 1]
  #  puts "public_base $public_base port_offsize $port_offsize pub_port_addr $pub_port_addr link_status $link_status"
    if {$link_up} {
        set link_speed [expr ($link_status & 0x1e) >> 1 ]
	 if {$link_speed == 2} { set link_str "UP - 1000Mbps" }
        if {$link_speed == 3} { set link_str "UP - 10Gbps" }
        if {$link_speed == 4} { set link_str "UP - 20Gbps" }
	 if {$link_speed == 5} { set link_str "UP - 40Gbps" }
	 if {$link_speed == 6} { set link_str "UP - 50Gbps" }
	 if {$link_speed == 7} { set link_str "UP - 100Gbps" }
	 if {$link_speed == 8} { set link_str "UP - 25Gbps" }
    } else {
        set link_str "DOWN"
    }
    return $link_str
}

proc is_ah_proc {} {
   return [is_e4_K2]
}

# Due to bug in AH convert the PF to the HW WOL PF
proc wol_pf_to_hw { pf } {
    set port_mode [reg_read -n MCP_REG_PORT_MODE]
    #4 ports
    if {$port_mode == 2} {
            return [expr $pf/4 + 4*($pf & 3)]
    } else { # 2 OR 1 ports
            return [expr $pf/2 + 8*($pf & 1)]
    }
}


proc wol_dump_dev {gport func_gap} {
    #  1. Check PME enabled
    set NIG_REG_MPKT_MAC_ADDR 0x508190
    set WOL_REG_MPKT_MAC_ADDR 0x608148
    set cur_dev $::current(DEV)    
    set rest_devices [set_device $gport]
    set is_ah [is_ah_proc]
  

    if {$is_ah} {
            set nig_reg_mpkt_enable [expr 0x608144]
            set nig_reg_wake_info   [expr 0x608184]
    } else {
            set nig_reg_mpkt_enable [expr 0x508188]
            set nig_reg_wake_info   [expr 0x5081c4]
    }


    #set magic_packet_reg [reg read $nig_reg_mpkt_enable]
    set mcp_reg_scratch 0xe20000
    set misc_reg_gen_purp_cr0 0x008c80
    set nvm_cfg_addr [reg read $misc_reg_gen_purp_cr0]
    set nvm_cfg1_offset [expr [reg read [expr $nvm_cfg_addr + 0x4]]]
    set port_cfg_addr [expr $nvm_cfg1_offset + $mcp_reg_scratch + 0x230 + $gport*0x258]
    set port_feature [expr [reg read [expr $port_cfg_addr+0x10]]]
    set link_str [get_link_status $gport]
   
    
    set wol_enabled_in_nvram [expr $port_feature]
    set func 0    
    
    for {set dev 0} {$dev <= $rest_devices} {incr dev} {
       set prev_pci_func $::current(PCI_FUNC)
       set wol_pf [wol_pf_to_hw $prev_pci_func]
    
       set magic_packet_regonition_enabled [expr [reg read [expr $nig_reg_mpkt_enable]] & 0x1]
       set magic_packet_received [expr [reg read [expr $nig_reg_wake_info] ] ]
     
       if {$is_ah} {
               pretend pf [expr $wol_pf + 1]
               set multi_function_wol_en_pf($func) [reg_read -n WOL_REG_MPKT_ENABLE]
               lappend mac_lo [format %08x [reg read [expr $WOL_REG_MPKT_MAC_ADDR]]]
               lappend mac_hi [format %04x [reg read [expr $WOL_REG_MPKT_MAC_ADDR + 4]]]
               pretend pf [expr $prev_pci_func +1]
       } else {
               set multi_function_wol_en_pf($func) [reg_read -n NIG_REG_MPKT_ENABLE]
               lappend mac_lo [format %08x [reg read [expr $NIG_REG_MPKT_MAC_ADDR]]]
               lappend mac_hi [format %04x [reg read [expr $NIG_REG_MPKT_MAC_ADDR + 4]]]
       }
	   
     
       if {$is_ah} {
               set ACPI_BASE 0x600000
               set pme_register 0x2044
       } else {
               set ACPI_BASE 0x500000
               set pme_register 0x204c
       }
      
      
       set pme_reg($func) [reg read $pme_register]
       set pme_enabled($func) [expr $pme_reg($func) & 0x100]
       set pme_status_asserted($func) [expr $pme_reg($func) & 0x8000]

	 if {$is_ah} {
               pretend pf [expr $wol_pf + 1]
       }
       set acpi_enable($func) [reg read [expr $ACPI_BASE + 0x8100]]
                                              
       set acpi_crc_0($func)  [reg read [expr $ACPI_BASE + 0x8104]]
       set acpi_crc_1($func)  [reg read [expr $ACPI_BASE + 0x810c]]
       set acpi_crc_2($func)  [reg read [expr $ACPI_BASE + 0x8114]]
       set acpi_crc_3($func)  [reg read [expr $ACPI_BASE + 0x811c]]
       set acpi_crc_4($func)  [reg read [expr $ACPI_BASE + 0x8124]]
       set acpi_crc_5($func)  [reg read [expr $ACPI_BASE + 0x812c]]
       set acpi_crc_6($func)  [reg read [expr $ACPI_BASE + 0x8134]]
       set acpi_crc_7($func)  [reg read [expr $ACPI_BASE + 0x813c]]
                                          
       set acpi_len_0($func)  [reg read [expr $ACPI_BASE + 0x8108]]
       set acpi_len_1($func)  [reg read [expr $ACPI_BASE + 0x8110]]
       set acpi_len_2($func)  [reg read [expr $ACPI_BASE + 0x8118]]
       set acpi_len_3($func)  [reg read [expr $ACPI_BASE + 0x8120]]
       set acpi_len_4($func)  [reg read [expr $ACPI_BASE + 0x8128]]
       set acpi_len_5($func)  [reg read [expr $ACPI_BASE + 0x8130]]
       set acpi_len_6($func)  [reg read [expr $ACPI_BASE + 0x8138]]
       set acpi_len_7($func)  [reg read [expr $ACPI_BASE + 0x8140]]

       if {$is_ah} {
               pretend pf [expr $prev_pci_func + 1]
       }
       incr func;
       
       if {[expr $::current(DEV) + $func_gap] > $rest_devices} {
            break;
       }
      
       #puts "Move to next device [expr $::current(DEV) + $func_gap]"
       device [expr $::current(DEV) + $func_gap] -no_display
       if {$prev_pci_func >= $::current(PCI_FUNC)} {
            break;
        }
    }
    set max_func $func
    puts "device \[$gport\]:"
    puts "LINK"
    puts "   $link_str"
    puts "WOL"
    puts "   Enabled in nvram: [yesno $wol_enabled_in_nvram]"
    puts "MAGIC PACKET"
    puts "   Recognition enabled: [yesno $magic_packet_regonition_enabled]"
    puts "   Received: [yesno $magic_packet_received]"
    puts "PME"
    for {set func 0} {$func < $max_func} {incr func} {
       puts "   pf$func: Reg $pme_register val: $pme_reg($func)"
       puts "        Enabled: [yesno $pme_enabled($func)]"
       puts "        Status_asserted: [yesno $pme_status_asserted($func)]"
    }
    puts "MAC ADDR LISTEN"
    for {set func 0} {$func < $max_func} {incr func} {
       puts "   pf$func: [yesno $multi_function_wol_en_pf($func)] [lindex $mac_hi $func][lindex $mac_lo $func]"
    }

    puts "ACPI pattern"
    for {set func 0} {$func < $max_func} {incr func} {
        puts "   ACPI pattern Enabled             [yesno $acpi_enable($func)]"
        puts "   ACPI pattern 0-7 Length in bytes [format "0x%02x 0x%02x 0x%02x 0x%02x\r\n                                    0x%02x 0x%02x 0x%02x 0x%02x" $acpi_len_0($func) $acpi_len_1($func) $acpi_len_2($func) $acpi_len_3($func) $acpi_len_4($func) $acpi_len_5($func) $acpi_len_6($func) $acpi_len_7($func)]"
        puts "   ACPI pattern 0-7 CRC32           [format "0x%08x 0x%08x 0x%08x 0x%08x\r\n                                    0x%08x 0x%08x 0x%08x 0x%08x" $acpi_crc_0($func) $acpi_crc_1($func) $acpi_crc_2($func) $acpi_crc_3($func) $acpi_crc_4($func) $acpi_crc_5($func) $acpi_crc_6($func) $acpi_crc_7($func)]"
    }   
    
    device $cur_dev -no_display
}

proc wol_dump {} {
    set MAX_FUNC 16
    set port_mode [reg_read -n MCP_REG_PORT_MODE]
    
    if {$port_mode == 2} {
	     set num_ports 4
         set num_func_per_port 4
    } else {
            if {$port_mode == 1} {
                    set num_ports 2
                    set num_func_per_port 8
            } else {
                    set num_ports 1
                    set num_func_per_port 16
            }
    }
    
    set func_gap [expr $MAX_FUNC / $num_func_per_port]
    for {set gport 0} {$gport < $num_ports} {incr gport} {
	     puts "\nRead WOL dump for port $gport:"
            wol_dump_dev $gport $func_gap
    }
}

proc rearm_wol {} {
        for {set dev 1} {$dev <= 4} {incr dev} {
       	 device $dev -no_display
       	 reg write 0x204c 0xa108

         set is_ah [is_ah_proc]
		 # Rearm Magic Packet WoL
         if {$is_ah} {
                 reg write 0x5081c0 0 
                 reg write 0x5081c0 1
         } else {
                 reg write 0x608180 0 
                 reg write 0x608180 1
         }
	 }
}
