##########################################################
# idle_chk.tcl                                           #
#                                                        #
#Copyright(c) 2014-2017 Cavium, Inc.                      #
#Proprietary and Confidential Information.               #
#                                                        #
#WARNING: DO NOT CHANGE THIS SCRIPT MANUALLY !           #
#SCRIPT IS CREATED AUTOMATICALY VIA create_idle_chk.tcl  #
#                                                        #
# Module Description:                                    #
# The script tests the value of important registers,     #
# to verify the chip is at IDLE state.                   #
##########################################################
global errors
global warnings
global total
global dumpMode
global outfileName
global outfile
# Check if condition exists, if so print a relevant message according to its sevirity
proc CONDITION_CHK {{val} {val1} {val2} {condition} {severity} {fail_msg}} {	
	global errors
	global warnings
	global total
	global outfileName
	global outfile	
	
	incr total
	
	if {[expr $condition]} {		
		switch $severity { 
			IDLE_CHK_ERROR
			{
				if {$outfileName != ""} { 
					puts $outfile "idle_chk. Error (level 1)  : $fail_msg"
				} else {
					puts "idle_chk. Error (level 1)  : $fail_msg"
				}
				incr errors				
			}
			IDLE_CHK_ERROR_NO_TRAFFIC
			{
				if {$outfileName != ""} { 
					puts $outfile "idle_chk. Error if no traffic (level 2)  : $fail_msg"
				} else {
					puts "idle_chk. Error if no traffic (level 2)  : $fail_msg"
				}
				incr errors							
			}
			IDLE_CHK_WARNING
			{
				if {$outfileName != ""} {
					puts $outfile "idle_chk. Warning (level 3): $fail_msg" 
				} else {
					puts "idle_chk. Warning (level 3): $fail_msg" 
				}
				
				incr warnings				
			}
			
			default
			{
				if {$outfileName != ""} {
					puts $outfile "Unknown severity level"
					puts $outfile "severity=$severity fail_msg=$fail_msg"
				} else {
					puts "Unknown severity level"
					puts "severity=$severity fail_msg=$fail_msg"
				}
			}
		} 
	}
}

# Check if test should be performed on this chip, according to chip_mask
proc CHIP_MASK_CHK {chip_mask} {		
	set test_chip 0
	set cur_chip_mask 0
	
	if {[is_e4_BigBear]} {
		if {[is_A0]} {
			set cur_chip_mask 0x1  
		} elseif {[is_B0]} {
			set cur_chip_mask 0x4
		}
	} elseif {[is_e4_K2]} {
		set cur_chip_mask 0x10
	}

	if {$cur_chip_mask == 0} {
		set chip_num [reg_read -n MISCS_REG_CHIP_NUM]
		set chip_rev [reg_read -n MISCS_REG_CHIP_REV]
		puts "Unknown chip_num ($chip_num) and/or chip_rev (chip_rev)"
	}

	if {[is_sim]} {
		set cur_chip_mask [expr $cur_chip_mask << 1]
	}
		
    if {$cur_chip_mask & $chip_mask} {
		set test_chip 1
	}               
        
	return $test_chip
}


# Check if this register's block is in reset
proc IS_BLOCK_IN_RESET {{reg}} {
	
	set block_name [lindex [split $reg "_"] 0]
	
	if {$block_name == "PGLUE"} {set block_name "PGLUE_B"}
		
	if {[is_block_in_reset $block_name]} {
		puts "Warning: block $block_name is in reset state, skipping this idle_chk.."
		return 1
	} else {
		return 0
	}
}

# IDLE_CHK_1: read one reg and check the condition
proc IDLE_CHK_1 {{chip_mask} {reg} {condition} {severity} {fail_msg}} {
	
	global dumpMode
	global outfileName
	global outfile
	global total
	global errors
	
	# Run this IDLE_CHK incase chip_mask matchs chip number, otherwise return
	if {![CHIP_MASK_CHK $chip_mask]} {
		return
	}
	
	# Don't check if block is in reset
	if {[IS_BLOCK_IN_RESET $reg]} {
		return
	}
	
	#Incase <reg> includes register+offset split it into <reg> and <offset>
	if {[lsearch $reg "+"] != -1} {
		set offset [lindex [split $reg "+"] 1]
		set reg [lindex [split $reg "+"] 0]
		#remove reduandant space and brackets
		regsub { } $reg {} reg		
	} else {
		set offset 0
	}
		
	#Incase of offset value e.g. 0x2114, read directly from offset
	if {[lsearch $reg "0x*"] != -1} {
		set start_offset $reg		
	} else {
		#otherwise get the register offset using reg_offset utility
		#incase of offset range (xxx-yyy), use the first offset
		if {[reg_offset -n $reg] == -1} {
			incr total
			incr errors
			return
		}
		set start_offset [lindex [split [reg_offset -n $reg] "-"] 0]		
	}	
	set val [reg read [expr ($start_offset + $offset)]]
	
	# Incase of dumpMode print the register, offset and value without condition check
	if {$dumpMode && ($outfileName != "")} {                
		puts $outfile "idle_chk_1 dump $reg: offset = [format "0x%x" [expr ($start_offset+$offset)]] val = [format "0x%x" $val]"		
		return
	}
	
	CONDITION_CHK $val 0 0 $condition $severity "$fail_msg, Value is [format 0x%x $val]"
}
	

# IDLE_CHK_2: loop to read one reg and check the condition 
proc IDLE_CHK_2 {{chip_mask} {reg} {loop} {inc} {condition} {severity} {fail_msg}} {
	
	global dumpMode
	global outfileName
	global outfile
	global total
	global errors
	
	
	# Run this IDLE_CHK incase chip_mask matchs chip number, otherwise return
	if {![CHIP_MASK_CHK $chip_mask]} {
		return
	}
	
	# Don't check if block is in reset
	if {[IS_BLOCK_IN_RESET $reg]} {
		return
	}
	
	#Incase <reg> includes register+offset split it into <reg> and <offset>
	if {[lsearch $reg "+"] != -1} {
		set offset [lindex [split $reg "+"] 1]
		set reg [lindex [split $reg "+"] 0]
		#remove reduandant space
		regsub { } $reg {} reg
	} else {
		set offset 0
	}
	
	#Incase of offset value e.g. 0x2114, read directly from offset
	if {[lsearch $reg "0x*"] != -1} {		
		set start_offset $reg
	} else {
		#Other wise use the utility reg_read -n <register_name>
		if {[reg_offset -n $reg] == -1} {
			incr total
			incr errors
			return
		}
		set start_offset [lindex [split [reg_offset -n $reg] "-"] 0]		
	}	
	
	incr start_offset $offset
	for {set i 0} {$i < $loop} {incr i} {		 
		set val [reg read [expr ($start_offset + $i*$inc)]]
		
		# Incase of dumpMode print the register, offset and value without condition check
		if {$dumpMode && ($outfileName != "")} {
			puts $outfile "idle_chk_2 dump $reg: offset = [format "0x%x" [expr ($start_offset + $i*$inc)]] val = [format "0x%x" $val]"			
		} else {
			CONDITION_CHK $val 0 0 $condition $severity "$reg+$i*$inc $fail_msg, Value is [format 0x%x $val]"
		}
	} 
}

# IDLE_CHK_3: read two regs and check the condition
proc IDLE_CHK_3 {{chip_mask} {reg1} {reg2} {condition} {severity} {fail_msg}} {
	
	global dumpMode
	global outfileName
	global outfile
	global total
	global errors
	
	# Run this IDLE_CHK incase chip_mask matchs chip number, otherwise return
	if {![CHIP_MASK_CHK $chip_mask]} {
		return
	}
	
	# Don't check if block is in reset
	if {[IS_BLOCK_IN_RESET $reg1] || [IS_BLOCK_IN_RESET $reg2]} {
		return
	}
	if {([reg_offset -n $reg1] == -1) || ([reg_offset -n $reg2] == -1) } {
			incr total
			incr errors
			return
	}
	
	#Incase <reg> includes register+offset split it into <reg> and <offset>
	if {[lsearch $reg1 "+"] != -1} {		
		set reg1 [lindex [split $reg1 "+"] 0]
		set offset1 [lindex [split $reg1 "+"] 1]
		#remove reduandant space
		regsub { } $reg1 {} reg
	
		set offset1 [expr {[reg_offset -n $reg1] + $offset1}]		
		
	} else {
		set offset1 [reg_offset -n $reg1]
	}
	
	if {[lsearch $reg2 "+"] != -1} {		
		set reg2 [lindex [split $reg2 "+"] 0]
		set offset2 [lindex [split $reg2 "+"] 1]		
		#remove reduandant space
		regsub { } $reg2 {} reg
		set offset2 [expr {[reg_offset -n $reg2] + $offset2}]				
	} else {
		set offset2 [reg_offset -n $reg2]	
		
	}
	
	set val1 [reg read $offset1]	
	set val2 [reg read $offset2]
	
	# Incase of dumpMode print the register, offset and value without condition check
	if {$dumpMode && ($outfileName != "")} {
		puts $outfile  "idle_chk_3 dump $reg1: offset = [format "0x%x" $offset1] val1 = [format "0x%x" $val1]"
		puts $outfile  "idle_chk_3 dump $reg2: offset = [format "0x%x" $offset2] val2 = [format "0x%x" $val2]"
		return
	}
	CONDITION_CHK 0 $val1 $val2 $condition $severity "$fail_msg, Values are [format 0x%x $val1] [format 0x%x $val2]"
}

# IDLE_CHK_4: read one reg and check according to CID_CAM
proc IDLE_CHK_4 {{chip_mask} {reg1} {reg2} {inc} {condition} {severity} {fail_msg}} {
	
	global dumpMode
	global outfileName
	global outfile
	global total
	global errors
	
	# Run this IDLE_CHK incase chip_mask matchs chip number, otherwise return
	if {![CHIP_MASK_CHK $chip_mask]} {
		return
	}
	
	# Don't check if block is in reset
	if {[IS_BLOCK_IN_RESET $reg1] || [IS_BLOCK_IN_RESET $reg2]} {
		return
	}
	
	if  {([reg_offset -n $reg1] == -1) || [reg_offset -n $reg2] == -1 } {
		incr total
		incr errors
		return		
	}
	set offset1 [lindex [split [reg_offset -n $reg1] "-"] 0]
	set offset2 [lindex [split [reg_offset -n $reg2] "-"] 0]	
			
	# Get the loop size
	set count [reg_count -n $reg1]
	set count [expr ($count >> 2)]
	
	#debug print
	#puts "IDLE_CHK_4 count=$count offset1=[format "0x%x" $offset1] offset2=[format "0x%x" $offset2]"
	
	for {set i 0} {$i < $count} {incr i} {				
		set val1 [reg read [expr ($offset1 + $i*$inc)]]
		set val2 [expr [reg read [expr ($offset2 + $i*$inc)]] >> 1]
		
		# Incase of dumpMode print the register, offset and value without condition check
		if {$dumpMode && ($outfileName != "")} {
			puts $outfile "idle_chk_4 dump $reg1: offset = [format "0x%x" [expr ($offset1 + $i*$inc)]] val = [format "0x%x" $val1]"
			puts $outfile "idle_chk_4 dump $reg2: offset = [format "0x%x" [expr ($offset2 + $i*$inc)]] val = [format "0x%x" $val2]"			
		} else {
			CONDITION_CHK 0 $val1 $val2 $condition $severity "$fail_msg - LCID $i CID_CAM [format 0x%x $val2] Value is [format 0x%x $val1]"
		}
	}
}


# IDLE_CHK_5: read one reg and check according to another reg */
proc IDLE_CHK_5 {{chip_mask} {reg} {reg1} {reg2} {condition} {severity} {fail_msg}} {
	
	global dumpMode
	global outfileName
	global outfile
	
	# Run this IDLE_CHK incase chip_mask matchs chip number, otherwise return
	if {![CHIP_MASK_CHK $chip_mask]} {
		return
	}
	
	# Don't check if block is in reset
	if {[IS_BLOCK_IN_RESET $reg1] || [IS_BLOCK_IN_RESET $reg2] || [IS_BLOCK_IN_RESET $reg]} {
		return
	}
	
	# Incase of dumpMode print the register, offset and value without condition check
	if {$dumpMode && ($outfileName != "")} {
		puts $outfile "idle_chk_5 dump $reg:  offset = [format "0x%x" [reg_offset -n $reg]]  val =  [format "0x%x" [reg_read -n $reg]]"
		puts $outfile "idle_chk_5 dump $reg1: offset = [format "0x%x" [reg_offset -n $reg1]] val1 = [format "0x%x" [reg_read -n $reg1]]"
		puts $outfile "idle_chk_5 dump $reg2: offset = [format "0x%x" [reg_offset -n $reg2]] val2 = [format "0x%x" [reg_read -n $reg2]]"			
	} else {
		if {![reg_read -n $reg]} {
			IDLE_CHK_3 $chip_mask $reg1 $reg2 $condition $severity $fail_msg
		}
	}
}

# IDLE_CHK_6: read wide-bus reg and check sub-fields
proc IDLE_CHK_6 {{chip_mask} {reg} {loop} {inc} {severity}} {
	
	global dumpMode
	global outfileName
	global outfile
	global total
	global errors
	
	# Run this IDLE_CHK incase chip_mask matchs chip number, otherwise return
	if {![CHIP_MASK_CHK $chip_mask]} {
		return
	}
	
	# Don't check if block is in reset
	if {[IS_BLOCK_IN_RESET $reg]} {
		return
	}
	
	if {[reg_offset -n $reg] == -1} {
		incr total
		incr errors
		return
	}
	set offset [lindex [split [reg_offset -n $reg] "-"] 0]	
	for {set i 0} {$i < $loop} {incr  i} {		    
		set val1 [reg read [expr ($offset + $i*$inc)]]
		set val2 [reg read [expr ($offset + $i*$inc + 4)]]
		#QM_REG_PTRTBL - check that
		#bits[53:30] read pointer == bits[29:6] write pointer
		set rd_ptr [expr ($val1 & 0x3FFFFFC0) >> 6]
		set wr_ptr [expr (((($val1 & 0xC0000000) >> 30) & 0x3) | (($val2 & 0x3FFFFF) << 2))]
		# Incase of dumpMode print the register, offset and value without condition check
		if {$dumpMode && ($outfileName != "")} {
			puts $outfile "idle_chk_6 dump $reg: offset = [format "0x%x" [expr ($offset + $i*$inc)]] val = [format "0x%x" $val1]"
			puts $outfile "idle_chk_6 dump $reg: offset = [format "0x%x" [expr ($offset + $i*$inc + 4)]] val = [format "0x%x" $val2]"						
		} else {
			CONDITION_CHK 0 $rd_ptr $wr_ptr "($rd_ptr != $wr_ptr)" $severity "QM: PTRTBL entry $i, rd_ptr is not equal to wr_ptr. Values are [format 0x%x $rd_ptr] [format 0x%x $wr_ptr]"
		}			
		
		#QM_REG_PTRTBL - check that:
		#bits[5:4] read bank0 == bits[1:0] write bank0
		set rd_bank [expr (($val1 & 0x30) >> 4)]
	        set wr_bank [expr ($val1 & 0x03)]
		if {!$dumpMode } {		
			CONDITION_CHK 0 $rd_bank $wr_bank "($rd_bank != $wr_bank)" $severity "QM: PTRTBL entry $i, rd_bank is not equal to wr_bank. Values are [format 0x%x $rd_bank] [format 0x%x $wr_bank]"
		}			
	} 
}

# IDLE_CHK_7:  Verify activity count value is as expected to this connType
proc IDLE_CHK_7 {{chip_mask} {ac_reg} {cfc_info_reg} {cfc_cid_cam} {inc} {condition} {severity} {fail_msg}} {

	global dumpMode
	global outfileName
	global outfile
	global total
	global errors
	
	# Run this IDLE_CHK incase chip_mask matchs chip number, otherwise return
	if {![CHIP_MASK_CHK $chip_mask]} {
		return
	}
	
	# Don't check if block is in reset
	if {[IS_BLOCK_IN_RESET $ac_reg] || [IS_BLOCK_IN_RESET $cfc_info_reg]} {
		return
	}
	
	if {([reg_offset -n $ac_reg] == -1) ||
	    ([reg_offset -n $cfc_info_reg] == -1) ||
	    ([reg_offset -n $cfc_cid_cam] == -1)} {
		incr total
		incr errors
		return
	}
	
	set ac_offset [lindex [split [reg_offset -n $ac_reg] "-"] 0]
	set cfc_info_offset [lindex [split [reg_offset -n $cfc_info_reg] "-"] 0]
	set cfc_cid_cam_offset [lindex [split [reg_offset -n $cfc_cid_cam] "-"] 0]	
	
	# Get the loop size
	set count [reg_count -n $cfc_info_reg]
	set count [expr {$count >> 4}]
	
	#debug print
	#puts "IDLE_CHK_7 count=$count offset1=[format "0x%x" $offset1] offset2=[format "0x%x" $offset2]"
	
	for {set i 0} {$i < $count} {incr  i} {
		# For dump mode only
		if {$dumpMode && ($outfileName != "")} {
			set tmp_offset [expr {$ac_offset + $i*4}]
			puts $outfile "idle_chk_7 dump $ac_reg: offset = [format "0x%x" $tmp_offset] val = [format "0x%x" [reg read $tmp_offset]]"
			set tmp_offset [expr {$cfc_info_offset + $i*$inc}]
			puts $outfile "idle_chk_7 dump $cfc_info_reg: offset = [format "0x%x" $tmp_offset] val = [format "0x%x" [reg read $tmp_offset]]"
			set tmp_offset [expr {$cfc_info_offset + $i*$inc + 4}]
			puts $outfile "idle_chk_7 dump $cfc_info_reg: offset = [format "0x%x" $tmp_offset] val = [format "0x%x" [reg read $tmp_offset]]"
			set tmp_offset [expr {$cfc_info_offset + $i*$inc + 8}]
			puts $outfile "idle_chk_7 dump $cfc_info_reg: offset = [format "0x%x" $tmp_offset] val = [format "0x%x" [reg read $tmp_offset]]"			
		}
		
		# read AC value
		# val2 holds the AC value
		set val2 [reg read [expr {$ac_offset + $i*4}]]		
		
		set chip_num [reg_read -n MISC_REG_CHIP_NUM]   
		# in E2 the CFC_REG_INFO_RAM was changed to 128 bits width
		# and the connection type location was changed to bits 92-89
		if {($chip_num == 0x1662) || ($chip_num == 0x1663) || ($chip_num == 0x1651) || ($chip_num == 0x1652)} {			
			set tmp_val [reg read [expr {$cfc_info_offset +$i*$inc}]]									
			set tmp_val [reg read [expr {$cfc_info_offset +$i*$inc + 4}]]			
			set tmp_val [reg read [expr {$cfc_info_offset + $i*$inc + 8}]]
			reg read [expr {$cfc_info_offset + $i*$inc + 12}]
			# val1 holds the connn_type value
			set val1 [expr {$tmp_val & 0x1E000000} >> 25]			
		} else {			
			set tmp_val [reg read [expr {$cfc_info_offset +$i*$inc}]]									
			set tmp_val [reg read [expr {$cfc_info_offset +$i*$inc + 4}]]			
			set tmp_val [reg read [expr {$cfc_info_offset + $i*$inc + 8}]]
			reg read [expr {$cfc_info_offset + $i*$inc + 12}]
			# val1 holds the connn_type value (bits 70-67 in CFC_REG_INFO_RAM)
			set val1 [expr {$tmp_val & 0x00000078} >> 3]
		}
		
		# read CID_CAM of this lcid entry
		# val holds the cid_cam value
		set val [reg read [expr {$cfc_cid_cam_offset + $i*4}]]		
				
		if {!$dumpMode } {		
			# check AC only if CID_CAM bit 0 is valid
			if {[expr {$val & 0x1}] == 0x1} {
				CONDITION_CHK $val $val1 $val2 $condition $severity "$fail_msg. Activity counter value is [format 0x%x $val2]. LCID $i CID_CAM $val"								
			}				
		}		
	}
}

proc get_vq_hoq {vq} {
    global dumpMode
    global outfileName
    global outfile
    # Only supported since E2
    set chip_num [reg_read -n MISC_REG_CHIP_NUM]
    if {[is_e1x]} {
        return "-"
    }

    # read HOQ at specified VQ
    reg_write -n PXP2_REG_RQ_HOQ_RAM_RD_REQ -v $vq
    for {set i 0} {$i < 200} {incr i} {
        if {[reg_read -n PXP2_REG_RQ_HOQ_RAM_RD_STATUS]} break
    }
    if {($i == 200) && !$dumpMode} {
        return "ERROR: Timeout while reading HOQ at VQ $vq"
    }
    set clients {"USDM" "CSDM" "XSDM" "TSDM" "PBF" "QM" "Timers" "Searcher" "CDU_RD" "DMAE" "USDM_DP" "HC" "CDU_WR" "DBG" "_INVALID_" "_INVALID_"}
    set rd0 [reg_read -n PXP2_REG_RQ_HOQ_RAM_DATA_RD_0]
    set rd1 [reg_read -n PXP2_REG_RQ_HOQ_RAM_DATA_RD_1]
    set rd2 [reg_read -n PXP2_REG_RQ_HOQ_RAM_DATA_RD_2]
    set rd3 [reg_read -n PXP2_REG_RQ_HOQ_RAM_DATA_RD_3]
    set read_bit [lsearch {1 2 3 5 6 7 8 9 10 11 28} [expr $vq]]
    set is_write [expr (1 << $vq) & 0x60200000]
    if {!$is_write && ($read_bit == -1 || ([reg_read -n PXP2_REG_RQ_CMG_VQ_RDWR_N] & (1 << $read_bit)))} {
        set rq_type "READ"
    } else {
        set rq_type "WRITE"
    }

    # Incase of dumpMode print the register values
    if {$dumpMode && ($outfileName != "")} {                
	puts $outfile "idle_chk_8 dump PXP2_REG_RQ_HOQ_RAM_DATA_RD_0: val = [format "0x%x" $rd0]"
	puts $outfile "idle_chk_8 dump PXP2_REG_RQ_HOQ_RAM_DATA_RD_1: val = [format "0x%x" $rd1]"
	puts $outfile "idle_chk_8 dump PXP2_REG_RQ_HOQ_RAM_DATA_RD_2: val = [format "0x%x" $rd2]"
	puts $outfile "idle_chk_8 dump PXP2_REG_RQ_HOQ_RAM_DATA_RD_3: val = [format "0x%x" $rd3]"		
    }

    set rq_id [expr $rd0 >> 16]
    set len [expr $rd0 & 0xffff]
    set addr_lo $rd1
    set addr_hi $rd2
    set flags [lindex {"None" "RO" "NS" "RO+NS"} [expr $rd3 & 3]]
    set client [lindex $clients [expr ($rd3 >> 2) & 0xf]]
    set done [expr ($rd3 >> 6) & 1]
    set pfid [expr ($rd3 >> 8) & 7]
    set vfvalid [expr ($rd3 >> 11) & 1]
    set vfid [expr ($rd3 >> 12) & 0x3f]
    set atc [expr ($rd3 >> 18) & 0x7]
    if {$vfvalid} {
        set func [format "VFID %d (PF %d)" $vfid $pfid]
    } else {
        set func [format "PF %d" $pfid]
    }

    return [format "  Request at VQ%d is %s %d bytes to/from address 0x%x_%08x by %s: rq_id=%d done=%d func=%s atc=%d flags=%s." \
        $vq $rq_type $len $addr_hi $addr_lo $client $rq_id $done $func $atc $flags]

}

# IDLE_CHK_8: check PXP VQ occupancy according to condition
proc IDLE_CHK_8 {{chip_mask} {reg} {condition} {severity} {fail_msg}} {
	
	global dumpMode
	global outfileName
	global outfile
	global total
	global errors
	
	# Run this IDLE_CHK incase chip_mask matchs chip number, otherwise return
	if {![CHIP_MASK_CHK $chip_mask]} {
		return
	}
	
	# Don't check if block is in reset
	if {[IS_BLOCK_IN_RESET $reg]} {
		return
	}
	
	#Incase <reg> includes register+offset split it into <reg> and <offset>
	if {[lsearch $reg "+"] != -1} {
		set offset [lindex [split $reg "+"] 1]
		set reg [lindex [split $reg "+"] 0]
		#remove reduandant space and brackets
		regsub { } $reg {} reg		
	} else {
		set offset 0
	}
		
	#Incase of offset value e.g. 0x2114, read directly from offset
	if {[lsearch $reg "0x*"] != -1} {
		set start_offset $reg		
	} else {
		#otherwise get the register offset using reg_offset utility
		#incase of offset range (xxx-yyy), use the first offset
		if {[reg_offset -n $reg] == -1} {
			incr total
			incr errors
			return
		}
		set start_offset [lindex [split [reg_offset -n $reg] "-"] 0]
	}	
	set val [reg read [expr ($start_offset + $offset)]]
	
	# Incase of dumpMode print the register, offset and value without condition check
	if {$dumpMode && ($outfileName != "")} {                
		puts $outfile "idle_chk_8 dump $reg: offset = [format "0x%x" [expr ($start_offset+$offset)]] val = [format "0x%x" $val]"		
	}

	if {(([expr $condition]) || ($dumpMode && ($outfileName != "")))} {
	    if {[scan $reg "PXP2_REG_RQ_VQ%d_ENTRY_CNT" vq] == 1} {
	        set msg [get_vq_hoq $vq]
	    } else {
	        set msg "  < ERROR: cannot extract VQ number from register $reg >"
	    }
	    if {$dumpMode && ($outfileName != "")} {
			return
	    }
            CONDITION_CHK 0 0 0 1 $severity "$fail_msg, Value is [format 0x%x $val]\n$msg"
        }
}

proc dump_vqs {} {
    for {set i 0} {$i < 32} {incr i} {
        set msg [get_vq_hoq $i]
        if {[string first "r/w 0 bytes" $msg] == -1} {
            puts $msg
        }
    }
}


# IDLE_CHK_9: Compare two memories
proc IDLE_CHK_9 {{chip_mask} {reg1} {reg2} {condition} {severity} {fail_msg}} {
	
	global dumpMode
	global outfileName
	global outfile
	global total
	global errors
	
	# Run this IDLE_CHK incase chip_mask matchs chip number, otherwise return
	if {![CHIP_MASK_CHK $chip_mask]} {
		return
	}
	
	# Don't check if block is in reset
	if {[IS_BLOCK_IN_RESET $reg1] || [IS_BLOCK_IN_RESET $reg2]} {
		return
	}
	
	if {([reg_offset -n $reg1] == -1) ||
	    ([reg_offset -n $reg2] == -1)} {
		incr total
		incr errors
		return
	}
	
	set reg1_start_offset [lindex [split [reg_offset -n $reg1] "-"] 0]
	set reg1_end_offset   [lindex [split [reg_offset -n $reg1] "-"] 1]
	
	set reg2_start_offset [lindex [split [reg_offset -n $reg2] "-"] 0]
	set reg2_end_offset   [lindex [split [reg_offset -n $reg2] "-"] 1]
	
	set offset1 $reg1_start_offset
	set offset2 $reg2_start_offset
	set i 0
	
	for {set offset1 $reg1_start_offset} {$offset1 < $reg1_end_offset} {incr i} {
		
		set val1 [reg read $offset1]		
		set val2 [reg read $offset2]
		
		# Incase of dumpMode print the register, offset and value without condition check
		if {$dumpMode && ($outfileName != "")} {
			puts $outfile  "idle_chk_9 dump $reg1: offset = [format "0x%x" $offset1] val1 = [format "0x%x" $val1]"
			puts $outfile  "idle_chk_9 dump $reg2: offset = [format "0x%x" $offset2] val2 = [format "0x%x" $val2]"			
		} else {
			CONDITION_CHK 0 $val1 $val2 $condition $severity "$fail_msg, Values are [format 0x%x $val1] [format 0x%x $val2]"
		}
		
		set offset1 [expr {$offset1 + $i*4}]
		set offset2 [expr {$offset2 + $i*4}]
	}
}

# IDLE_CHK_10: Verify register is in it's reset value
proc IDLE_CHK_10 {{chip_mask} {reg} {condition} {severity} {fail_msg}} {
	
	global dumpMode
	global outfileName
	global outfile
	global total
	global errors
	
	# Run this IDLE_CHK incase chip_mask matchs chip number, otherwise return
	if {![CHIP_MASK_CHK $chip_mask]} {
		return
	}
	
	# Don't check if block is in reset
	if {[IS_BLOCK_IN_RESET $reg]} {
		return
	}
	
	if {[reg_offset -n $reg] == -1} {
		incr total
		incr errors
		return
	}
	
	set val1 [reset_value -n $reg]
	
	set start_offset [lindex [split [reg_offset -n $reg] "-"] 0]
	set val2 [reg read $start_offset]
	
	# Incase of dumpMode print the register, offset and value without condition check
	if {$dumpMode && ($outfileName != "")} {                
		puts $outfile "idle_chk_10 dump $reg: offset = [format "0x%x" $start_offset] val = [format "0x%x" $val2]"		
		return
	}
	
	CONDITION_CHK  0 $val1 $val2 $condition $severity "$fail_msg, Value is [format 0x%x $val2], Reset Value is [format 0x%x $val1]\n"	
}

# IDLE_CHK_11: Verify memory registers are in their expected value
proc IDLE_CHK_11 {{chip_mask} {reg} {condition} {severity} {fail_msg}} {
	
	global dumpMode
	global outfileName
	global outfile
	global total
	global errors
	
	# Run this IDLE_CHK incase chip_mask matchs chip number, otherwise return
	if {![CHIP_MASK_CHK $chip_mask]} {
		return
	}
	
	# Don't check if block is in reset
	if {[IS_BLOCK_IN_RESET $reg]} {
		return
	}
	
	if {[reg_offset -n $reg] == -1} {
		incr total
		incr errors
		return
	}
	
	set start_offset [lindex [split [reg_offset -n $reg] "-"] 0]
	set end_offset   [lindex [split [reg_offset -n $reg] "-"] 1]	
	
	set i 0
	
	for {set offset $start_offset} {$offset < $end_offset} {incr i} {
		
		set val [reg read $offset]				
		
		# Incase of dumpMode print the register, offset and value without condition check
		if {$dumpMode && ($outfileName != "")} {
			puts $outfile  "idle_chk_11 dump $reg: offset = [format "0x%x" $offset] val = [format "0x%x" $val]"					
		} else {
			CONDITION_CHK $val 0 0 $condition $severity "$fail_msg, Value is [format 0x%x $val]"
		}
		
		set offset [expr {$offset + $i*4}]		
	}	
}


proc oldIdleChk {args} {
	global errors
	global warnings
	global total
	global dumpMode
	global outfileName
	global outfile
	set errors 0
	set warnings 0
	set noWarning 0
	set ignoreAssert 1
	set total 0
	set dumpMode 0
	set outfileName ""

	# Parse the options
	for {set i 0} {$i < [llength $args]} {incr i} {
		switch -- [lindex $args $i] {
		    "-ignoreAssert"  { set ignoreAssert 1 }
		    "-noWarn"        { set noWarning 1}
		    "-out"           { incr i; set outfileName [lindex $args $i]}
		    "-dumpMode"      { set dumpMode 1}
		}
	}
	if {$outfileName != ""} {
		set outfile [open $outfileName a]
	}
	if {$dumpMode && ($outfileName == "" )} {
		puts "Usage: -dumpMode -out filename "
		return 0
	}

	# Parse the RDF file
	parse_rdf
	# Read register 0x2104 val and check if condition on val exist
	IDLE_CHK_1 0x3f 0x2104 "((\$val & 0x0FD010) != 0)" IDLE_CHK_ERROR "PCIE: ucorr_err_status is not 0"
	# Read register 0x2104 val and check if condition on val exist
	IDLE_CHK_1 0x3f 0x2104 "((\$val & 0x100000) != 0)" IDLE_CHK_WARNING "PCIE: ucorr_err_status - Unsupported request error"
	# Read register 0x2104 val and check if condition on val exist
	IDLE_CHK_1 0x3f 0x2104 "((\$val & 0x2000) != 0)" IDLE_CHK_WARNING "PCIE: ucorr_err_status - Flow Control Protocol Error"
	# Read register 0x2814 val and check if condition on val exist
	IDLE_CHK_1 0x3f 0x2814 "((\$val & 0x2000000) != 0)" IDLE_CHK_WARNING "PCIE: TTX_BRIDGE_FORWARD_ERR - Received master request while BME was 0."
	# Read register 0x2814 val and check if condition on val exist
	IDLE_CHK_1 0x3f 0x2814 "((\$val & ~0x2040902) != 0)" IDLE_CHK_ERROR "PCIE: Func 0 1: attentions register is not 0x2040902"
	# Read register 0x2854 val and check if condition on val exist
	IDLE_CHK_1 0x3f 0x2854 "((\$val & ~0x10240902) != 0)" IDLE_CHK_ERROR "PCIE: Func 2 3 4: attentions register is not 0x10240902"
	# Read register 0x285c val and check if condition on val exist
	IDLE_CHK_1 0x3f 0x285c "((\$val & ~0x10240902) != 0)" IDLE_CHK_ERROR "PCIE: Func 5 6 7: attentions register is not 0x10240902"
	# Read register 0x3040 val and check if condition on val exist
	IDLE_CHK_1 0x3f 0x3040 "((\$val & 0x2) != 0)" IDLE_CHK_ERROR "PCIE: Overflow in DLP2TLP buffer"
	# Read register 0x211c val in loop (incr by 4) and check if condition on val exist
	IDLE_CHK_2 0x3f 0x211c 4 4 "((\$val != 0) && (\$errors > 0))" IDLE_CHK_WARNING "PCIE: error packet header is not 0"
	# Read register PGLUE_B_REG_TAGS_31_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_TAGS_31_0 "((\$val != 0xffffffff))" IDLE_CHK_WARNING "PGLUE_B: There are outstanding read requests for tags 0-31. Not all completios have arrived for read requests on tags that are marked with 0"
	# Read register PGLUE_B_REG_TAGS_63_32 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_TAGS_63_32 "((\$val != 0xffffffff))" IDLE_CHK_WARNING "PGLUE_B: There are outstanding read requests for tags 32-63. Not all completios have arrived for read requests on tags that are marked with 0"
	# Read register PGLUE_B_REG_TAGS_95_64 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_TAGS_95_64 "((\$val != 0xffffffff))" IDLE_CHK_WARNING "PGLUE_B: There are outstanding read requests for tags 64-95. Not all completios have arrived for read requests on tags that are marked with 0"
	# Read register PGLUE_B_REG_TAGS_127_96 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_TAGS_127_96 "((\$val != 0xffffffff))" IDLE_CHK_WARNING "PGLUE_B: There are outstanding read requests for tags 96-127. Not all completios have arrived for read requests on tags that are marked with 0"
	# Read register PGLUE_B_REG_TAGS_159_128 val and check if condition on val exist
	IDLE_CHK_1 0xf PGLUE_B_REG_TAGS_159_128 "((\$val != 0xffffffff))" IDLE_CHK_WARNING "PGLUE_B: There are outstanding read requests for tags 128-159. Not all completios have arrived for read requests on tags that are marked with 0"
	# Read register PGLUE_B_REG_TAGS_191_160 val and check if condition on val exist
	IDLE_CHK_1 0xf PGLUE_B_REG_TAGS_191_160 "((\$val != 0xffffffff))" IDLE_CHK_WARNING "PGLUE_B: There are outstanding read requests for tags 160-191. Not all completios have arrived for read requests on tags that are marked with 0"
	# Read register PGLUE_B_REG_TAGS_223_192 val and check if condition on val exist
	IDLE_CHK_1 0xf PGLUE_B_REG_TAGS_223_192 "((\$val != 0xffffffff))" IDLE_CHK_WARNING "PGLUE_B: There are outstanding read requests for tags 192-223. Not all completios have arrived for read requests on tags that are marked with 0"
	# Read register PGLUE_B_REG_TAGS_255_224 val and check if condition on val exist
	IDLE_CHK_1 0xf PGLUE_B_REG_TAGS_255_224 "((\$val != 0xffffffff))" IDLE_CHK_WARNING "PGLUE_B: There are outstanding read requests for tags 224-255. Not all completios have arrived for read requests on tags that are marked with 0"
	# Read register PGLUE_B_REG_INCORRECT_RCV_DETAILS val and check if condition on val exist
	IDLE_CHK_1 0xf PGLUE_B_REG_INCORRECT_RCV_DETAILS "(\$val != 0)" IDLE_CHK_ERROR "PGLUE_B: Packet received from PCIe not according to the rules."
	# Read register PGLUE_B_REG_WAS_ERROR_VF_31_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_WAS_ERROR_VF_31_0 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: was_error for VFs 0-31 is not 0"
	# Read register PGLUE_B_REG_WAS_ERROR_VF_63_32 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_WAS_ERROR_VF_63_32 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: was_error for VFs 32-63 is not 0"
	# Read register PGLUE_B_REG_WAS_ERROR_VF_95_64 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_WAS_ERROR_VF_95_64 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: was_error for VFs 64-95 is not 0"
	# Read register PGLUE_B_REG_WAS_ERROR_VF_127_96 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_WAS_ERROR_VF_127_96 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: was_error for VFs 96-127 is not 0"
	# Read register PGLUE_B_REG_WAS_ERROR_VF_159_128 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_WAS_ERROR_VF_159_128 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: was_error for VFs 128-159 is not 0"
	# Read register PGLUE_B_REG_WAS_ERROR_VF_191_160 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_WAS_ERROR_VF_191_160 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: was_error for VFs 160-191 is not 0"
	# Read register PGLUE_B_REG_WAS_ERROR_VF_223_192 val and check if condition on val exist
	IDLE_CHK_1 0xf PGLUE_B_REG_WAS_ERROR_VF_223_192 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: was_error for VFs 192-223 is not 0"
	# Read register PGLUE_B_REG_WAS_ERROR_VF_255_224 val and check if condition on val exist
	IDLE_CHK_1 0xf PGLUE_B_REG_WAS_ERROR_VF_255_224 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: was_error for VFs 224-255 is not 0"
	# Read register PGLUE_B_REG_WAS_ERROR_PF_31_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_WAS_ERROR_PF_31_0 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: was_error for PFs 0-15 is not 0"
	# Read register PGLUE_B_REG_RX_ERR_DETAILS val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_RX_ERR_DETAILS "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Completion received with error. (3:0) - PFID. (4) - VF_VALID. (12:5) - VFID. (14:13) - Error code : 0 - Completion Timeout ; 1 - Unsupported Request; 2 - Completer Abort. (15) - valid bit."
	# Read register PGLUE_B_REG_RX_TCPL_ERR_DETAILS val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_RX_TCPL_ERR_DETAILS "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: ATS TCPL received with error. (3:0) - PFID. (4) - VF_VALID. (12:5) - VFID. (14:13) - Error code : 0 - Completion Timeout ; 1 - Unsupported Request; 2 - Completer Abort. (19:15) - OTB Entry ID. (20) - valid bit."
	# Read register PGLUE_B_REG_TX_ERR_WR_ADD_31_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_TX_ERR_WR_ADD_31_0 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Error in master write. Address(31:0) is not 0"
	# Read register PGLUE_B_REG_TX_ERR_WR_ADD_63_32 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_TX_ERR_WR_ADD_63_32 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Error in master write. Address(63:32) is not 0"
	# Read register PGLUE_B_REG_TX_ERR_WR_DETAILS val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_TX_ERR_WR_DETAILS "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Error in master write. Error details register is not 0. (4:0) VQID. (17:5) - Length in bytes. (19) - VF_VALID. (23:20) - PFID. (31:24) - VFID."
	# Read register PGLUE_B_REG_TX_ERR_WR_DETAILS2 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_TX_ERR_WR_DETAILS2 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Error in master write. Error details 2nd register is not 0.  (15:0) Request ID. (20:16) client ID. (21) - was_error set; (22) - BME cleared; (23) - FID_enable cleared; (24) - VF with parent PF FLR_request or IOV_disable_request. (25) - last SR. (26) - valid bit."
	# Read register PGLUE_B_REG_TX_ERR_RD_ADD_31_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_TX_ERR_RD_ADD_31_0 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE: Error in master read address(31:0) is not 0"
	# Read register PGLUE_B_REG_TX_ERR_RD_ADD_63_32 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_TX_ERR_RD_ADD_63_32 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Error in master read address(63:32) is not 0"
	# Read register PGLUE_B_REG_TX_ERR_RD_DETAILS val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_TX_ERR_RD_DETAILS "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Error in master read Error details register is not 0. (4:0) VQID. (5) Is TREQ. (18:6) - Length in bytes. (19) - VF_VALID. (23:20) - PFID. (31:24) - VFID."
	# Read register PGLUE_B_REG_TX_ERR_RD_DETAILS2 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_TX_ERR_RD_DETAILS2 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Error in master read Error details 2nd register is not 0. (15:0) Request ID. (20:16) client ID. (21) - was_error set; (22) - BME cleared; (23) - FID_enable cleared; (24) - VF with parent PF FLR_request or IOV_disable_request. (25) last SR. (26) valid bit."
	# Read register PGLUE_B_REG_VF_LENGTH_VIOLATION_DETAILS val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_VF_LENGTH_VIOLATION_DETAILS "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Target VF length violation access. (12:0) Address in DWs (bits (14:2) of byte address). (14:13) BAR. (22:15) VFID. (26:23) - PFID. "
	# Read register PGLUE_B_REG_VF_LENGTH_VIOLATION_DETAILS2 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_VF_LENGTH_VIOLATION_DETAILS2 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Target VF length violation access. (5:0) - Length in DWs. (6) valid bit. "
	# Read register PGLUE_B_REG_VF_GRC_SPACE_VIOLATION_DETAILS val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_VF_GRC_SPACE_VIOLATION_DETAILS "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Target VF GRC space access failed permission check. (14:0) Address. (15) w_nr: 0 - Read; 1 - Write. (23:16) VFID. (27:24) - PFID. (28) - valid bit."
	# Read register PGLUE_B_REG_PGL_WRITE_BLOCKED val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_PGL_WRITE_BLOCKED "(\$val != 0)" IDLE_CHK_WARNING "PXP2: pgl_write_blocked is not 0"
	# Read register PGLUE_B_REG_PGL_READ_BLOCKED val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_PGL_READ_BLOCKED "(\$val != 0)" IDLE_CHK_WARNING "PXP2: pgl_read_blocked is not 0"
	# Read register PGLUE_B_REG_PGL_TXW_CDTS val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_PGL_TXW_CDTS "(((\$val >> 17) & 1) != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PXP2: There is data which is ready"
	# Read register PGLUE_B_REG_MASTER_ZLR_ERR_ADD_31_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_MASTER_ZLR_ERR_ADD_31_0 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE: Address(31:0) of first master read request with length equals to 0."
	# Read register PGLUE_B_REG_MASTER_ZLR_ERR_ADD_63_32 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_MASTER_ZLR_ERR_ADD_63_32 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE: Address(63:32) of first master read request with length equals to 0."
	# Read register PGLUE_B_REG_MASTER_ZLR_ERR_DETAILS val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_MASTER_ZLR_ERR_DETAILS "(\$val != 0)" IDLE_CHK_WARNING "Details of first read request with length = 0. (4:0) VQID. (5) Is TREQ. (9:6) - PFID. (10) - VF_VALID. (18:11) - VFID. (23:19) client ID. (24) - last SR. (25) valid."
	# Read register PGLUE_B_REG_ADMIN_WINDOW_VIOLATION_DETAILS val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_ADMIN_WINDOW_VIOLATION_DETAILS "(\$val != 0)" IDLE_CHK_WARNING "Details of first target Read Write access to the admin window that have a length bigger than 1DW or first byte enable != 0xf . (9:0) Address in DWs (bits (11:2) of byte address). (13:10) BE first. (17:14) BE last. (21:18) - PFID. (27:22) - Length in DWs. (28) valid bit."
	# Read register PGLUE_B_REG_OUT_OF_RANGE_FUNCTION_IN_PRETEND_ADDRESS val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_OUT_OF_RANGE_FUNCTION_IN_PRETEND_ADDRESS "(\$val != 0)" IDLE_CHK_ERROR "Address of first target Read Write access where pretend register contains an out of range function."
	# Read register PGLUE_B_REG_OUT_OF_RANGE_FUNCTION_IN_PRETEND_DETAILS val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_OUT_OF_RANGE_FUNCTION_IN_PRETEND_DETAILS "(\$val != 0)" IDLE_CHK_ERROR "Details of first target Read/Write access where pretend register contains an out of range function. (3:0) - original PFID. (7:4) Pretend PFID. (15:8) Pretend VFID. (16) Pretend vf_valid. (20:17) Pretend register: 0-11 - One of the PF windows pretend. 12 - global pretend register. (21) valid bit."
	# Read register PGLUE_B_REG_WRITE_FIFO_OCCUPANCY_LEVEL val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_WRITE_FIFO_OCCUPANCY_LEVEL "(\$val != 0)" IDLE_CHK_WARNING "PGL TX Write FIFO is not empty."
	# Read register PGLUE_B_REG_READ_FIFO_OCCUPANCY_LEVEL val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_READ_FIFO_OCCUPANCY_LEVEL "(\$val != 0)" IDLE_CHK_WARNING "PGL TX Read FIFO is not empty."
	# Read register PGLUE_B_REG_ILLEGAL_ADDRESS_ADD_31_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_ILLEGAL_ADDRESS_ADD_31_0 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Illegal address in master flow. Address(31:0)."
	# Read register PGLUE_B_REG_ILLEGAL_ADDRESS_ADD_63_32 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_ILLEGAL_ADDRESS_ADD_63_32 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Illegal address in master flow. Address(63:32)."
	# Read register PGLUE_B_REG_ILLEGAL_ADDRESS_DETAILS val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_ILLEGAL_ADDRESS_DETAILS "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Illegal address in master flow. (4:0) VQID. (5) - first SR. (18:6) - Length in bytes. (19) - VF_VALID. (23:20) - PFID. (31:24) - VFID."
	# Read register PGLUE_B_REG_ILLEGAL_ADDRESS_DETAILS2 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_ILLEGAL_ADDRESS_DETAILS2 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Illegal address in master flow. (15:0) Request ID. (20:16) client ID. (21) Illegal address cause: 0 - address was smaller than minimal_address_log;  1 - address was bigger than maximal_address_log. (22) - write_n_read: 0 - read; 1 - write. (23) last SR. (24) valid bit."
	# Read register PGLUE_B_REG_RX_LEGACY_ERRORS val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_RX_LEGACY_ERRORS "(\$val != 0)" IDLE_CHK_WARNING "Bit 0 - Target memory read with a correctable error. Bit 1 - Target memory read with an uncorrectable error. Bit 2 - Configuration RW with a correctable error. Bit 3 - Configuration RW with an uncorrectable error. Bit 4 - Target memory write or MSGD with a correctable error. Bit 5 - Target memory write or MSGD with an uncorrectable error. Bit 6 - Master completion with a correctable error. Bit 7 - Master completion with an uncorrectable error."
	# Read register PGLUE_B_REG_FLR_REQUEST_VF_31_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_FLR_REQUEST_VF_31_0 "(\$val  != 0)" IDLE_CHK_WARNING "PGL: FLR request is set for VF addresses 31-0"
	# Read register PGLUE_B_REG_FLR_REQUEST_VF_63_32 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_FLR_REQUEST_VF_63_32 "(\$val  != 0)" IDLE_CHK_WARNING "PGL: FLR request is set for VF addresses 63-32"
	# Read register PGLUE_B_REG_FLR_REQUEST_VF_95_64 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_FLR_REQUEST_VF_95_64 "(\$val  != 0)" IDLE_CHK_WARNING "PGL: FLR request is set for VF addresses 95-64"
	# Read register PGLUE_B_REG_FLR_REQUEST_VF_127_96 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_FLR_REQUEST_VF_127_96 "(\$val  != 0)" IDLE_CHK_WARNING "PGL: FLR request is set for VF addresses 127-96"
	# Read register PGLUE_B_REG_FLR_REQUEST_VF_159_128 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_FLR_REQUEST_VF_159_128 "(\$val  != 0)" IDLE_CHK_WARNING "PGL: FLR request is set for VF addresses 159-128"
	# Read register PGLUE_B_REG_FLR_REQUEST_VF_191_160 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_FLR_REQUEST_VF_191_160 "(\$val  != 0)" IDLE_CHK_WARNING "PGL: FLR request is set for VF addresses 191-160"
	# Read register PGLUE_B_REG_FLR_REQUEST_VF_223_192 val and check if condition on val exist
	IDLE_CHK_1 0xf PGLUE_B_REG_FLR_REQUEST_VF_223_192 "(\$val  != 0)" IDLE_CHK_WARNING "PGL: FLR request is set for VF addresses 223-192"
	# Read register PGLUE_B_REG_FLR_REQUEST_VF_255_224 val and check if condition on val exist
	IDLE_CHK_1 0xf PGLUE_B_REG_FLR_REQUEST_VF_255_224 "(\$val  != 0)" IDLE_CHK_WARNING "PGL: FLR request is set for VF addresses 255-224"
	# Read register PGLUE_B_REG_FLR_REQUEST_PF_31_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_FLR_REQUEST_PF_31_0 "(\$val  != 0)" IDLE_CHK_WARNING "PGL: FLR request is set for PF addresses 7-0"
	# Read register PGLUE_B_REG_SR_IOV_DISABLED_REQUEST val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_SR_IOV_DISABLED_REQUEST "(\$val  != 0)" IDLE_CHK_WARNING "PGL: SR-IOV disable request is set "
	# Read register PGLUE_B_REG_CFG_SPACE_A_REQUEST val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_CFG_SPACE_A_REQUEST "(\$val  != 0)" IDLE_CHK_WARNING "PGL: Cfg-Space A request is set"
	# Read register PGLUE_B_REG_CFG_SPACE_B_REQUEST val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_CFG_SPACE_B_REQUEST "(\$val  != 0)" IDLE_CHK_WARNING "PGL: Cfg-Space B request is set"
	# Read register PGLUE_B_REG_VF_ILT_ERR_ADD_31_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_VF_ILT_ERR_ADD_31_0 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Transaction with VF ILT error was dropped. Logical address(31:0)."
	# Read register PGLUE_B_REG_VF_ILT_ERR_ADD_63_32 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_VF_ILT_ERR_ADD_63_32 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Transaction with VF ILT error was dropped. Logical address(63:32)."
	# Read register PGLUE_B_REG_VF_ILT_ERR_DETAILS val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_VF_ILT_ERR_DETAILS "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Transaction with VF ILT error was dropped. (4:0) VQID. (5) - first SR. (18:6) - Length in bytes. (19) - VF_VALID. (23:20) - PFID. (31:24) - VFID."
	# Read register PGLUE_B_REG_VF_ILT_ERR_DETAILS2 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_VF_ILT_ERR_DETAILS2 "(\$val != 0)" IDLE_CHK_WARNING "PGLUE_B: Transaction with VF ILT error was dropped. (15:0) Request ID. (20:16) client ID. (21) write_n_read: 0 - read;  1 - write. (22) last SR. (23) valid bit."
	# Read register PGLUE_B_REG_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_INT_STS "((\$val & ~0x80000) != 0)" IDLE_CHK_WARNING "PGLUE_B: Interrupt status is not 0x80000"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ0_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ0_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ0 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ1_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ1_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ1 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ2_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ2_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ2 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ3_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ3_ENTRY_CNT "(\$val > 2)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ3 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ4_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ4_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ4 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ5_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ5_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ5 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ6_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ6_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ6 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ7_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ7_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ7 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ8_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ8_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ8 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ9_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ9_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ9 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ10_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ10_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ10 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ11_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ11_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ11 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ12_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ12_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ12 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ13_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ13_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ13 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ14_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ14_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ14 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ15_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ15_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ15 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ16_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ16_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ16 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ17_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ17_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ17 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ18_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ18_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ18 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ19_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ19_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ19 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ20_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ20_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ20 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ21_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ21_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ21 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ22_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ22_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ22 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ23_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ23_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ23 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ24_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ24_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ24 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ25_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ25_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ25 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ26_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ26_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ26 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ27_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ27_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ27 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ28_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ28_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ28 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ29_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ29_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ29 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ30_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ30_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ30 is not empty"
	# Check PXP VQ occupancy PSWRQ2_REG_VQ31_ENTRY_CNT and check if condition on val exist
	IDLE_CHK_8 0x3f PSWRQ2_REG_VQ31_ENTRY_CNT "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: VQ31 is not empty"
	# Read register PSWRQ2_REG_RBC_DONE val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWRQ2_REG_RBC_DONE "(\$val != 1)" IDLE_CHK_ERROR "PSWRQ2: rq_rbc_done is not 1"
	# Read register PSWRQ2_REG_CFG_DONE val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWRQ2_REG_CFG_DONE "(\$val != 1)" IDLE_CHK_ERROR "PSWRQ2: rq_cfg_done is not 1"
	# Read register PSWRQ2_REG_L2P_CLOSE_GATE_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWRQ2_REG_L2P_CLOSE_GATE_STS "(\$val != 0)" IDLE_CHK_ERROR "PSWRQ2: L2P close the gate is asseted"
	# Read register PSWRQ2_REG_MISC_CLOSE_GATE_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWRQ2_REG_MISC_CLOSE_GATE_STS "(\$val != 0)" IDLE_CHK_ERROR "PSWRQ2: MISC close the gate is asseted"
	# Read register PSWRQ2_REG_MISC_STALL_MEM_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWRQ2_REG_MISC_STALL_MEM_STS "(\$val != 0)" IDLE_CHK_WARNING "PSWRQ2: MISC stall mem is asseted"
	# Read register PSWRQ2_REG_BW_CREDIT val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWRQ2_REG_BW_CREDIT "(\$val != 0xb7)" IDLE_CHK_ERROR "PSWRQ2: There are used credits on PSWRQ - PGLUE request interface."
	# Read register PSWRQ2_REG_TREQ_FIFO_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWRQ2_REG_TREQ_FIFO_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: TREQ FIFO is not empty"
	# Read register PSWRQ2_REG_ICPL_FIFO_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWRQ2_REG_ICPL_FIFO_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: ICPL FIFO is not empty"
	# Read register PSWRQ2_REG_L2P_ERR_ADD_31_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWRQ2_REG_L2P_ERR_ADD_31_0 "(\$val != 0)" IDLE_CHK_ERROR "PSWRQ: L2P error. Address(31:0) is not 0"
	# Read register PSWRQ2_REG_L2P_ERR_ADD_63_32 val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWRQ2_REG_L2P_ERR_ADD_63_32 "(\$val != 0)" IDLE_CHK_ERROR "PSWRQ: L2P error. Address(63:23) is not 0"
	# Read register PSWRQ2_REG_L2P_ERR_DETAILS val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWRQ2_REG_L2P_ERR_DETAILS "(\$val != 0)" IDLE_CHK_ERROR "PSWRQ: L2P error. Erorr details register is not 0. (12:0) - Length in bytes. (16:13) - PFID. (17) - VF_VALID. (25:18) - VFID. "
	# Read register PSWRQ2_REG_L2P_ERR_DETAILS2 val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWRQ2_REG_L2P_ERR_DETAILS2 "(\$val != 0)" IDLE_CHK_ERROR "PSWRQ: L2P error. Erorr details register 2 is not 0. (15:0) - Request ID. (20:16) - client ID. (21) - Error type - 0 - rq_l2p_vf_err; 1 - rq_elt_addr. (22) - w_nr - 0 - read; 1 - write.(27:23) - VQID. (28) - valid bit. "
	# Read register PSWRQ2_REG_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWRQ2_REG_INT_STS "(\$val != 0)" IDLE_CHK_WARNING "PSWRQ2: Interrupt status is not 0"
	# Read register PSWRQ_REG_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWRQ_REG_INT_STS "(\$val != 0)" IDLE_CHK_WARNING "PSWRQ: Interrupt status is not 0"
	# Read register PSWRQ2_REG_SR_CNT val1 and register PSWRQ2_REG_SR_NUM_CFG val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT PSWRQ2_REG_SR_NUM_CFG "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: sr_cnt is not equal to sr_num_cfg"
	# Read register PSWRQ2_REG_BLK_CNT val1 and register PSWRQ2_REG_BLK_NUM_CFG val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT PSWRQ2_REG_BLK_NUM_CFG "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt is not equal to blk_num_cfg"
	# Read register PSWRQ2_REG_SR_CNT val1 and register PSWRQ2_REG_SR_NUM_CFG val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT PSWRQ2_REG_SR_NUM_CFG "(\$val1 < (\$val2-4))" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: There are more than four used SRs"
	# Read register PSWRQ2_REG_BLK_CNT val1 and register PSWRQ2_REG_BLK_NUM_CFG val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT PSWRQ2_REG_BLK_NUM_CFG "(\$val1 < (\$val2-8))" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRQ2: There are more than eight used blocks"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_0 val1 and register PSWRQ2_REG_MAX_BLKS_VQ0 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_0 PSWRQ2_REG_MAX_BLKS_VQ0 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_1 val1 and register PSWRQ2_REG_MAX_BLKS_VQ1 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_1 PSWRQ2_REG_MAX_BLKS_VQ1 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_2 val1 and register PSWRQ2_REG_MAX_BLKS_VQ2 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_2 PSWRQ2_REG_MAX_BLKS_VQ2 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_3 val1 and register PSWRQ2_REG_MAX_BLKS_VQ3 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_3 PSWRQ2_REG_MAX_BLKS_VQ3 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_4 val1 and register PSWRQ2_REG_MAX_BLKS_VQ4 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_4 PSWRQ2_REG_MAX_BLKS_VQ4 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_5 val1 and register PSWRQ2_REG_MAX_BLKS_VQ5 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_5 PSWRQ2_REG_MAX_BLKS_VQ5 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_6 val1 and register PSWRQ2_REG_MAX_BLKS_VQ6 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_6 PSWRQ2_REG_MAX_BLKS_VQ6 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_7 val1 and register PSWRQ2_REG_MAX_BLKS_VQ7 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_7 PSWRQ2_REG_MAX_BLKS_VQ7 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_8 val1 and register PSWRQ2_REG_MAX_BLKS_VQ8 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_8 PSWRQ2_REG_MAX_BLKS_VQ8 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_9 val1 and register PSWRQ2_REG_MAX_BLKS_VQ9 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_9 PSWRQ2_REG_MAX_BLKS_VQ9 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_10 val1 and register PSWRQ2_REG_MAX_BLKS_VQ10 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_10 PSWRQ2_REG_MAX_BLKS_VQ10 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_11 val1 and register PSWRQ2_REG_MAX_BLKS_VQ11 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_11 PSWRQ2_REG_MAX_BLKS_VQ11 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_12 val1 and register PSWRQ2_REG_MAX_BLKS_VQ12 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_12 PSWRQ2_REG_MAX_BLKS_VQ12 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_13 val1 and register PSWRQ2_REG_MAX_BLKS_VQ13 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_13 PSWRQ2_REG_MAX_BLKS_VQ13 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_14 val1 and register PSWRQ2_REG_MAX_BLKS_VQ14 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_14 PSWRQ2_REG_MAX_BLKS_VQ14 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_15 val1 and register PSWRQ2_REG_MAX_BLKS_VQ15 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_15 PSWRQ2_REG_MAX_BLKS_VQ15 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_16 val1 and register PSWRQ2_REG_MAX_BLKS_VQ16 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_16 PSWRQ2_REG_MAX_BLKS_VQ16 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_17 val1 and register PSWRQ2_REG_MAX_BLKS_VQ17 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_17 PSWRQ2_REG_MAX_BLKS_VQ17 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_18 val1 and register PSWRQ2_REG_MAX_BLKS_VQ18 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_18 PSWRQ2_REG_MAX_BLKS_VQ18 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_19 val1 and register PSWRQ2_REG_MAX_BLKS_VQ19 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_19 PSWRQ2_REG_MAX_BLKS_VQ19 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_20 val1 and register PSWRQ2_REG_MAX_BLKS_VQ20 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_20 PSWRQ2_REG_MAX_BLKS_VQ20 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_21 val1 and register PSWRQ2_REG_MAX_BLKS_VQ21 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_21 PSWRQ2_REG_MAX_BLKS_VQ21 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_22 val1 and register PSWRQ2_REG_MAX_BLKS_VQ22 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_22 PSWRQ2_REG_MAX_BLKS_VQ22 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_23 val1 and register PSWRQ2_REG_MAX_BLKS_VQ23 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_23 PSWRQ2_REG_MAX_BLKS_VQ23 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_24 val1 and register PSWRQ2_REG_MAX_BLKS_VQ24 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_24 PSWRQ2_REG_MAX_BLKS_VQ24 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_25 val1 and register PSWRQ2_REG_MAX_BLKS_VQ25 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_25 PSWRQ2_REG_MAX_BLKS_VQ25 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_26 val1 and register PSWRQ2_REG_MAX_BLKS_VQ26 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_26 PSWRQ2_REG_MAX_BLKS_VQ26 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_27 val1 and register PSWRQ2_REG_MAX_BLKS_VQ27 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_27 PSWRQ2_REG_MAX_BLKS_VQ27 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_28 val1 and register PSWRQ2_REG_MAX_BLKS_VQ28 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_28 PSWRQ2_REG_MAX_BLKS_VQ28 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_29 val1 and register PSWRQ2_REG_MAX_BLKS_VQ29 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_29 PSWRQ2_REG_MAX_BLKS_VQ29 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_30 val1 and register PSWRQ2_REG_MAX_BLKS_VQ30 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_30 PSWRQ2_REG_MAX_BLKS_VQ30 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_BLK_CNT_PER_VQ_31 val1 and register PSWRQ2_REG_MAX_BLKS_VQ31 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_BLK_CNT_PER_VQ_31 PSWRQ2_REG_MAX_BLKS_VQ31 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: blk_cnt_per_vq is not equal to max_blks_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_0 val1 and register PSWRQ2_REG_MAX_SRS_VQ0 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_0 PSWRQ2_REG_MAX_SRS_VQ0 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_1 val1 and register PSWRQ2_REG_MAX_SRS_VQ1 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_1 PSWRQ2_REG_MAX_SRS_VQ1 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_2 val1 and register PSWRQ2_REG_MAX_SRS_VQ2 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_2 PSWRQ2_REG_MAX_SRS_VQ2 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_3 val1 and register PSWRQ2_REG_MAX_SRS_VQ3 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_3 PSWRQ2_REG_MAX_SRS_VQ3 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_4 val1 and register PSWRQ2_REG_MAX_SRS_VQ4 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_4 PSWRQ2_REG_MAX_SRS_VQ4 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_5 val1 and register PSWRQ2_REG_MAX_SRS_VQ5 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_5 PSWRQ2_REG_MAX_SRS_VQ5 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_6 val1 and register PSWRQ2_REG_MAX_SRS_VQ6 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_6 PSWRQ2_REG_MAX_SRS_VQ6 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_7 val1 and register PSWRQ2_REG_MAX_SRS_VQ7 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_7 PSWRQ2_REG_MAX_SRS_VQ7 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_8 val1 and register PSWRQ2_REG_MAX_SRS_VQ8 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_8 PSWRQ2_REG_MAX_SRS_VQ8 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_9 val1 and register PSWRQ2_REG_MAX_SRS_VQ9 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_9 PSWRQ2_REG_MAX_SRS_VQ9 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_10 val1 and register PSWRQ2_REG_MAX_SRS_VQ10 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_10 PSWRQ2_REG_MAX_SRS_VQ10 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_11 val1 and register PSWRQ2_REG_MAX_SRS_VQ11 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_11 PSWRQ2_REG_MAX_SRS_VQ11 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_12 val1 and register PSWRQ2_REG_MAX_SRS_VQ12 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_12 PSWRQ2_REG_MAX_SRS_VQ12 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_13 val1 and register PSWRQ2_REG_MAX_SRS_VQ13 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_13 PSWRQ2_REG_MAX_SRS_VQ13 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_14 val1 and register PSWRQ2_REG_MAX_SRS_VQ14 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_14 PSWRQ2_REG_MAX_SRS_VQ14 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_15 val1 and register PSWRQ2_REG_MAX_SRS_VQ15 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_15 PSWRQ2_REG_MAX_SRS_VQ15 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_16 val1 and register PSWRQ2_REG_MAX_SRS_VQ16 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_16 PSWRQ2_REG_MAX_SRS_VQ16 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_17 val1 and register PSWRQ2_REG_MAX_SRS_VQ17 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_17 PSWRQ2_REG_MAX_SRS_VQ17 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_18 val1 and register PSWRQ2_REG_MAX_SRS_VQ18 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_18 PSWRQ2_REG_MAX_SRS_VQ18 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_19 val1 and register PSWRQ2_REG_MAX_SRS_VQ19 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_19 PSWRQ2_REG_MAX_SRS_VQ19 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_20 val1 and register PSWRQ2_REG_MAX_SRS_VQ20 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_20 PSWRQ2_REG_MAX_SRS_VQ20 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_21 val1 and register PSWRQ2_REG_MAX_SRS_VQ21 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_21 PSWRQ2_REG_MAX_SRS_VQ21 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_22 val1 and register PSWRQ2_REG_MAX_SRS_VQ22 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_22 PSWRQ2_REG_MAX_SRS_VQ22 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_23 val1 and register PSWRQ2_REG_MAX_SRS_VQ23 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_23 PSWRQ2_REG_MAX_SRS_VQ23 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_24 val1 and register PSWRQ2_REG_MAX_SRS_VQ24 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_24 PSWRQ2_REG_MAX_SRS_VQ24 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_25 val1 and register PSWRQ2_REG_MAX_SRS_VQ25 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_25 PSWRQ2_REG_MAX_SRS_VQ25 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_26 val1 and register PSWRQ2_REG_MAX_SRS_VQ26 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_26 PSWRQ2_REG_MAX_SRS_VQ26 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_27 val1 and register PSWRQ2_REG_MAX_SRS_VQ27 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_27 PSWRQ2_REG_MAX_SRS_VQ27 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_28 val1 and register PSWRQ2_REG_MAX_SRS_VQ28 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_28 PSWRQ2_REG_MAX_SRS_VQ28 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_29 val1 and register PSWRQ2_REG_MAX_SRS_VQ29 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_29 PSWRQ2_REG_MAX_SRS_VQ29 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_30 val1 and register PSWRQ2_REG_MAX_SRS_VQ30 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_30 PSWRQ2_REG_MAX_SRS_VQ30 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWRQ2_REG_SR_CNT_PER_VQ_31 val1 and register PSWRQ2_REG_MAX_SRS_VQ31 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWRQ2_REG_SR_CNT_PER_VQ_31 PSWRQ2_REG_MAX_SRS_VQ31 "(\$val1 != \$val2)" IDLE_CHK_WARNING "PSWRQ2: SR_CNT_per_vq is not equal to MAX_SRS_vq"
	# Read register PSWWR2_REG_PGLUE_EOP_ERR_DETAILS val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWWR2_REG_PGLUE_EOP_ERR_DETAILS "(\$val != 0)" IDLE_CHK_ERROR "PSWWR: Details of first request that triggered any of the 3 EOP interrupts: [4:0] - client ID. [7:5] - (sum1[5:3] + 1) or (sum1[5:4] + 1) according to the definition in the spec. [10:8] - number_of_valid_64bit_words[2:0] or number_of_valid_128bit_words[1:0] according to the definition in the spec. [13:11] - The type of interrupt the logging corresponds to: [11] - pglue_eop_error; [12] - pglue_lsr_error; [13] - pglue_eop_error_in_line. [14] - valid - indicates if there was a request that triggered EOP interrupt since this register was cleared."
	# Read register PSWWR2_REG_PRM_CURR_FILL_LEVEL val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWWR2_REG_PRM_CURR_FILL_LEVEL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWWR: PRM payload FIFO is not empty."
	# Read register PSWWR2_REG_CDU_CURR_FILL_LEVEL val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWWR2_REG_CDU_CURR_FILL_LEVEL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWWR: CDU payload FIFO is not empty."
	# Read register PSWWR2_REG_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWWR2_REG_INT_STS "(\$val != 0)" IDLE_CHK_WARNING "PSWWR2: Interrupt status is not 0"
	# Read register PSWWR_REG_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWWR_REG_INT_STS "(\$val != 0)" IDLE_CHK_WARNING "PSWWR: Interrupt status is not 0"
	# Read register PSWRD2_REG_START_INIT val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWRD2_REG_START_INIT "(\$val != 1)" IDLE_CHK_ERROR "PSWRD2: rd_start_init is not 1"
	# Read register PSWRD2_REG_INIT_DONE val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWRD2_REG_INIT_DONE "(\$val != 1)" IDLE_CHK_ERROR "PSWRD2: rd_init_done is not 1"
	# Read register PSWRD2_REG_PORT_IS_IDLE_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWRD2_REG_PORT_IS_IDLE_0 "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRD2: P0 All delivery ports are not idle"
	# Read register PSWRD2_REG_PORT_IS_IDLE_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWRD2_REG_PORT_IS_IDLE_1 "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWRD2: P1 All delivery ports are not idle"
	# Read register PSWRD2_REG_ALMOST_FULL_0 val in loop (incr by 4) and check if condition on val exist
	IDLE_CHK_2 0x3f PSWRD2_REG_ALMOST_FULL_0 15 4 "(\$val != 0)" IDLE_CHK_ERROR "PSWRD2: rd_almost_full is not 0"
	# Read register PSWRD2_REG_DISABLE_INPUTS val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWRD2_REG_DISABLE_INPUTS "(\$val != 0)" IDLE_CHK_ERROR "PSWRD2: PSWRD inputs are disabled"
	# Read register PSWRD_REG_FIFO_FULL_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWRD_REG_FIFO_FULL_STATUS "(\$val != 0)" IDLE_CHK_ERROR "PSWRD: Full is asserted from at least one client."
	# Read register PSWRD2_REG_CPL_ERR_DETAILS val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWRD2_REG_CPL_ERR_DETAILS "(\$val != 0)" IDLE_CHK_WARNING "PSWRD2: Completion received with error. Error details register is not 0. (15:0) - ECHO. (28:16) - Sub Request length minus 1. (29) - first SR. (30) - last SR."
	# Read register PSWRD2_REG_CPL_ERR_DETAILS2 val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWRD2_REG_CPL_ERR_DETAILS2 "(\$val != 0)" IDLE_CHK_WARNING "PSWRD2: Completion received with error. Error details 2nd register is not 0. (4:0) - VQ ID. (9:5) - client ID. (10) - valid bit."
	# Read register PSWRD2_REG_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWRD2_REG_INT_STS "(\$val != 0)" IDLE_CHK_WARNING "PSWRD2: Interrupt status is not 0"
	# Read register PSWRD_REG_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWRD_REG_INT_STS "(\$val != 0)" IDLE_CHK_WARNING "PSWRD: Interrupt status is not 0"
	# Read register PSWHST2_REG_HEADER_FIFO_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWHST2_REG_HEADER_FIFO_STATUS "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWHST2: HST header FIFO status is not 0"
	# Read register PSWHST2_REG_DATA_FIFO_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWHST2_REG_DATA_FIFO_STATUS "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSWHST2: HST data FIFO status is not 0"
	# Read register PSWHST_REG_ARB_IS_IDLE val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWHST_REG_ARB_IS_IDLE "(\$val != 3)" IDLE_CHK_WARNING "PSWHST: HST arbiter is not idle"
	# Read register PSWHST_REG_DISCARD_INTERNAL_WRITES_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWHST_REG_DISCARD_INTERNAL_WRITES_STATUS "(\$val != 0)" IDLE_CHK_WARNING "PSWHST: HST Close the gates: Discarding internal writes"
	# Read register PSWHST_REG_DISCARD_DOORBELLS_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWHST_REG_DISCARD_DOORBELLS_STATUS "(\$val != 0)" IDLE_CHK_WARNING "PSWHST: HST Close the gates: Discarding doorbells"
	# Read register PSWHST_REG_SOURCE_CREDITS_AVAIL val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWHST_REG_SOURCE_CREDITS_AVAIL "(\$val != 141995)" IDLE_CHK_WARNING "PSWHST: HST: not all credits available from internal write sources. (1:0) usdm; (3:2) xsdm; (5:4) msdm; (7:6) ysdm; (9:8) psdm; (11:10) tsdm; (13:12) pbf; (16:14) qm; (18:17) nig"
	# Read register PSWHST_REG_SOURCE_CREDIT_VIOL_VALID val1 and register PSWHST_REG_SOURCE_CREDIT_VIOL_DATA val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWHST_REG_SOURCE_CREDIT_VIOL_VALID PSWHST_REG_SOURCE_CREDIT_VIOL_DATA "(\$val1 != 0)" IDLE_CHK_WARNING "PSWHST: HST: Internal write source client consumed more credits than it is allowed. 0 TSDM; 1 MSDM; 2 USDM; 3 XSDM; 4 YSDM; 5 PSDM; 6 TSDM; 7 PBF; 8 QM; 9 NIG."
	# Read register PSWHST_REG_DEST_CREDITS_AVAIL val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWHST_REG_DEST_CREDITS_AVAIL "(\$val != 60074)" IDLE_CHK_WARNING "PSWHST: HST: not all credits available to internal write destinations. (1:0) usdm; (3:2) xsdm; (5:4) msdm; (7:6) ysdm; (9:8) psdm; (11:10) tsdm; (13:12) igu; (15:14) cau"
	# Read register PSWHST_REG_CLIENTS_WAITING_TO_SOURCE_ARB val in loop (incr by 4) and check if condition on val exist
	IDLE_CHK_2 0x3f PSWHST_REG_CLIENTS_WAITING_TO_SOURCE_ARB 8 4 "(\$val != 0)" IDLE_CHK_WARNING "PSWHST: Each entry contains a bit mask for PSWHST source arbiter clients. '1' means this client is waiting for the arbiter. Each entry refers to a different source arbiter. Entry decoding: (0 TSDM; 1 MSDM; 2 USDM; 3 XSDM; 4 YSDM; 5 PSDM; 6 IGU; 7 CAU). Bit mask decoding: (0 TSDM; 1 MSDM; 2 USDM; 3 XSDM; 4 YSDM; 5 PSDM; 6 PBF; 7 QM; 8 NIG; 9 HOST WR; 10 HOST RD)"
	# Read register PSWHST_REG_IS_IN_DRAIN_MODE val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWHST_REG_IS_IN_DRAIN_MODE "(\$val !=0)" IDLE_CHK_WARNING "PSWHST: HST: Is in drain mode (after target timeout)."
	# Read register PSWHST_REG_TIMEOUT_VALID val1 and register PSWHST_REG_TIMEOUT_DATA val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWHST_REG_TIMEOUT_VALID PSWHST_REG_TIMEOUT_DATA "(\$val1 != 0)" IDLE_CHK_WARNING "PSWHST: HST: Target Timeout details. (29:24) - length in DWs; (23:16) - byte enable; (15:13) - pfid; (12:6) - vfid; (5) - vf_valid; (4:1) - client (0 TSDM; 1 MSDM; 2 USDM; 3 XSDM; 4 YSDM; 5 PSDM; 6 HC; 7 GRC; 8 DQ; 9 ATC; 10 RESERVED SPACE); (0) - 0 - read 1 - write."
	# Read register PSWHST_REG_TIMEOUT_VALID val1 and register PSWHST_REG_TIMEOUT_ADDRESS val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWHST_REG_TIMEOUT_VALID PSWHST_REG_TIMEOUT_ADDRESS "(\$val1 != 0)" IDLE_CHK_WARNING "PSWHST: HST: Target Timeout address."
	# Read register PSWHST_REG_VF_DISABLED_ERROR_VALID val1 and register PSWHST_REG_VF_DISABLED_ERROR_DATA val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f PSWHST_REG_VF_DISABLED_ERROR_VALID PSWHST_REG_VF_DISABLED_ERROR_DATA "(\$val1 != 0)" IDLE_CHK_WARNING "PSWHST: Access to disabled VF took place. (0) - 0 - read 1 - write. (4:1) - client (0 TSDM; 1 MSDM; 2 USDM; 3 XSDM; 4 YSDM; 5 PSDM; 6 HC; 7 GRC; 8 DQ; 9 ATC; 10 RESERVED SPACE). (5) vf_valid. (12:6) vfid. (15:13) pfid."
	# Read register PSWHST_REG_PER_VIOLATION_VALID val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWHST_REG_PER_VIOLATION_VALID "(\$val != 0)" IDLE_CHK_WARNING "PSWHST: Zone A permission violation occurred"
	# Read register PSWHST_REG_INCORRECT_ACCESS_VALID val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWHST_REG_INCORRECT_ACCESS_VALID "(\$val != 0)" IDLE_CHK_WARNING "PSWHST: Incorrect transaction took place"
	# Read register PSWHST2_REG_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWHST2_REG_INT_STS "(\$val != 0)" IDLE_CHK_WARNING "PSWHST2: Interrupt status is not 0"
	# Read register PSWHST_REG_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWHST_REG_INT_STS "(\$val != 0)" IDLE_CHK_WARNING "PSWHST: Interrupt status is not 0"
	# Read register RSS_REG_FULL_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f RSS_REG_FULL_STATUS "(\$val != 0)" IDLE_CHK_WARNING "RSS: FIFOS are full"
	# Read register RSS_REG_EMPTY_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f RSS_REG_EMPTY_STATUS "(\$val != 31)" IDLE_CHK_WARNING "RSS: FIFOS are not empty"
	# Read register RSS_REG_COUNTERS_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f RSS_REG_COUNTERS_STATUS "(\$val != 0)" IDLE_CHK_WARNING "RSS: Counters are not 0"
	# Read register RSS_REG_STATE_MACHINES val and check if condition on val exist
	IDLE_CHK_1 0x3f RSS_REG_STATE_MACHINES "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "RSS: State machines are not in reset state"
	# Read register RSS_REG_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f RSS_REG_INT_STS "(\$val != 0)" IDLE_CHK_ERROR "RSS: Interrupt status is set"
	# Read register BRB_REG_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f BRB_REG_INT_STS_0 "(\$val != 0)" IDLE_CHK_ERROR "BRB: Interrupt status 0 is set"
	# Read register BRB_REG_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f BRB_REG_INT_STS_1 "(\$val != 0)" IDLE_CHK_ERROR "BRB: Interrupt status 1 is set"
	# Read register BRB_REG_INT_STS_2 val and check if condition on val exist
	IDLE_CHK_1 0x3f BRB_REG_INT_STS_2 "(\$val != 0)" IDLE_CHK_ERROR "BRB: Interrupt status 2 is set"
	# Read register BRB_REG_INT_STS_3 val and check if condition on val exist
	IDLE_CHK_1 0x3f BRB_REG_INT_STS_3 "(\$val != 0)" IDLE_CHK_ERROR "BRB: Interrupt status 3 is set"
	# Read register BRB_REG_INT_STS_4 val and check if condition on val exist
	IDLE_CHK_1 0x3f BRB_REG_INT_STS_4 "(\$val != 0x200000)" IDLE_CHK_ERROR "BRB: Interrupt status 4 is set to valus differ from 0x20_0000"
	# Read register BRB_REG_LL_ARB_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f BRB_REG_LL_ARB_EMPTY "(\$val != 2)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Link list arbiter is not empty"
	# Read register BRB_REG_RC_PKT_EMPTY_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f BRB_REG_RC_PKT_EMPTY_0 "(\$val != 255)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Read packet client 0 FIFOs are not empty"
	# Read register BRB_REG_RC_PKT_EMPTY_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f BRB_REG_RC_PKT_EMPTY_1 "(\$val != 255)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Read packet client 1 FIFOs are not empty"
	# Read register BRB_REG_RC_PKT_EMPTY_2 val and check if condition on val exist
	IDLE_CHK_1 0x3f BRB_REG_RC_PKT_EMPTY_2 "(\$val != 255)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Read packet client 2 FIFOs are not empty"
	# Read register BRB_REG_RC_PKT_EMPTY_3 val and check if condition on val exist
	IDLE_CHK_1 0x3f BRB_REG_RC_PKT_EMPTY_3 "(\$val != 255)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Read packet client 3 FIFOs is not empty"
	# Read register BRB_REG_RC_PKT_EMPTY_4 val and check if condition on val exist
	IDLE_CHK_1 0x3f BRB_REG_RC_PKT_EMPTY_4 "(\$val != 255)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Read packet client 4 FIFOs are not empty"
	# Read register BRB_REG_RC_SOP_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f BRB_REG_RC_SOP_EMPTY "(\$val != 15)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Read SOP client FIFOs are not empty"
	# Read register BRB_REG_RC_EOP_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0xf BRB_REG_RC_EOP_EMPTY "(\$val != 3)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Read EOP clients FIFOs are not empty"
	# Read register BRB_REG_RC_EOP_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x30 BRB_REG_RC_EOP_EMPTY "(\$val != 15)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Read EOP clients FIFOs are not empty"
	# Read register BRB_REG_WC_EMPTY_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f BRB_REG_WC_EMPTY_0 "(\$val != 8190)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Write client 0 FIFOs are not empty (value != 0x1ffe)"
	# Read register BRB_REG_WC_EMPTY_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f BRB_REG_WC_EMPTY_1 "(\$val != 8190)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Write client 1 FIFOs are not empty (value != 0x1ffe)"
	# Read register BRB_REG_WC_EMPTY_2 val and check if condition on val exist
	IDLE_CHK_1 0x3f BRB_REG_WC_EMPTY_2 "(\$val != 8190)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Write client 2 FIFOs are not empty (value != 0x1ffe)"
	# Read register BRB_REG_WC_EMPTY_3 val and check if condition on val exist
	IDLE_CHK_1 0x3f BRB_REG_WC_EMPTY_3 "(\$val != 8190)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Write client 3 FIFOs are not empty (value != 0x1ffe)"
	# Read register BRB_REG_WC_EMPTY_4 val and check if condition on val exist
	IDLE_CHK_1 0x30 BRB_REG_WC_EMPTY_4 "(\$val != 8190)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Write client 4 FIFOs are not empty (value != 0x1ffe)"
	# Read register BRB_REG_WC_EMPTY_5 val and check if condition on val exist
	IDLE_CHK_1 0x30 BRB_REG_WC_EMPTY_5 "(\$val != 8190)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Write client 5 FIFOs are not empty (value != 0x1ffe)"
	# Read register BRB_REG_WC_EMPTY_6 val and check if condition on val exist
	IDLE_CHK_1 0x30 BRB_REG_WC_EMPTY_6 "(\$val != 8190)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Write client 6 FIFOs are not empty (value != 0x1ffe)"
	# Read register BRB_REG_WC_EMPTY_7 val and check if condition on val exist
	IDLE_CHK_1 0x30 BRB_REG_WC_EMPTY_7 "(\$val != 8190)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Write client 7 FIFOs are not empty (value != 0x1ffe)"
	# Read register BRB_REG_WC_STATUS_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f BRB_REG_WC_STATUS_0 "(\$val != 2)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Write client 0 status is not 0"
	# Read register BRB_REG_WC_STATUS_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f BRB_REG_WC_STATUS_1 "(\$val != 2)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Write client 1 status is not 0"
	# Read register BRB_REG_WC_STATUS_2 val and check if condition on val exist
	IDLE_CHK_1 0x3f BRB_REG_WC_STATUS_2 "(\$val != 2)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Write client 2 status is not 0"
	# Read register BRB_REG_WC_STATUS_3 val and check if condition on val exist
	IDLE_CHK_1 0x3f BRB_REG_WC_STATUS_3 "(\$val != 2)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Write client 3 status is not 0"
	# Read register BRB_REG_WC_STATUS_4 val and check if condition on val exist
	IDLE_CHK_1 0x30 BRB_REG_WC_STATUS_4 "(\$val != 2)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Write client 4 status is not 0"
	# Read register BRB_REG_WC_STATUS_5 val and check if condition on val exist
	IDLE_CHK_1 0x30 BRB_REG_WC_STATUS_5 "(\$val != 2)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Write client 5 status is not 0"
	# Read register BRB_REG_WC_STATUS_6 val and check if condition on val exist
	IDLE_CHK_1 0x30 BRB_REG_WC_STATUS_6 "(\$val != 2)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Write client 6 status is not 0"
	# Read register BRB_REG_WC_STATUS_7 val and check if condition on val exist
	IDLE_CHK_1 0x30 BRB_REG_WC_STATUS_7 "(\$val != 2)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Write client 7 status is not 0"
	# Read register BRB_REG_WC_BANDWIDTH_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f BRB_REG_WC_BANDWIDTH_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Write client bandwidth full IF is not 0"
	# Read register BRB_REG_RC_PKT_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f BRB_REG_RC_PKT_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Read packet client IF full is not 0"
	# Read register BRB_REG_RC_PKT_STATE val and check if condition on val exist
	IDLE_CHK_1 0x3f BRB_REG_RC_PKT_STATE "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Read packet client state machines are not idle"
	# Read register BRB_REG_STOP_PACKET_COUNTER val and check if condition on val exist
	IDLE_CHK_1 0x3f BRB_REG_STOP_PACKET_COUNTER "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Stop packet counter status is not 0"
	# Read register BRB_REG_STOP_BYTE_COUNTER val and check if condition on val exist
	IDLE_CHK_1 0x3f BRB_REG_STOP_BYTE_COUNTER "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "BRB: Stop byte counter status is not 0"
	# Read register BTB_REG_WC_EMPTY_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f BTB_REG_WC_EMPTY_0 "(\$val != 8190)" IDLE_CHK_ERROR_NO_TRAFFIC "BTB: Write client 0 FIFOs are not empty (value != 0x1ffe)"
	# Read register BTB_REG_WC_STATUS_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f BTB_REG_WC_STATUS_0 "(\$val != 2)" IDLE_CHK_ERROR_NO_TRAFFIC "BTB: Write client status is not 2"
	# Read register BTB_REG_WC_DUP_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f BTB_REG_WC_DUP_EMPTY "(\$val != 15)" IDLE_CHK_ERROR_NO_TRAFFIC "BTB: Duplicated Write client FIFOs are not empty"
	# Read register BTB_REG_WC_DUP_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f BTB_REG_WC_DUP_STATUS "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "BTB: Duplicated Write client status is not 0"
	# Read register BTB_REG_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f BTB_REG_INT_STS_0 "(\$val != 0)" IDLE_CHK_ERROR "BTB: Interrupt status 0 is set"
	# Read register BTB_REG_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f BTB_REG_INT_STS_1 "(\$val != 0)" IDLE_CHK_ERROR "BTB: Interrupt status 1 is set"
	# Read register BTB_REG_INT_STS_2 val and check if condition on val exist
	IDLE_CHK_1 0x3f BTB_REG_INT_STS_2 "(\$val != 0)" IDLE_CHK_ERROR "BTB: Interrupt status 2 is set"
	# Read register BTB_REG_INT_STS_3 val and check if condition on val exist
	IDLE_CHK_1 0x3f BTB_REG_INT_STS_3 "(\$val != 0)" IDLE_CHK_ERROR "BTB: Interrupt status 3 is set"
	# Read register BTB_REG_INT_STS_4 val and check if condition on val exist
	IDLE_CHK_1 0x3f BTB_REG_INT_STS_4 "(\$val != 0)" IDLE_CHK_ERROR "BTB: Interrupt status 4 is set"
	# Read register BTB_REG_BLOCK_OCCUPANCY val and check if condition on val exist
	IDLE_CHK_1 0x3f BTB_REG_BLOCK_OCCUPANCY "(\$val > 46)" IDLE_CHK_ERROR "BTB: Blocks occupacy > 46"
	# Read register BTB_REG_RC_SOP_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f BTB_REG_RC_SOP_EMPTY "(\$val != 15)" IDLE_CHK_ERROR_NO_TRAFFIC "BTB: Read SOP client FIFOs are not empty"
	# Read register BTB_REG_RC_PKT_EMPTY_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f BTB_REG_RC_PKT_EMPTY_0 "(\$val != 255)" IDLE_CHK_ERROR_NO_TRAFFIC "BTB: Read packet client 0 FIFOs are not empty"
	# Read register BTB_REG_RC_PKT_EMPTY_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f BTB_REG_RC_PKT_EMPTY_1 "(\$val != 255)" IDLE_CHK_ERROR_NO_TRAFFIC "BTB: Read packet client 1 FIFOs are not empty"
	# Read register BTB_REG_RC_PKT_EMPTY_2 val and check if condition on val exist
	IDLE_CHK_1 0x3f BTB_REG_RC_PKT_EMPTY_2 "(\$val != 255)" IDLE_CHK_ERROR_NO_TRAFFIC "BTB: Read packet client 2 FIFOs are not empty"
	# Read register BTB_REG_RC_PKT_EMPTY_3 val and check if condition on val exist
	IDLE_CHK_1 0x3f BTB_REG_RC_PKT_EMPTY_3 "(\$val != 255)" IDLE_CHK_ERROR_NO_TRAFFIC "BTB: Read packet client 3 FIFOs are not empty"
	# Read register BTB_REG_RC_PKT_EMPTY_4 val and check if condition on val exist
	IDLE_CHK_1 0x30 BTB_REG_RC_PKT_EMPTY_4 "(\$val != 255)" IDLE_CHK_ERROR_NO_TRAFFIC "BTB: Read packet client 4 FIFOs are not empty"
	# Read register BTB_REG_RC_PKT_EMPTY_5 val and check if condition on val exist
	IDLE_CHK_1 0x30 BTB_REG_RC_PKT_EMPTY_5 "(\$val != 255)" IDLE_CHK_ERROR_NO_TRAFFIC "BTB: Read packet client 5 FIFOs are not empty"
	# Read register BTB_REG_RC_PKT_EMPTY_6 val and check if condition on val exist
	IDLE_CHK_1 0x30 BTB_REG_RC_PKT_EMPTY_6 "(\$val != 255)" IDLE_CHK_ERROR_NO_TRAFFIC "BTB: Read packet client 6 FIFOs are not empty"
	# Read register BTB_REG_RC_PKT_EMPTY_7 val and check if condition on val exist
	IDLE_CHK_1 0x30 BTB_REG_RC_PKT_EMPTY_7 "(\$val != 255)" IDLE_CHK_ERROR_NO_TRAFFIC "BTB: Read packet client 7 FIFOs are not empty"
	# Read register BTB_REG_LL_ARB_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f BTB_REG_LL_ARB_EMPTY "(\$val != 2)" IDLE_CHK_ERROR_NO_TRAFFIC "BTB: Link list arbiter is not empty"
	# Read register BTB_REG_WC_BANDWIDTH_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f BTB_REG_WC_BANDWIDTH_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "BTB: Write client bandwidth full IF is not 0"
	# Read register BTB_REG_RC_PKT_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f BTB_REG_RC_PKT_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "BTB: Read packet client IF full is not 0"
	# Read register BTB_REG_RC_PKT_STATE val and check if condition on val exist
	IDLE_CHK_1 0x3f BTB_REG_RC_PKT_STATE "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "BTB: Read packet client state machines are not idle"
	# Read register DMAE_REG_GO_C0 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C0 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 0 go is not 0"
	# Read register DMAE_REG_GO_C1 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C1 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 1 go is not 0"
	# Read register DMAE_REG_GO_C2 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C2 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 2 go is not 0"
	# Read register DMAE_REG_GO_C3 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C3 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 3 go is not 0"
	# Read register DMAE_REG_GO_C4 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C4 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 4 go is not 0"
	# Read register DMAE_REG_GO_C5 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C5 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 5 go is not 0"
	# Read register DMAE_REG_GO_C6 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C6 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 6 go is not 0"
	# Read register DMAE_REG_GO_C7 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C7 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 7 go is not 0"
	# Read register DMAE_REG_GO_C8 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C8 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 8 go is not 0"
	# Read register DMAE_REG_GO_C9 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C9 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 9 go is not 0"
	# Read register DMAE_REG_GO_C10 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C10 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 10 go is not 0"
	# Read register DMAE_REG_GO_C11 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C11 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 11 go is not 0"
	# Read register DMAE_REG_GO_C12 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C12 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 12 go is not 0"
	# Read register DMAE_REG_GO_C13 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C13 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 13 go is not 0"
	# Read register DMAE_REG_GO_C14 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C14 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 14 go is not 0"
	# Read register DMAE_REG_GO_C15 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C15 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 15 go is not 0"
	# Read register DMAE_REG_GO_C16 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C16 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 16 go is not 0"
	# Read register DMAE_REG_GO_C17 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C17 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 17 go is not 0"
	# Read register DMAE_REG_GO_C18 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C18 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 18 go is not 0"
	# Read register DMAE_REG_GO_C19 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C19 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 19 go is not 0"
	# Read register DMAE_REG_GO_C20 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C20 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 20 go is not 0"
	# Read register DMAE_REG_GO_C21 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C21 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 21 go is not 0"
	# Read register DMAE_REG_GO_C22 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C22 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 22 go is not 0"
	# Read register DMAE_REG_GO_C23 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C23 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 23 go is not 0"
	# Read register DMAE_REG_GO_C24 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C24 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 24 go is not 0"
	# Read register DMAE_REG_GO_C25 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C25 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 25 go is not 0"
	# Read register DMAE_REG_GO_C26 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C26 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 26 go is not 0"
	# Read register DMAE_REG_GO_C27 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C27 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 27 go is not 0"
	# Read register DMAE_REG_GO_C28 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C28 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 28 go is not 0"
	# Read register DMAE_REG_GO_C29 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C29 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 29 go is not 0"
	# Read register DMAE_REG_GO_C30 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C30 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 30 go is not 0"
	# Read register DMAE_REG_GO_C31 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_GO_C31 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "DMAE: command 31 go is not 0"
	# Read register MCM_REG_XX_LCID_CAM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_XX_LCID_CAM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "MCM: XX protection CAM is not empty"
	# Read register MCM_REG_XX_LOCK_CNT val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_XX_LOCK_CNT "(\$val != 0)" IDLE_CHK_ERROR "MCM: there is locked LCID in XX"
	# Read register MCM_REG_XX_CBYP_TBL_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_XX_CBYP_TBL_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "MCM: XX Cbypass Table is not empty"
	# Read register MCM_REG_XX_TBYP_TBL_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_XX_TBYP_TBL_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "MCM: XX Tbypass Table is not empty"
	# Read register MCM_REG_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_INT_STS_0 "(\$val != 0)" IDLE_CHK_ERROR "MCM: Interrupt status 0 is not 0"
	# Read register MCM_REG_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_INT_STS_1 "(\$val != 0)" IDLE_CHK_ERROR "MCM: Interrupt status 1 is not 0"
	# Check reset value
	IDLE_CHK_10 0x3f MCM_REG_CCFC_INIT_CRD "(\$val1 != \$val2)" IDLE_CHK_ERROR "MCM: CCFC_INIT_CRD is different than reset value"
	# Check reset value
	IDLE_CHK_10 0x3f MCM_REG_TCFC_INIT_CRD "(\$val1 != \$val2)" IDLE_CHK_ERROR "MCM: TCFC_INIT_CRD is different than reset value"
	# Check reset value
	IDLE_CHK_10 0x3f MCM_REG_QM_INIT_CRD0 "(\$val1 != \$val2)" IDLE_CHK_ERROR "MCM: QM_INIT_CRD0 is different than reset value"
	# Check reset value
	IDLE_CHK_10 0x3f MCM_REG_FIC_INIT_CRD "(\$val1 != \$val2)" IDLE_CHK_ERROR "MCM: FIC_INIT_CRD is different than reset value"
	# Check reset value
	IDLE_CHK_10 0x3f MCM_REG_XX_FREE_CNT "(\$val1 != \$val2)" IDLE_CHK_ERROR "MCM: XX_FREE_CNT is different than reset value"
	# Read register MCM_REG_USDM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_USDM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "MCM: USDM declared message length unequal to actual"
	# Read register MCM_REG_YSDM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_YSDM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "MCM: YSDM declared message length unequal to actual"
	# Read register MCM_REG_PBF_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_PBF_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "MCM: PBF declared message length unequal to actual"
	# Read register MCM_REG_MSDM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_MSDM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "MCM: MSDM declared message length unequal to actual"
	# Read register MCM_REG_TMLD_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_TMLD_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "MCM: TMLD declared message length unequal to actual"
	# Read register MCM_REG_IS_STORM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_IS_STORM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "MCM: Local STORM input stage buffer is not empty"
	# Read register MCM_REG_IS_QM_P_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_IS_QM_P_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "MCM: QM_P input stage buffer is not empty"
	# Read register MCM_REG_IS_QM_S_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_IS_QM_S_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "MCM: QM_S input stage buffer is not empty"
	# Read register MCM_REG_IS_USDM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_IS_USDM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "MCM: USDM input stage buffer is not empty"
	# Read register MCM_REG_IS_YSDM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_IS_YSDM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "MCM: YSDM input stage buffer is not empty"
	# Read register MCM_REG_IS_PBF_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_IS_PBF_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "MCM: PBF input stage buffer is not empty"
	# Read register MCM_REG_IS_MSDM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_IS_MSDM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "MCM: MSDM input stage buffer is not empty"
	# Read register MCM_REG_IS_USEM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_IS_USEM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "MCM: USEM input stage buffer is not empty"
	# Read register MCM_REG_IS_YSEM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_IS_YSEM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "MCM: YSEM input stage buffer is not empty"
	# Read register MCM_REG_IS_TMLD_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_IS_TMLD_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "MCM: TMLD input stage buffer is not empty"
	# Read register MCM_REG_AGG_CON_FIC_BUF_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_AGG_CON_FIC_BUF_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "MCM: Aggregation Connection FIC buffer is not empty"
	# Read register MCM_REG_SM_CON_FIC_BUF_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_SM_CON_FIC_BUF_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "MCM: STORM Connection FIC buffer is not empty"
	# Read register MCM_REG_AGG_TASK_FIC_BUF_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_AGG_TASK_FIC_BUF_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "MCM: Aggregation Task FIC buffer is not empty"
	# Read register MCM_REG_SM_TASK_FIC_BUF_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_SM_TASK_FIC_BUF_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "MCM: STORM Task FIC buffer is not empty"
	# Read register MCM_REG_IA_AGG_CON_PART_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_IA_AGG_CON_PART_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "MCM: Aggregation Connection IA part buffer is not empty"
	# Read register MCM_REG_IA_SM_CON_PART_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_IA_SM_CON_PART_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "MCM: STORM Connection IA part buffer is not empty"
	# Read register MCM_REG_IA_AGG_TASK_PART_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_IA_AGG_TASK_PART_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "MCM: Aggregation Task IA part buffer is not empty"
	# Read register MCM_REG_IA_SM_TASK_PART_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_IA_SM_TASK_PART_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "MCM: STORM Task IA part buffer is not empty"
	# Read register MCM_REG_IA_TRANS_PART_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_IA_TRANS_PART_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "MCM: Transparent part buffer is not empty"
	# Read register MCM_REG_XX_CBYP_TBL_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_XX_CBYP_TBL_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "MCM: XX Connection bypass table is not empty"
	# Read register MCM_REG_XX_TBYP_TBL_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_XX_TBYP_TBL_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "MCM: XX Task bypass table is not empty"
	# Read register MCM_REG_IN_PRCS_TBL_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_IN_PRCS_TBL_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "MCM: In-process table is not empty"
	# Read register MCM_REG_FI_DESC_INPUT_VIOLATE val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_FI_DESC_INPUT_VIOLATE "(\$val != 0)" IDLE_CHK_ERROR "MCM: Input violation in message first descriptor"
	# Read register MCM_REG_SE_DESC_INPUT_VIOLATE val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_SE_DESC_INPUT_VIOLATE "(\$val != 0)" IDLE_CHK_ERROR "MCM: Input violation in message second descriptor"
	# Read register PCM_REG_XX_LCID_CAM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f PCM_REG_XX_LCID_CAM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "PCM: XX protection CAM is not empty"
	# Read register PCM_REG_XX_LOCK_CNT val and check if condition on val exist
	IDLE_CHK_1 0x3f PCM_REG_XX_LOCK_CNT "(\$val != 0)" IDLE_CHK_ERROR "PCM: there is locked LCID in XX"
	# Read register PCM_REG_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PCM_REG_INT_STS_0 "(\$val != 0)" IDLE_CHK_ERROR "PCM: Interrupt status 0 is not 0"
	# Read register PCM_REG_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f PCM_REG_INT_STS_1 "(\$val != 0)" IDLE_CHK_ERROR "PCM: Interrupt status 1 is not 0"
	# Check reset value
	IDLE_CHK_10 0x3f PCM_REG_CCFC_INIT_CRD "(\$val1 != \$val2)" IDLE_CHK_ERROR "PCM: CCFC_INIT_CRD is different than reset value"
	# Check reset value
	IDLE_CHK_10 0x3f PCM_REG_FIC_INIT_CRD "(\$val1 != \$val2)" IDLE_CHK_ERROR "PCM: FIC_INIT_CRD is different than reset value"
	# Check reset value
	IDLE_CHK_10 0x3f PCM_REG_XX_FREE_CNT "(\$val1 != \$val2)" IDLE_CHK_ERROR "PCM: XX_FREE_CNT is different than reset value"
	# Read register PCM_REG_PSDM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x3f PCM_REG_PSDM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "PCM: PSDM declared message length unequal to actual"
	# Read register PCM_REG_PBF_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x3f PCM_REG_PBF_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "PCM: PBF declared message length unequal to actual"
	# Read register PCM_REG_IS_STORM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f PCM_REG_IS_STORM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "PCM: Local STORM input stage buffer is not empty"
	# Read register PCM_REG_IS_PSDM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f PCM_REG_IS_PSDM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "PCM: PSDM input stage buffer is not empty"
	# Read register PCM_REG_IS_PBF_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f PCM_REG_IS_PBF_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "PCM: PBF input stage buffer is not empty"
	# Read register PCM_REG_SM_CON_FIC_BUF_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f PCM_REG_SM_CON_FIC_BUF_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "PCM: STORM Connection FIC buffer is not empty"
	# Read register PCM_REG_IA_SM_CON_PART_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f PCM_REG_IA_SM_CON_PART_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "PCM: STORM Connection IA part buffer is not empty"
	# Read register PCM_REG_IA_TRANS_PART_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f PCM_REG_IA_TRANS_PART_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "PCM: Transparent part buffer is not empty"
	# Read register PCM_REG_IN_PRCS_TBL_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f PCM_REG_IN_PRCS_TBL_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "PCM: In-process table is not empty"
	# Read register PCM_REG_FI_DESC_INPUT_VIOLATE val and check if condition on val exist
	IDLE_CHK_1 0x3f PCM_REG_FI_DESC_INPUT_VIOLATE "(\$val != 0)" IDLE_CHK_ERROR "PCM: Input violation in message first descriptor"
	# Read register TCM_REG_XX_LCID_CAM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_XX_LCID_CAM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "TCM: XX protection CAM is not empty"
	# Read register TCM_REG_XX_LOCK_CNT val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_XX_LOCK_CNT "(\$val != 0)" IDLE_CHK_ERROR "TCM: there is locked LCID in XX"
	# Read register TCM_REG_XX_CBYP_TBL_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_XX_CBYP_TBL_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "TCM: XX Cbypass Table is not empty"
	# Read register TCM_REG_XX_TBYP_TBL_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_XX_TBYP_TBL_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "TCM: XX Tbypass Table is not empty"
	# Read register TCM_REG_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_INT_STS_0 "(\$val != 0)" IDLE_CHK_ERROR "TCM: Interrupt status 0 is not 0"
	# Read register TCM_REG_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_INT_STS_1 "(\$val != 0)" IDLE_CHK_ERROR "TCM: Interrupt status 1 is not 0"
	# Check reset value
	IDLE_CHK_10 0x3f TCM_REG_CCFC_INIT_CRD "(\$val1 != \$val2)" IDLE_CHK_ERROR "TCM: CCFC_INIT_CRD is different than reset value"
	# Check reset value
	IDLE_CHK_10 0x3f TCM_REG_TCFC_INIT_CRD "(\$val1 != \$val2)" IDLE_CHK_ERROR "TCM: TCFC_INIT_CRD is different than reset value"
	# Check reset value
	IDLE_CHK_10 0x3f TCM_REG_QM_INIT_CRD0 "(\$val1 != \$val2)" IDLE_CHK_ERROR "TCM: QM_INIT_CRD0 is different than reset value"
	# Check reset value
	IDLE_CHK_10 0x3f TCM_REG_TM_INIT_CRD "(\$val1 != \$val2)" IDLE_CHK_ERROR "TCM: TM_INIT_CRD is different than reset value"
	# Check reset value
	IDLE_CHK_10 0x3f TCM_REG_FIC_INIT_CRD "(\$val1 != \$val2)" IDLE_CHK_ERROR "TCM: FIC_INIT_CRD is different than reset value"
	# Check reset value
	IDLE_CHK_10 0x3f TCM_REG_XX_FREE_CNT "(\$val1 != \$val2)" IDLE_CHK_ERROR "TCM: XX_FREE_CNT is different than reset value"
	# Read register TCM_REG_TSDM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_TSDM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "TCM: TSDM declared message length unequal to actual"
	# Read register TCM_REG_PBF_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_PBF_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "TCM: PBF declared message length unequal to actual"
	# Read register TCM_REG_PRS_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_PRS_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "TCM: PRS declared message length unequal to actual"
	# Read register TCM_REG_DORQ_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_DORQ_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "TCM: DORQ declared message length unequal to actual"
	# Read register TCM_REG_IS_STORM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_IS_STORM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "TCM: Local STORM input stage buffer is not empty"
	# Read register TCM_REG_IS_QM_P_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_IS_QM_P_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "TCM: QM_P input stage buffer is not empty"
	# Read register TCM_REG_IS_QM_S_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_IS_QM_S_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "TCM: QM_S input stage buffer is not empty"
	# Read register TCM_REG_IS_TSDM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_IS_TSDM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "TCM: TSDM input stage buffer is not empty"
	# Read register TCM_REG_IS_MSEM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_IS_MSEM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "TCM: MSEM input stage buffer is not empty"
	# Read register TCM_REG_IS_PBF_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_IS_PBF_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "TCM: PBF input stage buffer is not empty"
	# Read register TCM_REG_IS_PRS_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_IS_PRS_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "TCM: PRS input stage buffer is not empty"
	# Read register TCM_REG_IS_DORQ_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_IS_DORQ_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "TCM: DORQ input stage buffer is not empty"
	# Read register TCM_REG_IS_TM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_IS_TM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "TCM: TM input stage buffer is not empty"
	# Read register TCM_REG_AGG_CON_FIC_BUF_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_AGG_CON_FIC_BUF_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "TCM: Aggregation Connection FIC buffer is not empty"
	# Read register TCM_REG_SM_CON_FIC_BUF_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_SM_CON_FIC_BUF_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "TCM: STORM Connection FIC buffer is not empty"
	# Read register TCM_REG_AGG_TASK_FIC_BUF_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_AGG_TASK_FIC_BUF_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "TCM: Aggregation Task FIC buffer is not empty"
	# Read register TCM_REG_SM_TASK_FIC_BUF_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_SM_TASK_FIC_BUF_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "TCM: STORM Task FIC buffer is not empty"
	# Read register TCM_REG_IA_AGG_CON_PART_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_IA_AGG_CON_PART_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "TCM: Aggregation Connection IA part buffer is not empty"
	# Read register TCM_REG_IA_SM_CON_PART_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_IA_SM_CON_PART_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "TCM: STORM Connection IA part buffer is not empty"
	# Read register TCM_REG_IA_AGG_TASK_PART_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_IA_AGG_TASK_PART_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "TCM: Aggregation Task IA part buffer is not empty"
	# Read register TCM_REG_IA_SM_TASK_PART_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_IA_SM_TASK_PART_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "TCM: STORM Task IA part buffer is not empty"
	# Read register TCM_REG_IA_TRANS_PART_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_IA_TRANS_PART_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "TCM: Transparent part buffer is not empty"
	# Read register TCM_REG_XX_CBYP_TBL_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_XX_CBYP_TBL_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "TCM: XX Connection bypass table is not empty"
	# Read register TCM_REG_XX_TBYP_TBL_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_XX_TBYP_TBL_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "TCM: XX Task bypass table is not empty"
	# Read register TCM_REG_IN_PRCS_TBL_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_IN_PRCS_TBL_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "TCM: In-process table is not empty"
	# Read register TCM_REG_FI_DESC_INPUT_VIOLATE val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_FI_DESC_INPUT_VIOLATE "(\$val != 0)" IDLE_CHK_ERROR "TCM: Input violation in message first descriptor"
	# Read register TCM_REG_SE_DESC_INPUT_VIOLATE val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_SE_DESC_INPUT_VIOLATE "(\$val != 0)" IDLE_CHK_ERROR "TCM: Input violation in message second descriptor"
	# Read register UCM_REG_XX_LCID_CAM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_XX_LCID_CAM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "UCM: XX protection CAM is not empty"
	# Read register UCM_REG_XX_LOCK_CNT val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_XX_LOCK_CNT "(\$val != 0)" IDLE_CHK_ERROR "UCM: there is locked LCID in XX"
	# Read register UCM_REG_XX_CBYP_TBL_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_XX_CBYP_TBL_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "UCM: XX Cbypass Table is not empty"
	# Read register UCM_REG_XX_TBYP_TBL_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_XX_TBYP_TBL_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "UCM: XX Tbypass Table is not empty"
	# Read register UCM_REG_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_INT_STS_0 "(\$val != 0)" IDLE_CHK_ERROR "UCM: Interrupt status 0 is not 0"
	# Read register UCM_REG_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_INT_STS_1 "(\$val != 0)" IDLE_CHK_ERROR "UCM: Interrupt status 1 is not 0"
	# Check reset value
	IDLE_CHK_10 0x3f UCM_REG_CCFC_INIT_CRD "(\$val1 != \$val2)" IDLE_CHK_ERROR "UCM: CCFC_INIT_CRD is different than reset value"
	# Check reset value
	IDLE_CHK_10 0x3f UCM_REG_TCFC_INIT_CRD "(\$val1 != \$val2)" IDLE_CHK_ERROR "UCM: TCFC_INIT_CRD is different than reset value"
	# Check reset value
	IDLE_CHK_10 0x3f UCM_REG_QM_INIT_CRD0 "(\$val1 != \$val2)" IDLE_CHK_ERROR "UCM: QM_INIT_CRD0 is different than reset value"
	# Check reset value
	IDLE_CHK_10 0x3f UCM_REG_TM_INIT_CRD "(\$val1 != \$val2)" IDLE_CHK_ERROR "UCM: TM_INIT_CRD is different than reset value"
	# Check reset value
	IDLE_CHK_10 0x3f UCM_REG_FIC_INIT_CRD "(\$val1 != \$val2)" IDLE_CHK_ERROR "UCM: FIC_INIT_CRD is different than reset value"
	# Check reset value
	IDLE_CHK_10 0x3f UCM_REG_XX_FREE_CNT "(\$val1 != \$val2)" IDLE_CHK_ERROR "UCM: XX_FREE_CNT is different than reset value"
	# Read register UCM_REG_USDM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_USDM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "UCM: USDM declared message length unequal to actual"
	# Read register UCM_REG_YSDM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_YSDM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "UCM: YSDM declared message length unequal to actual"
	# Read register UCM_REG_PBF_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_PBF_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "UCM: PBF declared message length unequal to actual"
	# Read register UCM_REG_DORQ_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_DORQ_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "UCM: DORQ declared message length unequal to actual"
	# Read register UCM_REG_RDIF_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_RDIF_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "UCM: RDIF declared message length unequal to actual"
	# Read register UCM_REG_TDIF_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_TDIF_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "UCM: TDIF declared message length unequal to actual"
	# Read register UCM_REG_MULD_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_MULD_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "UCM: MULD declared message length unequal to actual"
	# Read register UCM_REG_YULD_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_YULD_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "UCM: YULD declared message length unequal to actual"
	# Read register UCM_REG_IS_STORM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_IS_STORM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "UCM: Local STORM input stage buffer is not empty"
	# Read register UCM_REG_IS_QM_P_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_IS_QM_P_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "UCM: QM_P input stage buffer is not empty"
	# Read register UCM_REG_IS_QM_S_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_IS_QM_S_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "UCM: QM_S input stage buffer is not empty"
	# Read register UCM_REG_IS_USDM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_IS_USDM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "UCM: USDM input stage buffer is not empty"
	# Read register UCM_REG_IS_YSDM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_IS_YSDM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "UCM: YSDM input stage buffer is not empty"
	# Read register UCM_REG_IS_PBF_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_IS_PBF_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "UCM: PBF input stage buffer is not empty"
	# Read register UCM_REG_IS_DORQ_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_IS_DORQ_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "UCM: DORQ input stage buffer is not empty"
	# Read register UCM_REG_IS_RDIF_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_IS_RDIF_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "UCM: RDIF input stage buffer is not empty"
	# Read register UCM_REG_IS_TDIF_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_IS_TDIF_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "UCM: TDIF input stage buffer is not empty"
	# Read register UCM_REG_IS_MULD_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_IS_MULD_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "UCM: MULD input stage buffer is not empty"
	# Read register UCM_REG_IS_YULD_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_IS_YULD_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "UCM: YULD input stage buffer is not empty"
	# Read register UCM_REG_IS_TM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_IS_TM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "UCM: TM input stage buffer is not empty"
	# Read register UCM_REG_AGG_CON_FIC_BUF_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_AGG_CON_FIC_BUF_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "UCM: Aggregation Connection FIC buffer is not empty"
	# Read register UCM_REG_SM_CON_FIC_BUF_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_SM_CON_FIC_BUF_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "UCM: STORM Connection FIC buffer is not empty"
	# Read register UCM_REG_AGG_TASK_FIC_BUF_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_AGG_TASK_FIC_BUF_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "UCM: Aggregation Task FIC buffer is not empty"
	# Read register UCM_REG_SM_TASK_FIC_BUF_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_SM_TASK_FIC_BUF_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "UCM: STORM Task FIC buffer is not empty"
	# Read register UCM_REG_IA_AGG_CON_PART_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_IA_AGG_CON_PART_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "UCM: Aggregation Connection IA part buffer is not empty"
	# Read register UCM_REG_IA_SM_CON_PART_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_IA_SM_CON_PART_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "UCM: STORM Connection IA part buffer is not empty"
	# Read register UCM_REG_IA_AGG_TASK_PART_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_IA_AGG_TASK_PART_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "UCM: Aggregation Task IA part buffer is not empty"
	# Read register UCM_REG_IA_SM_TASK_PART_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_IA_SM_TASK_PART_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "UCM: STORM Task IA part buffer is not empty"
	# Read register UCM_REG_IA_TRANS_PART_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_IA_TRANS_PART_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "UCM: Transparent part buffer is not empty"
	# Read register UCM_REG_XX_CBYP_TBL_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_XX_CBYP_TBL_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "UCM: XX Connection bypass table is not empty"
	# Read register UCM_REG_XX_TBYP_TBL_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_XX_TBYP_TBL_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "UCM: XX Task bypass table is not empty"
	# Read register UCM_REG_IN_PRCS_TBL_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_IN_PRCS_TBL_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "UCM: In-process table is not empty"
	# Read register UCM_REG_FI_DESC_INPUT_VIOLATE val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_FI_DESC_INPUT_VIOLATE "(\$val != 0)" IDLE_CHK_ERROR "UCM: Input violation in message first descriptor"
	# Read register UCM_REG_SE_DESC_INPUT_VIOLATE val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_SE_DESC_INPUT_VIOLATE "(\$val != 0)" IDLE_CHK_ERROR "UCM: Input violation in message second descriptor"
	# Read register XCM_REG_XX_LCID_CAM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_XX_LCID_CAM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "XCM: XX protection CAM is not empty"
	# Read register XCM_REG_XX_LOCK_CNT val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_XX_LOCK_CNT "(\$val != 0)" IDLE_CHK_ERROR "XCM: there is locked LCID in XX"
	# Read register XCM_REG_XX_CBYP_TBL_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_XX_CBYP_TBL_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "XCM: XX Cbypass Table is not empty"
	# Read register XCM_REG_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_INT_STS_0 "(\$val != 0)" IDLE_CHK_ERROR "XCM: Interrupt status 0 is not 0"
	# Read register XCM_REG_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_INT_STS_1 "(\$val != 0)" IDLE_CHK_ERROR "XCM: Interrupt status 1 is not 0"
	# Check reset value
	IDLE_CHK_10 0x3f XCM_REG_CCFC_INIT_CRD "(\$val1 != \$val2)" IDLE_CHK_ERROR "XCM: CCFC_INIT_CRD is different than reset value"
	# Check reset value
	IDLE_CHK_10 0x3f XCM_REG_QM_INIT_CRD0 "(\$val1 != \$val2)" IDLE_CHK_ERROR "XCM: XQM_INIT_CRD0 is different than reset value"
	# Check reset value
	IDLE_CHK_10 0x3f XCM_REG_QM_INIT_CRD1 "(\$val1 != \$val2)" IDLE_CHK_ERROR "XCM: XQM_INIT_CRD1 is different than reset value"
	# Check reset value
	IDLE_CHK_10 0x3f XCM_REG_TM_INIT_CRD "(\$val1 != \$val2)" IDLE_CHK_ERROR "XCM: TM_INIT_CRD is different than reset value"
	# Check reset value
	IDLE_CHK_10 0x3f XCM_REG_FIC_INIT_CRD "(\$val1 != \$val2)" IDLE_CHK_ERROR "XCM: FIC_INIT_CRD is different than reset value"
	# Check reset value
	IDLE_CHK_10 0x3f XCM_REG_XX_FREE_CNT "(\$val1 != \$val2)" IDLE_CHK_ERROR "XCM: XX_FREE_CNT is different than reset value"
	# Read register XCM_REG_MSDM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_MSDM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "XCM: MSDM declared message length unequal to actual"
	# Read register XCM_REG_XSDM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_XSDM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "XCM: XSDM declared message length unequal to actual"
	# Read register XCM_REG_YSDM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_YSDM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "XCM: YSDM declared message length unequal to actual"
	# Read register XCM_REG_PBF_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_PBF_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "XCM: PBF declared message length unequal to actual"
	# Read register XCM_REG_DORQ_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_DORQ_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "XCM: DORQ declared message length unequal to actual"
	# Read register XCM_REG_IS_STORM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_IS_STORM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "XCM: Local STORM input stage buffer is not empty"
	# Read register XCM_REG_IS_QM_P_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_IS_QM_P_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "XCM: QM_P input stage buffer is not empty"
	# Read register XCM_REG_IS_QM_S_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_IS_QM_S_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "XCM: QM_S input stage buffer is not empty"
	# Read register XCM_REG_IS_MSDM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_IS_MSDM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "XCM: MSDM input stage buffer is not empty"
	# Read register XCM_REG_IS_XSDM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_IS_XSDM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "XCM: XSDM input stage buffer is not empty"
	# Read register XCM_REG_IS_YSDM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_IS_YSDM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "XCM: YSDM input stage buffer is not empty"
	# Read register XCM_REG_IS_MSEM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_IS_MSEM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "XCM: MSEM input stage buffer is not empty"
	# Read register XCM_REG_IS_USEM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_IS_USEM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "XCM: USEM input stage buffer is not empty"
	# Read register XCM_REG_IS_YSEM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_IS_YSEM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "XCM: YSEM input stage buffer is not empty"
	# Read register XCM_REG_IS_DORQ_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_IS_DORQ_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "XCM: DORQ input stage buffer is not empty"
	# Read register XCM_REG_IS_PBF_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_IS_PBF_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "XCM: PBF input stage buffer is not empty"
	# Read register XCM_REG_IS_TM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_IS_TM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "XCM: TM input stage buffer is not empty"
	# Read register XCM_REG_AGG_CON_FIC_BUF_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_AGG_CON_FIC_BUF_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "XCM: Aggregation Connection FIC buffer is not empty"
	# Read register XCM_REG_SM_CON_FIC_BUF_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_SM_CON_FIC_BUF_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "XCM: STORM Connection FIC buffer is not empty"
	# Read register XCM_REG_IA_AGG_CON_PART_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_IA_AGG_CON_PART_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "XCM: Aggregation Connection IA part buffer is not empty"
	# Read register XCM_REG_IA_SM_CON_PART_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_IA_SM_CON_PART_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "XCM: STORM Connection IA part buffer is not empty"
	# Read register XCM_REG_IA_TRANS_PART_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_IA_TRANS_PART_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "XCM: Transparent part buffer is not empty"
	# Read register XCM_REG_XX_CBYP_TBL_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_XX_CBYP_TBL_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "XCM: XX Connection bypass table is not empty"
	# Read register XCM_REG_IN_PRCS_TBL_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_IN_PRCS_TBL_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "XCM: In-process table is not empty"
	# Read register XCM_REG_FI_DESC_INPUT_VIOLATE val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_FI_DESC_INPUT_VIOLATE "(\$val != 0)" IDLE_CHK_ERROR "XCM: Input violation in message first descriptor"
	# Verify values of memory registers
	IDLE_CHK_11 0x3f XCM_REG_QM_ACT_ST_CNT "(\$val != 0)" IDLE_CHK_ERROR "XCM: QM Active State Counter is not 0"
	# Read register YCM_REG_XX_LCID_CAM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_XX_LCID_CAM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "YCM: XX protection CAM is not empty"
	# Read register YCM_REG_XX_LOCK_CNT val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_XX_LOCK_CNT "(\$val != 0)" IDLE_CHK_ERROR "YCM: there is locked LCID in XX"
	# Read register YCM_REG_XX_CBYP_TBL_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_XX_CBYP_TBL_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "YCM: XX Cbypass Table is not empty"
	# Read register YCM_REG_XX_TBYP_TBL_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_XX_TBYP_TBL_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "YCM: XX Tbypass Table is not empty"
	# Read register YCM_REG_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_INT_STS_0 "(\$val != 0)" IDLE_CHK_ERROR "YCM: Interrupt status 0 is not 0"
	# Read register YCM_REG_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_INT_STS_1 "(\$val != 0)" IDLE_CHK_ERROR "YCM: Interrupt status 1 is not 0"
	# Check reset value
	IDLE_CHK_10 0x3f YCM_REG_CCFC_INIT_CRD "(\$val1 != \$val2)" IDLE_CHK_ERROR "YCM: CCFC_INIT_CRD is different than reset value"
	# Check reset value
	IDLE_CHK_10 0x3f YCM_REG_TCFC_INIT_CRD "(\$val1 != \$val2)" IDLE_CHK_ERROR "YCM: TCFC_INIT_CRD is different than reset value"
	# Check reset value
	IDLE_CHK_10 0x3f YCM_REG_QM_INIT_CRD0 "(\$val1 != \$val2)" IDLE_CHK_ERROR "YCM: QM_INIT_CRD0 is different than reset value"
	# Check reset value
	IDLE_CHK_10 0x3f YCM_REG_FIC_INIT_CRD "(\$val1 != \$val2)" IDLE_CHK_ERROR "YCM: FIC_INIT_CRD is different than reset value"
	# Check reset value
	IDLE_CHK_10 0x3f YCM_REG_XX_FREE_CNT "(\$val1 != \$val2)" IDLE_CHK_ERROR "YCM: XX_FREE_CNT is different than reset value"
	# Read register YCM_REG_YSDM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_YSDM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "YCM: YSDM declared message length unequal to actual"
	# Read register YCM_REG_PBF_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_PBF_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "YCM: PBF declared message length unequal to actual"
	# Read register YCM_REG_MSDM_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_MSDM_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "YCM: MSDM declared message length unequal to actual"
	# Read register YCM_REG_XYLD_LENGTH_MIS val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_XYLD_LENGTH_MIS "(\$val != 0)" IDLE_CHK_ERROR "YCM: XYLD declared message length unequal to actual"
	# Read register YCM_REG_IS_STORM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_IS_STORM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "YCM: Local STORM input stage buffer is not empty"
	# Read register YCM_REG_IS_QM_P_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_IS_QM_P_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "YCM: QM_P input stage buffer is not empty"
	# Read register YCM_REG_IS_QM_S_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_IS_QM_S_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "YCM: QM_S input stage buffer is not empty"
	# Read register YCM_REG_IS_YSDM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_IS_YSDM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "YCM: YSDM input stage buffer is not empty"
	# Read register YCM_REG_IS_PBF_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_IS_PBF_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "YCM: PBF input stage buffer is not empty"
	# Read register YCM_REG_IS_MSEM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_IS_MSEM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "YCM: MSEM input stage buffer is not empty"
	# Read register YCM_REG_IS_USEM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_IS_USEM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "YCM: USEM input stage buffer is not empty"
	# Read register YCM_REG_IS_XYLD_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_IS_XYLD_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "YCM: XYLD input stage buffer is not empty"
	# Read register YCM_REG_IS_MSDM_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_IS_MSDM_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "YCM: MSDM input stage buffer is not empty"
	# Read register YCM_REG_SM_CON_FIC_BUF_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_SM_CON_FIC_BUF_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "YCM: STORM Connection FIC buffer is not empty"
	# Read register YCM_REG_AGG_TASK_FIC_BUF_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_AGG_TASK_FIC_BUF_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "YCM: Aggregation Task FIC buffer is not empty"
	# Read register YCM_REG_SM_TASK_FIC_BUF_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_SM_TASK_FIC_BUF_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "YCM: STORM Task FIC buffer is not empty"
	# Read register YCM_REG_IA_SM_CON_PART_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_IA_SM_CON_PART_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "YCM: STORM Connection IA part buffer is not empty"
	# Read register YCM_REG_IA_AGG_TASK_PART_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_IA_AGG_TASK_PART_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "YCM: Aggregation Task IA part buffer is not empty"
	# Read register YCM_REG_IA_SM_TASK_PART_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_IA_SM_TASK_PART_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "YCM: STORM Task IA part buffer is not empty"
	# Read register YCM_REG_IA_TRANS_PART_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_IA_TRANS_PART_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "YCM: Transparent part buffer is not empty"
	# Read register YCM_REG_XX_CBYP_TBL_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_XX_CBYP_TBL_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "YCM: XX Connection bypass table is not empty"
	# Read register YCM_REG_XX_TBYP_TBL_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_XX_TBYP_TBL_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "YCM: XX Task bypass table is not empty"
	# Read register YCM_REG_IN_PRCS_TBL_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_IN_PRCS_TBL_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "YCM: In-process table is not empty"
	# Read register YCM_REG_FI_DESC_INPUT_VIOLATE val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_FI_DESC_INPUT_VIOLATE "(\$val != 0)" IDLE_CHK_ERROR "YCM: Input violation in message first descriptor"
	# Read register YCM_REG_SE_DESC_INPUT_VIOLATE val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_SE_DESC_INPUT_VIOLATE "(\$val != 0)" IDLE_CHK_ERROR "YCM: Input violation in message second descriptor"
	# Read register QM_REG_WRC_FIFOLVL_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f QM_REG_WRC_FIFOLVL_0 "(\$val != 0)" IDLE_CHK_ERROR "QM: wrc_fifolvl_0 is not 0 (see qm_rf for cm idx enumeration)"
	# Read register QM_REG_WRC_FIFOLVL_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f QM_REG_WRC_FIFOLVL_1 "(\$val != 0)" IDLE_CHK_ERROR "QM: wrc_fifolvl_1 is not 0 (see qm_rf for cm idx enumeration)"
	# Read register QM_REG_WRC_FIFOLVL_2 val and check if condition on val exist
	IDLE_CHK_1 0x3f QM_REG_WRC_FIFOLVL_2 "(\$val != 0)" IDLE_CHK_ERROR "QM: wrc_fifolvl_2 is not 0 (see qm_rf for cm idx enumeration)"
	# Read register QM_REG_WRC_FIFOLVL_3 val and check if condition on val exist
	IDLE_CHK_1 0x3f QM_REG_WRC_FIFOLVL_3 "(\$val != 0)" IDLE_CHK_ERROR "QM: wrc_fifolvl_3 is not 0 (see qm_rf for cm idx enumeration)"
	# Read register QM_REG_WRC_FIFOLVL_4 val and check if condition on val exist
	IDLE_CHK_1 0x3f QM_REG_WRC_FIFOLVL_4 "(\$val != 0)" IDLE_CHK_ERROR "QM: wrc_fifolvl_4 is not 0 (see qm_rf for cm idx enumeration)"
	# Read register QM_REG_WRC_FIFOLVL_5 val and check if condition on val exist
	IDLE_CHK_1 0x3f QM_REG_WRC_FIFOLVL_5 "(\$val != 0)" IDLE_CHK_ERROR "QM: wrc_fifolvl_5 is not 0 (see qm_rf for cm idx enumeration)"
	# Read register QM_REG_OUTLDREQCRDCONNTX val1 and register QM_REG_OUTLDREQSIZECONNTX val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f QM_REG_OUTLDREQCRDCONNTX QM_REG_OUTLDREQSIZECONNTX "(\$val1 != \$val2)" IDLE_CHK_ERROR "QM: CCFC load request buffer credit for TX PQ-s is not equal to the initial credit"
	# Read register QM_REG_OUTLDREQCRDCONNOTHER val1 and register QM_REG_OUTLDREQSIZECONNOTHER val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f QM_REG_OUTLDREQCRDCONNOTHER QM_REG_OUTLDREQSIZECONNOTHER "(\$val1 != \$val2)" IDLE_CHK_ERROR "QM: CCFC/TCFC load request buffer credit for Other PQ-s is not equal to the initial credit"
	# Special check for QM PTRTBL
	IDLE_CHK_6 0xf QM_REG_PTRTBLTX 448 8 IDLE_CHK_ERROR_NO_TRAFFIC
	# Special check for QM PTRTBL
	IDLE_CHK_6 0x30 QM_REG_PTRTBLTX 512 8 IDLE_CHK_ERROR_NO_TRAFFIC
	# Special check for QM PTRTBL
	IDLE_CHK_6 0xf QM_REG_PTRTBLOTHER 64 8 IDLE_CHK_ERROR_NO_TRAFFIC
	# Special check for QM PTRTBL
	IDLE_CHK_6 0x30 QM_REG_PTRTBLOTHER 128 8 IDLE_CHK_ERROR_NO_TRAFFIC
	# Read register QM_REG_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f QM_REG_INT_STS "(\$val != 0)" IDLE_CHK_ERROR "QM: Interrupt status is not 0"
	# Read register QM_REG_QSTATUSTX_0 val in loop (incr by 4) and check if condition on val exist
	IDLE_CHK_2 0x3f QM_REG_QSTATUSTX_0 14 4 "(\$val != 0)" IDLE_CHK_ERROR "QM: QSTATUSTX reg is not 0. "
	# Read register QM_REG_QSTATUSOTHER_0 val in loop (incr by 4) and check if condition on val exist
	IDLE_CHK_2 0x3f QM_REG_QSTATUSOTHER_0 2 4 "(\$val != 0)" IDLE_CHK_ERROR "QM: QSTATUSOTHER reg is not 0. "
	# Read register QM_REG_OVFQNUMTX val and check if condition on val exist
	IDLE_CHK_1 0x3f QM_REG_OVFQNUMTX "(\$val != 0)" IDLE_CHK_ERROR "QM: OVFQNUMTX is not 0"
	# Read register QM_REG_OVFQNUMOTHER val and check if condition on val exist
	IDLE_CHK_1 0x3f QM_REG_OVFQNUMOTHER "(\$val != 0)" IDLE_CHK_ERROR "QM: OVFQNUMOTHER is not 0"
	# Read register QM_REG_OVFERRORTX val and check if condition on val exist
	IDLE_CHK_1 0x3f QM_REG_OVFERRORTX "(\$val != 0)" IDLE_CHK_ERROR "QM: OVFERRORTX is not 0"
	# Read register QM_REG_OVFERROROTHER val and check if condition on val exist
	IDLE_CHK_1 0x3f QM_REG_OVFERROROTHER "(\$val != 0)" IDLE_CHK_ERROR "QM: OVFERROROTHER is not 0"
	# Compare memories
	IDLE_CHK_9 0x3f QM_REG_VOQCRDLINE QM_REG_VOQINITCRDLINE "(\$val1 != \$val2)" IDLE_CHK_ERROR "QM: VOQ line credit is not equal to init credit"
	# Compare memories
	IDLE_CHK_9 0x3f QM_REG_VOQCRDBYTE QM_REG_VOQINITCRDBYTE "(\$val1 != \$val2)" IDLE_CHK_ERROR "QM: VOQ byte credit is not equal to init credit"
	# Read register QM_REG_CMCRD_0 val1 and register QM_REG_CMINITCRD_0 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f QM_REG_CMCRD_0 QM_REG_CMINITCRD_0 "(\$val1 != \$val2)" IDLE_CHK_ERROR "QM: CM credit on the pop interface #0 is not equal to init val (see qm_rf for cm idx enumeration)"
	# Read register QM_REG_CMCRD_1 val1 and register QM_REG_CMINITCRD_1 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f QM_REG_CMCRD_1 QM_REG_CMINITCRD_1 "(\$val1 != \$val2)" IDLE_CHK_ERROR "QM: CM credit on the pop interface #1 is not equal to init val (see qm_rf for cm idx enumeration)"
	# Read register QM_REG_CMCRD_2 val1 and register QM_REG_CMINITCRD_2 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f QM_REG_CMCRD_2 QM_REG_CMINITCRD_2 "(\$val1 != \$val2)" IDLE_CHK_ERROR "QM: CM credit on the pop interface #2 is not equal to init val (see qm_rf for cm idx enumeration)"
	# Read register QM_REG_CMCRD_3 val1 and register QM_REG_CMINITCRD_3 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f QM_REG_CMCRD_3 QM_REG_CMINITCRD_3 "(\$val1 != \$val2)" IDLE_CHK_ERROR "QM: CM credit on the pop interface #3 is not equal to init val (see qm_rf for cm idx enumeration)"
	# Read register QM_REG_CMCRD_4 val1 and register QM_REG_CMINITCRD_4 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f QM_REG_CMCRD_4 QM_REG_CMINITCRD_4 "(\$val1 != \$val2)" IDLE_CHK_ERROR "QM: CM credit on the pop interface #4 is not equal to init val (see qm_rf for cm idx enumeration)"
	# Read register QM_REG_CMCRD_5 val1 and register QM_REG_CMINITCRD_5 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f QM_REG_CMCRD_5 QM_REG_CMINITCRD_5 "(\$val1 != \$val2)" IDLE_CHK_ERROR "QM: CM credit on the pop interface #5 is not equal to init val (see qm_rf for cm idx enumeration)"
	# Read register QM_REG_CMCRD_6 val1 and register QM_REG_CMINITCRD_6 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f QM_REG_CMCRD_6 QM_REG_CMINITCRD_6 "(\$val1 != \$val2)" IDLE_CHK_ERROR "QM: CM credit on the pop interface #6 is not equal to init val (see qm_rf for cm idx enumeration)"
	# Read register QM_REG_CMCRD_7 val1 and register QM_REG_CMINITCRD_7 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f QM_REG_CMCRD_7 QM_REG_CMINITCRD_7 "(\$val1 != \$val2)" IDLE_CHK_ERROR "QM: CM credit on the pop interface #7 is not equal to init val (see qm_rf for cm idx enumeration)"
	# Read register QM_REG_CMCRD_8 val1 and register QM_REG_CMINITCRD_8 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f QM_REG_CMCRD_8 QM_REG_CMINITCRD_8 "(\$val1 != \$val2)" IDLE_CHK_ERROR "QM: CM credit on the pop interface #8 is not equal to init val (see qm_rf for cm idx enumeration)"
	# Read register QM_REG_CMCRD_9 val1 and register QM_REG_CMINITCRD_9 val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f QM_REG_CMCRD_9 QM_REG_CMINITCRD_9 "(\$val1 != \$val2)" IDLE_CHK_ERROR "QM: CM credit on the pop interface #9 is not equal to init val (see qm_rf for cm idx enumeration)"
	# Read register IGU_REG_ERROR_HANDLING_DATA_VALID val and check if condition on val exist
	IDLE_CHK_1 0x3f IGU_REG_ERROR_HANDLING_DATA_VALID "(\$val  != 0)" IDLE_CHK_ERROR "IGU: some unauthorized commands arrived to the IGU. Use igu_dump_fifo utility for more details."
	# Read register IGU_REG_ATTN_WRITE_DONE_PENDING val and check if condition on val exist
	IDLE_CHK_1 0x3f IGU_REG_ATTN_WRITE_DONE_PENDING "(\$val  != 0)" IDLE_CHK_WARNING "IGU attention message write done pending is not empty"
	# Read register IGU_REG_WRITE_DONE_PENDING val and check if condition on val exist
	IDLE_CHK_1 0x3f IGU_REG_WRITE_DONE_PENDING "(\$val  != 0)" IDLE_CHK_WARNING "IGU MSI/MSIX message write done pending is not empty"
	# Read register IGU_REG_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f IGU_REG_PRTY_STS "(\$val != 0)" IDLE_CHK_WARNING "IGU: parity status is not 0"
	# Read register IGU_REG_SILENT_DROP val and check if condition on val exist
	IDLE_CHK_1 0x3f IGU_REG_SILENT_DROP "(\$val  != 0)" IDLE_CHK_ERROR "Some messages were not executed in the IGU."
	# Read register IGU_REG_SB_CTRL_FSM val and check if condition on val exist
	IDLE_CHK_1 0x3f IGU_REG_SB_CTRL_FSM "(\$val  != 0)" IDLE_CHK_WARNING "IGU: block is not in idle. SB_CTRL_FSM should be zero in idle state"
	# Read register IGU_REG_INT_HANDLE_FSM val and check if condition on val exist
	IDLE_CHK_1 0x3f IGU_REG_INT_HANDLE_FSM "(\$val  != 0)" IDLE_CHK_WARNING "IGU: block is not in idle. INT_HANDLE_FSM should be zero in idle state"
	# Read register IGU_REG_ATTN_FSM val and check if condition on val exist
	IDLE_CHK_1 0x3f IGU_REG_ATTN_FSM "((\$val & ~0x2)  != 0)" IDLE_CHK_WARNING "IGU: block is not in idle. SB_ATTN_FSMshould be zeroor two in idle state"
	# Read register IGU_REG_CTRL_FSM val and check if condition on val exist
	IDLE_CHK_1 0x3f IGU_REG_CTRL_FSM "((\$val & ~0x1)  != 0)" IDLE_CHK_WARNING "IGU: block is not in idle. SB_CTRL_FSM should be zero in idle state"
	# Read register IGU_REG_PXP_ARB_FSM val and check if condition on val exist
	IDLE_CHK_1 0x3f IGU_REG_PXP_ARB_FSM "((\$val & ~0x1)  != 0)" IDLE_CHK_WARNING "IGU: block is not in idle. SB_ARB_FSM should be zero in idle state"
	# Read register IGU_REG_PENDING_BITS_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f IGU_REG_PENDING_BITS_STATUS "(\$val  != 0)" IDLE_CHK_WARNING "IGU: block is not in idle. There are pending MSI/MSIX messeges"
	# Read register IGU_REG_INTERRUPT_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f IGU_REG_INTERRUPT_STATUS "(\$val  !=0 )" IDLE_CHK_WARNING "IGU: block is not in idle. Some interrupts are asserted"
	# Read register CAU_REG_PARITY_LATCH_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f CAU_REG_PARITY_LATCH_STATUS "(\$val  !=0 )" IDLE_CHK_ERROR "CAU: Fatal parity error occurred in CAU"
	# Read register CAU_REG_ERROR_PXP_REQ val and check if condition on val exist
	IDLE_CHK_1 0x3f CAU_REG_ERROR_PXP_REQ "(\$val  !=0 )" IDLE_CHK_ERROR "CAU: Fatal unauthorized command arrived from PXP"
	# Read register CAU_REG_ERROR_FSM_LINE val and check if condition on val exist
	IDLE_CHK_1 0x3f CAU_REG_ERROR_FSM_LINE "(\$val  !=0 )" IDLE_CHK_ERROR "CAU: Fatal. CAU arrived to invalid FSM line."
	# Read register CAU_REG_ERROR_CLEANUP_CMD_REG val and check if condition on val exist
	IDLE_CHK_1 0x3f CAU_REG_ERROR_CLEANUP_CMD_REG "(\$val  !=0 )" IDLE_CHK_ERROR "CAU: Fatal unauthorized cleanup command arrived from GRC"
	# Read register CAU_REG_AGG_UNITS_0TO15_STATE val and check if condition on val exist
	IDLE_CHK_1 0x3f CAU_REG_AGG_UNITS_0TO15_STATE "(\$val  !=0 )" IDLE_CHK_WARNING "cau: Block is not in idle"
	# Read register CAU_REG_AGG_UNITS_16TO31_STATE val and check if condition on val exist
	IDLE_CHK_1 0x3f CAU_REG_AGG_UNITS_16TO31_STATE "(\$val  !=0 )" IDLE_CHK_WARNING "cau: Block is not in idle"
	# Read register CAU_REG_AGG_UNITS_32TO47_STATE val and check if condition on val exist
	IDLE_CHK_1 0x3f CAU_REG_AGG_UNITS_32TO47_STATE "(\$val  !=0 )" IDLE_CHK_WARNING "cau: Block is not in idle"
	# Read register CAU_REG_AGG_UNITS_48TO63_STATE val and check if condition on val exist
	IDLE_CHK_1 0x3f CAU_REG_AGG_UNITS_48TO63_STATE "(\$val  !=0 )" IDLE_CHK_WARNING "cau: Block is not in idle"
	# Read register CAU_REG_DEBUG_FIFO_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f CAU_REG_DEBUG_FIFO_STATUS "(\$val  !=0 )" IDLE_CHK_WARNING "cau: Block is not in idle"
	# Read register CAU_REG_MAIN_FSM_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f CAU_REG_MAIN_FSM_STATUS "(\$val  !=0 )" IDLE_CHK_WARNING "cau: Block is not in idle"
	# Read register CAU_REG_VAR_READ_FSM_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f CAU_REG_VAR_READ_FSM_STATUS "(\$val  !=0 )" IDLE_CHK_WARNING "cau: Block is not in idle"
	# Read register CAU_REG_IGU_DMA_FSM_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f CAU_REG_IGU_DMA_FSM_STATUS "(\$val  !=0 )" IDLE_CHK_WARNING "cau: Block is not in idle"
	# Read register CAU_REG_IGU_CQE_CMD_FSM_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f CAU_REG_IGU_CQE_CMD_FSM_STATUS "(\$val  !=0 )" IDLE_CHK_WARNING "cau: Block is not in idle"
	# Read register CAU_REG_IGU_CQE_AGG_FSM_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f CAU_REG_IGU_CQE_AGG_FSM_STATUS "(\$val  !=0 )" IDLE_CHK_WARNING "cau: Block is not in idle"
	# Read register CAU_REG_REQ_COUNTER val1 and register CAU_REG_ACK_COUNTER val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f CAU_REG_REQ_COUNTER CAU_REG_ACK_COUNTER "(\$val1 != \$val2)" IDLE_CHK_WARNING "cau: Block is not in idle"
	# Read register CAU_REG_REQ_COUNTER val1 and register CAU_REG_WDONE_COUNTER val2 and check if condition on val1,val2 exist
	IDLE_CHK_3 0x3f CAU_REG_REQ_COUNTER CAU_REG_WDONE_COUNTER "(\$val1 != \$val2)" IDLE_CHK_WARNING "cau: Block is not in idle"
	# Read register CAU_REG_IGU_REQ_CREDIT_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f CAU_REG_IGU_REQ_CREDIT_STATUS "(\$val  !=1 )" IDLE_CHK_WARNING "cau: Block is not in idle"
	# Read register CAU_REG_IGU_CMD_CREDIT_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f CAU_REG_IGU_CMD_CREDIT_STATUS "(\$val  !=1 )" IDLE_CHK_WARNING "cau: Block is not in idle"
	# Read register NIG_REG_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f NIG_REG_INT_STS_1 "((\$val & 0xffffffff)!= 0)" IDLE_CHK_ERROR "NIG: Error in SOPQs."
	# Read register NIG_REG_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f NIG_REG_INT_STS_0 "((\$val & 0xffffffff)!= 0)" IDLE_CHK_ERROR "NIG: Error in FIFOs or accessed address."
	# Read register NIG_REG_TX_BTB_FIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f NIG_REG_TX_BTB_FIFO_EMPTY "(\$val  !=1 )" IDLE_CHK_ERROR_NO_TRAFFIC "NIG: Error in BTB FIFO for TX path for the port ."
	# Read register NIG_REG_LB_BTB_FIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f NIG_REG_LB_BTB_FIFO_EMPTY "(\$val  !=1 )" IDLE_CHK_ERROR_NO_TRAFFIC "NIG: Error in BTB FIFO for LB path for the port ."
	# Read register NIG_REG_DEBUG_FIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f NIG_REG_DEBUG_FIFO_EMPTY "(\$val  !=1 )" IDLE_CHK_ERROR_NO_TRAFFIC "NIG: Debug FIFO is not empty"
	# Read register NIG_REG_RX_LLH_RFIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f NIG_REG_RX_LLH_RFIFO_EMPTY "(\$val  !=1 )" IDLE_CHK_ERROR_NO_TRAFFIC "NIG: RX LLH RFIFO for the port is not empty"
	# Read register NIG_REG_LB_LLH_RFIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f NIG_REG_LB_LLH_RFIFO_EMPTY "(\$val  !=1 )" IDLE_CHK_ERROR_NO_TRAFFIC "NIG: LB LLH RFIFO for the port is not empty"
	# Read register NIG_REG_RX_MACFIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f NIG_REG_RX_MACFIFO_EMPTY "(\$val  !=1 )" IDLE_CHK_ERROR_NO_TRAFFIC "NIG: RX MAC FIFO for the port is not empty"
	# Read register NIG_REG_TX_MACFIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f NIG_REG_TX_MACFIFO_EMPTY "(\$val  !=1 )" IDLE_CHK_ERROR_NO_TRAFFIC "NIG: TX MAC FIFO for the port is not empty"
	# Read register NIG_REG_LB_SOPQ_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f NIG_REG_LB_SOPQ_EMPTY "(\$val  !=0x000fffff )" IDLE_CHK_ERROR_NO_TRAFFIC "NIG: LB SOPQs are not empty"
	# Read register NIG_REG_TX_SOPQ_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f NIG_REG_TX_SOPQ_EMPTY "(\$val  !=0x0000ffff )" IDLE_CHK_ERROR_NO_TRAFFIC "NIG: TX SOPQs are not empty"
	# Read register DORQ_REG_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f DORQ_REG_INT_STS "(\$val != 0)" IDLE_CHK_ERROR "DORQ: Interrupt Status is not 0"
	# Read register DORQ_REG_DORQ_FIFO_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f DORQ_REG_DORQ_FIFO_FILL_LVL "(\$val != 0)" IDLE_CHK_ERROR "DORQ: DORQ FIFO is not empty"
	# Check reset value
	IDLE_CHK_10 0x3f DORQ_REG_XCM_MSG_INIT_CRD "(\$val1 != \$val2)" IDLE_CHK_ERROR "DORQ: Credit to XCM is different than default value"
	# Check reset value
	IDLE_CHK_10 0x3f DORQ_REG_UCM_MSG_INIT_CRD "(\$val1 != \$val2)" IDLE_CHK_ERROR "DORQ: Credit to UCM is different than default value"
	# Check reset value
	IDLE_CHK_10 0x3f DORQ_REG_TCM_MSG_INIT_CRD "(\$val1 != \$val2)" IDLE_CHK_ERROR "DORQ: Credit to TCM is different than default value"
	# Check reset value
	IDLE_CHK_10 0x3f DORQ_REG_PBF_CMD_INIT_CRD "(\$val1 != \$val2)" IDLE_CHK_ERROR "DORQ: Credit to PBF is different than default value"
	# Read register DORQ_REG_PF_USAGE_CNT val and check if condition on val exist
	IDLE_CHK_1 0x3f DORQ_REG_PF_USAGE_CNT "(\$val1 != 0)" IDLE_CHK_ERROR "DORQ: PF Usage counter is not 0"
	# Read register DORQ_REG_VF_USAGE_CNT val and check if condition on val exist
	IDLE_CHK_1 0x3f DORQ_REG_VF_USAGE_CNT "(\$val1 != 0)" IDLE_CHK_ERROR "DORQ: VF Usage counter is not 0"
	# Read register DORQ_REG_CFC_LD_REQ_FIFO_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f DORQ_REG_CFC_LD_REQ_FIFO_FILL_LVL "(\$val1 != 0)" IDLE_CHK_ERROR "DORQ: CFC load request FIFO current fill level is not 0"
	# Read register DORQ_REG_DPM_TBL_FILL_LVL val and check if condition on val exist
	IDLE_CHK_1 0x3f DORQ_REG_DPM_TBL_FILL_LVL "(\$val1 != 0)" IDLE_CHK_ERROR "DORQ: DPM Table  current fill level is not 0"
	# Read register PRS_REG_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PRS_REG_INT_STS_0 "(\$val != 0)" IDLE_CHK_ERROR "PRS: Interrupt Status is not 0"
	# Read register PRS_REG_CCFC_SEARCH_CURRENT_CREDIT val and check if condition on val exist
	IDLE_CHK_1 0x3f PRS_REG_CCFC_SEARCH_CURRENT_CREDIT "(\$val != 0)" IDLE_CHK_ERROR "PRS: CCFC Search Credit Count is not 0 "
	# Read register PRS_REG_TCFC_SEARCH_CURRENT_CREDIT val and check if condition on val exist
	IDLE_CHK_1 0x3f PRS_REG_TCFC_SEARCH_CURRENT_CREDIT "(\$val != 0)" IDLE_CHK_ERROR "PRS: TCFC Search Credit Count is not 0 "
	# Read register PRS_REG_TCM_CURRENT_CREDIT val and check if condition on val exist
	IDLE_CHK_1 0x3f PRS_REG_TCM_CURRENT_CREDIT "(\$val != 0)" IDLE_CHK_ERROR "PRS: TCM Credit Count is not 0 "
	# Read register TSEM_REG_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f TSEM_REG_INT_STS_0 "(\$val != 0)" IDLE_CHK_ERROR "TSEM: interrupt status 0 is not 0"
	# Read register TSEM_REG_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f TSEM_REG_INT_STS_1 "(\$val != 0)" IDLE_CHK_ERROR "TSEM: interrupt status 1 is not 0"
	# Read register TSEM_REG_DRA_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSEM_REG_DRA_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "TSEM: DRA is not empty"
	# Read register TSEM_REG_EXT_PAS_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSEM_REG_EXT_PAS_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "TSEM: External passive is not empty"
	# Read register TSEM_REG_EXT_STORE_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f TSEM_REG_EXT_STORE_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "TSEM: External store IF is full"
	# Read register TSEM_REG_PAS_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f TSEM_REG_PAS_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "TSEM: Passive IF is full"
	# Read register TSEM_REG_RAM_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f TSEM_REG_RAM_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "TSEM: RAM IF is full"
	# Read register TSEM_REG_SLOW_DBG_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSEM_REG_SLOW_DBG_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "TSEM: Slow dbg fifo is not empty"
	# Read register TSEM_REG_SLOW_DRA_FIN_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSEM_REG_SLOW_DRA_FIN_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "TSEM: Slow dra fin fifo is not empty"
	# Read register TSEM_REG_SLOW_DRA_RD_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSEM_REG_SLOW_DRA_RD_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "TSEM: Slow dra read fifo is not empty"
	# Read register TSEM_REG_SLOW_DRA_WR_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSEM_REG_SLOW_DRA_WR_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "TSEM: Slow dra write fifo is not empty"
	# Read register TSEM_REG_SLOW_EXT_STORE_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSEM_REG_SLOW_EXT_STORE_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "TSEM: Slow ext store fifo is not empty"
	# Read register TSEM_REG_SLOW_EXT_LOAD_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSEM_REG_SLOW_EXT_LOAD_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "TSEM: Slow ext load fifo is not empty"
	# Read register TSEM_REG_SLOW_RAM_RD_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSEM_REG_SLOW_RAM_RD_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "TSEM: Slow ram read fifo is not empty"
	# Read register TSEM_REG_SLOW_RAM_WR_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSEM_REG_SLOW_RAM_WR_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "TSEM: Slow ram write fifo is not empty"
	# Read register TSEM_REG_SYNC_DBG_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSEM_REG_SYNC_DBG_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "TSEM: SYNC DBG fifo is not empty"
	# Read register TSEM_REG_THREAD_FIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSEM_REG_THREAD_FIFO_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "TSEM: Thread fifo is not empty"
	# Read register TSEM_REG_THREAD_RDY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSEM_REG_THREAD_RDY "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "TSEM: Some threads are ready"
	# Read register TSEM_REG_THREAD_VALID val and check if condition on val exist
	IDLE_CHK_1 0x3f TSEM_REG_THREAD_VALID "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "TSEM: some threads are valid"
	# Read register MSEM_REG_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f MSEM_REG_INT_STS_0 "(\$val != 0)" IDLE_CHK_ERROR "MSEM: interrupt status 0 is not 0"
	# Read register MSEM_REG_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f MSEM_REG_INT_STS_1 "(\$val != 0)" IDLE_CHK_ERROR "MSEM: interrupt status 1 is not 0"
	# Read register MSEM_REG_DRA_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSEM_REG_DRA_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "MSEM: DRA is not empty"
	# Read register MSEM_REG_EXT_PAS_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSEM_REG_EXT_PAS_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "MSEM: External passive is not empty"
	# Read register MSEM_REG_EXT_STORE_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f MSEM_REG_EXT_STORE_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "MSEM: External store IF is full"
	# Read register MSEM_REG_PAS_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f MSEM_REG_PAS_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "MSEM: Passive IF is full"
	# Read register MSEM_REG_RAM_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f MSEM_REG_RAM_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "MSEM: RAM IF is full"
	# Read register MSEM_REG_SLOW_DBG_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSEM_REG_SLOW_DBG_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "MSEM: Slow dbg fifo is not empty"
	# Read register MSEM_REG_SLOW_DRA_FIN_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSEM_REG_SLOW_DRA_FIN_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "MSEM: Slow dra fin fifo is not empty"
	# Read register MSEM_REG_SLOW_DRA_RD_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSEM_REG_SLOW_DRA_RD_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "MSEM: Slow dra read fifo is not empty"
	# Read register MSEM_REG_SLOW_DRA_WR_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSEM_REG_SLOW_DRA_WR_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "MSEM: Slow dra write fifo is not empty"
	# Read register MSEM_REG_SLOW_EXT_STORE_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSEM_REG_SLOW_EXT_STORE_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "MSEM: Slow ext store fifo is not empty"
	# Read register MSEM_REG_SLOW_EXT_LOAD_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSEM_REG_SLOW_EXT_LOAD_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "MSEM: Slow ext load fifo is not empty"
	# Read register MSEM_REG_SLOW_RAM_RD_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSEM_REG_SLOW_RAM_RD_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "MSEM: Slow ram read fifo is not empty"
	# Read register MSEM_REG_SLOW_RAM_WR_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSEM_REG_SLOW_RAM_WR_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "MSEM: Slow ram write fifo is not empty"
	# Read register MSEM_REG_SYNC_DBG_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSEM_REG_SYNC_DBG_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "MSEM: SYNC DBG fifo is not empty"
	# Read register MSEM_REG_THREAD_FIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSEM_REG_THREAD_FIFO_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "MSEM: Thread fifo is not empty"
	# Read register MSEM_REG_THREAD_RDY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSEM_REG_THREAD_RDY "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "MSEM: Some threads are ready"
	# Read register MSEM_REG_THREAD_VALID val and check if condition on val exist
	IDLE_CHK_1 0x3f MSEM_REG_THREAD_VALID "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "MSEM: some threads are valid"
	# Read register XSEM_REG_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f XSEM_REG_INT_STS_0 "(\$val != 0)" IDLE_CHK_ERROR "XSEM: interrupt status 0 is not 0"
	# Read register XSEM_REG_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f XSEM_REG_INT_STS_1 "(\$val != 0)" IDLE_CHK_ERROR "XSEM: interrupt status 1 is not 0"
	# Read register XSEM_REG_DRA_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSEM_REG_DRA_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "XSEM: DRA is not empty"
	# Read register XSEM_REG_EXT_PAS_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSEM_REG_EXT_PAS_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "XSEM: External passive is not empty"
	# Read register XSEM_REG_EXT_STORE_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f XSEM_REG_EXT_STORE_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "XSEM: External store IF is full"
	# Read register XSEM_REG_PAS_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f XSEM_REG_PAS_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "XSEM: Passive IF is full"
	# Read register XSEM_REG_RAM_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f XSEM_REG_RAM_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "XSEM: RAM IF is full"
	# Read register XSEM_REG_SLOW_DBG_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSEM_REG_SLOW_DBG_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "XSEM: Slow dbg fifo is not empty"
	# Read register XSEM_REG_SLOW_DRA_FIN_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSEM_REG_SLOW_DRA_FIN_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "XSEM: Slow dra fin fifo is not empty"
	# Read register XSEM_REG_SLOW_DRA_RD_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSEM_REG_SLOW_DRA_RD_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "XSEM: Slow dra read fifo is not empty"
	# Read register XSEM_REG_SLOW_DRA_WR_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSEM_REG_SLOW_DRA_WR_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "XSEM: Slow dra write fifo is not empty"
	# Read register XSEM_REG_SLOW_EXT_STORE_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSEM_REG_SLOW_EXT_STORE_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "XSEM: Slow ext store fifo is not empty"
	# Read register XSEM_REG_SLOW_EXT_LOAD_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSEM_REG_SLOW_EXT_LOAD_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "XSEM: Slow ext load fifo is not empty"
	# Read register XSEM_REG_SLOW_RAM_RD_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSEM_REG_SLOW_RAM_RD_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "XSEM: Slow ram read fifo is not empty"
	# Read register XSEM_REG_SLOW_RAM_WR_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSEM_REG_SLOW_RAM_WR_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "XSEM: Slow ram write fifo is not empty"
	# Read register XSEM_REG_SYNC_DBG_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSEM_REG_SYNC_DBG_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "XSEM: SYNC DBG fifo is not empty"
	# Read register XSEM_REG_THREAD_FIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSEM_REG_THREAD_FIFO_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "XSEM: Thread fifo is not empty"
	# Read register XSEM_REG_THREAD_RDY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSEM_REG_THREAD_RDY "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "XSEM: Some threads are ready"
	# Read register XSEM_REG_THREAD_VALID val and check if condition on val exist
	IDLE_CHK_1 0x3f XSEM_REG_THREAD_VALID "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "XSEM: some threads are valid"
	# Read register PSEM_REG_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PSEM_REG_INT_STS_0 "(\$val != 0)" IDLE_CHK_ERROR "PSEM: interrupt status 0 is not 0"
	# Read register PSEM_REG_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f PSEM_REG_INT_STS_1 "(\$val != 0)" IDLE_CHK_ERROR "PSEM: interrupt status 1 is not 0"
	# Read register PSEM_REG_DRA_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSEM_REG_DRA_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "PSEM: DRA is not empty"
	# Read register PSEM_REG_EXT_PAS_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSEM_REG_EXT_PAS_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "PSEM: External passive is not empty"
	# Read register PSEM_REG_EXT_STORE_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f PSEM_REG_EXT_STORE_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSEM: External store IF is full"
	# Read register PSEM_REG_PAS_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f PSEM_REG_PAS_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSEM: Passive IF is full"
	# Read register PSEM_REG_RAM_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f PSEM_REG_RAM_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSEM: RAM IF is full"
	# Read register PSEM_REG_SLOW_DBG_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSEM_REG_SLOW_DBG_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "PSEM: Slow dbg fifo is not empty"
	# Read register PSEM_REG_SLOW_DRA_FIN_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSEM_REG_SLOW_DRA_FIN_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "PSEM: Slow dra fin fifo is not empty"
	# Read register PSEM_REG_SLOW_DRA_RD_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSEM_REG_SLOW_DRA_RD_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "PSEM: Slow dra read fifo is not empty"
	# Read register PSEM_REG_SLOW_DRA_WR_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSEM_REG_SLOW_DRA_WR_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "PSEM: Slow dra write fifo is not empty"
	# Read register PSEM_REG_SLOW_EXT_STORE_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSEM_REG_SLOW_EXT_STORE_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "PSEM: Slow ext store fifo is not empty"
	# Read register PSEM_REG_SLOW_EXT_LOAD_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSEM_REG_SLOW_EXT_LOAD_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "PSEM: Slow ext load fifo is not empty"
	# Read register PSEM_REG_SLOW_RAM_RD_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSEM_REG_SLOW_RAM_RD_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "PSEM: Slow ram read fifo is not empty"
	# Read register PSEM_REG_SLOW_RAM_WR_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSEM_REG_SLOW_RAM_WR_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "PSEM: Slow ram write fifo is not empty"
	# Read register PSEM_REG_SYNC_DBG_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSEM_REG_SYNC_DBG_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "PSEM: SYNC DBG fifo is not empty"
	# Read register PSEM_REG_THREAD_FIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSEM_REG_THREAD_FIFO_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "PSEM: Thread fifo is not empty"
	# Read register PSEM_REG_THREAD_RDY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSEM_REG_THREAD_RDY "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSEM: Some threads are ready"
	# Read register PSEM_REG_THREAD_VALID val and check if condition on val exist
	IDLE_CHK_1 0x3f PSEM_REG_THREAD_VALID "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PSEM: some threads are valid"
	# Read register YSEM_REG_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f YSEM_REG_INT_STS_0 "(\$val != 0)" IDLE_CHK_ERROR "YSEM: interrupt status 0 is not 0"
	# Read register YSEM_REG_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f YSEM_REG_INT_STS_1 "(\$val != 0)" IDLE_CHK_ERROR "YSEM: interrupt status 1 is not 0"
	# Read register YSEM_REG_DRA_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSEM_REG_DRA_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "YSEM: DRA is not empty"
	# Read register YSEM_REG_EXT_PAS_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSEM_REG_EXT_PAS_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "YSEM: External passive is not empty"
	# Read register YSEM_REG_EXT_STORE_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f YSEM_REG_EXT_STORE_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "YSEM: External store IF is full"
	# Read register YSEM_REG_PAS_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f YSEM_REG_PAS_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "YSEM: Passive IF is full"
	# Read register YSEM_REG_RAM_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f YSEM_REG_RAM_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "YSEM: RAM IF is full"
	# Read register YSEM_REG_SLOW_DBG_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSEM_REG_SLOW_DBG_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "YSEM: Slow dbg fifo is not empty"
	# Read register YSEM_REG_SLOW_DRA_FIN_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSEM_REG_SLOW_DRA_FIN_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "YSEM: Slow dra fin fifo is not empty"
	# Read register YSEM_REG_SLOW_DRA_RD_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSEM_REG_SLOW_DRA_RD_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "YSEM: Slow dra read fifo is not empty"
	# Read register YSEM_REG_SLOW_DRA_WR_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSEM_REG_SLOW_DRA_WR_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "YSEM: Slow dra write fifo is not empty"
	# Read register YSEM_REG_SLOW_EXT_STORE_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSEM_REG_SLOW_EXT_STORE_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "YSEM: Slow ext store fifo is not empty"
	# Read register YSEM_REG_SLOW_EXT_LOAD_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSEM_REG_SLOW_EXT_LOAD_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "YSEM: Slow ext load fifo is not empty"
	# Read register YSEM_REG_SLOW_RAM_RD_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSEM_REG_SLOW_RAM_RD_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "YSEM: Slow ram read fifo is not empty"
	# Read register YSEM_REG_SLOW_RAM_WR_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSEM_REG_SLOW_RAM_WR_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "YSEM: Slow ram write fifo is not empty"
	# Read register YSEM_REG_SYNC_DBG_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSEM_REG_SYNC_DBG_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "YSEM: SYNC DBG fifo is not empty"
	# Read register YSEM_REG_THREAD_FIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSEM_REG_THREAD_FIFO_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "YSEM: Thread fifo is not empty"
	# Read register YSEM_REG_THREAD_RDY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSEM_REG_THREAD_RDY "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "YSEM: Some threads are ready"
	# Read register YSEM_REG_THREAD_VALID val and check if condition on val exist
	IDLE_CHK_1 0x3f YSEM_REG_THREAD_VALID "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "YSEM: some threads are valid"
	# Read register USEM_REG_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f USEM_REG_INT_STS_0 "(\$val != 0)" IDLE_CHK_ERROR "USEM: interrupt status 0 is not 0"
	# Read register USEM_REG_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f USEM_REG_INT_STS_1 "(\$val != 0)" IDLE_CHK_ERROR "USEM: interrupt status 1 is not 0"
	# Read register USEM_REG_DRA_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USEM_REG_DRA_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "USEM: DRA is not empty"
	# Read register USEM_REG_EXT_PAS_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USEM_REG_EXT_PAS_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "USEM: External passive is not empty"
	# Read register USEM_REG_EXT_STORE_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f USEM_REG_EXT_STORE_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "USEM: External store IF is full"
	# Read register USEM_REG_PAS_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f USEM_REG_PAS_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "USEM: Passive IF is full"
	# Read register USEM_REG_RAM_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f USEM_REG_RAM_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "USEM: RAM IF is full"
	# Read register USEM_REG_SLOW_DBG_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USEM_REG_SLOW_DBG_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "USEM: Slow dbg fifo is not empty"
	# Read register USEM_REG_SLOW_DRA_FIN_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USEM_REG_SLOW_DRA_FIN_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "USEM: Slow dra fin fifo is not empty"
	# Read register USEM_REG_SLOW_DRA_RD_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USEM_REG_SLOW_DRA_RD_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "USEM: Slow dra read fifo is not empty"
	# Read register USEM_REG_SLOW_DRA_WR_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USEM_REG_SLOW_DRA_WR_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "USEM: Slow dra write fifo is not empty"
	# Read register USEM_REG_SLOW_EXT_STORE_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USEM_REG_SLOW_EXT_STORE_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "USEM: Slow ext store fifo is not empty"
	# Read register USEM_REG_SLOW_EXT_LOAD_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USEM_REG_SLOW_EXT_LOAD_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "USEM: Slow ext load fifo is not empty"
	# Read register USEM_REG_SLOW_RAM_RD_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USEM_REG_SLOW_RAM_RD_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "USEM: Slow ram read fifo is not empty"
	# Read register USEM_REG_SLOW_RAM_WR_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USEM_REG_SLOW_RAM_WR_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "USEM: Slow ram write fifo is not empty"
	# Read register USEM_REG_SYNC_DBG_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USEM_REG_SYNC_DBG_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "USEM: SYNC DBG fifo is not empty"
	# Read register USEM_REG_THREAD_FIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USEM_REG_THREAD_FIFO_EMPTY "(\$val != 1)" IDLE_CHK_ERROR_NO_TRAFFIC "USEM: Thread fifo is not empty"
	# Read register USEM_REG_THREAD_RDY val and check if condition on val exist
	IDLE_CHK_1 0x3f USEM_REG_THREAD_RDY "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "USEM: Some threads are ready"
	# Read register USEM_REG_THREAD_VALID val and check if condition on val exist
	IDLE_CHK_1 0x3f USEM_REG_THREAD_VALID "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "USEM: some threads are valid"
	# Read register CCFC_REG_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f CCFC_REG_INT_STS_0 "(\$val != 0)" IDLE_CHK_ERROR "CCFC: Interrupt is set"
	# Read register TCFC_REG_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f TCFC_REG_INT_STS_0 "(\$val != 0)" IDLE_CHK_ERROR "TCFC: Interrupt is set"
	# Read register TSDM_REG_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f TSDM_REG_INT_STS "(\$val != 0)" IDLE_CHK_ERROR "TSDM: Interrupt 0 is set"
	# Read register TSDM_REG_ASYNC_HOST_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSDM_REG_ASYNC_HOST_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "TSDM: TSDM_REG_ASYNC_HOST_EMPTY = 0"
	# Read register TSDM_REG_CCFC_LOAD_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSDM_REG_CCFC_LOAD_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "TSDM: CCFC_LOAD_PEND_EMPTY = 0"
	# Read register TSDM_REG_CM_DELAY_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSDM_REG_CM_DELAY_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "TSDM: TSDM_REG_CM_DELAY_EMPTY = 0"
	# Read register TSDM_REG_CMSG_QUE_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSDM_REG_CMSG_QUE_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "TSDM: TSDM_REG_CMSG_QUE_EMPTY = 0"
	# Read register TSDM_REG_DELAY_FIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSDM_REG_DELAY_FIFO_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "TSDM: TSDM_REG_DELAY_FIFO_EMPTY = 0"
	# Read register TSDM_REG_DST_BRB_SRC_ADDR_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSDM_REG_DST_BRB_SRC_ADDR_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "TSDM: TSDM_REG_DST_BRB_SRC_ADDR_EMPTY = 0"
	# Read register TSDM_REG_DST_BRB_SRC_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSDM_REG_DST_BRB_SRC_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "TSDM: TSDM_REG_DST_BRB_SRC_PEND_EMPTY = 0"
	# Read register TSDM_REG_DST_INT_RAM_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f TSDM_REG_DST_INT_RAM_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "TSDM: internal ram IF is full"
	# Read register TSDM_REG_DST_INT_RAM_WAIT_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSDM_REG_DST_INT_RAM_WAIT_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "TSDM: TSDM_REG_DST_INT_RAM_WAIT_EMPTY = 0"
	# Read register TSDM_REG_DST_PAS_BUF_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f TSDM_REG_DST_PAS_BUF_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "TSDM: passive buffer IF is full"
	# Read register TSDM_REG_DST_PAS_BUF_WAIT_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSDM_REG_DST_PAS_BUF_WAIT_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "TSDM: TSDM_REG_DST_PAS_BUF_WAIT_EMPTY = 0"
	# Read register TSDM_REG_DST_PXP_DST_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSDM_REG_DST_PXP_DST_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "TSDM: TSDM_REG_DST_PXP_DST_PEND_EMPTY = 0"
	# Read register TSDM_REG_DST_PXP_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f TSDM_REG_DST_PXP_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "TSDM: Interface from PXP is full"
	# Read register TSDM_REG_DST_PXP_IMMED_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSDM_REG_DST_PXP_IMMED_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "TSDM: Immidiate FIFo to PXP DST is not empty"
	# Read register TSDM_REG_DST_PXP_LINK_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSDM_REG_DST_PXP_LINK_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "TSDM: TSDM_REG_DST_PXP_LINK_EMPTY = 0"
	# Read register TSDM_REG_DST_PXP_SRC_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSDM_REG_DST_PXP_SRC_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "TSDM: TSDM_REG_DST_PXP_SRC_PEND_EMPTY = 0"
	# Read register TSDM_REG_QUEUE_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSDM_REG_QUEUE_EMPTY "(\$val != 511)" IDLE_CHK_ERROR "TSDM: TSDM input queues are not all empty"
	# Read register TSDM_REG_RSP_BRB_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f TSDM_REG_RSP_BRB_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "TSDM:  Interface to BRB is full"
	# Read register TSDM_REG_RSP_BRB_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSDM_REG_RSP_BRB_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "TSDM: TSDM_REG_RSP_BRB_PEND_EMPTY = 0"
	# Read register TSDM_REG_RSP_BRB_RDATA_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSDM_REG_RSP_BRB_RDATA_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "TSDM: TSDM_REG_RSP_BRB_RDATA_EMPTY = 0"
	# Read register TSDM_REG_RSP_INT_RAM_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSDM_REG_RSP_INT_RAM_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "TSDM: TSDM_REG_RSP_INT_RAM_PEND_EMPTY = 0"
	# Read register TSDM_REG_RSP_INT_RAM_RDATA_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSDM_REG_RSP_INT_RAM_RDATA_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "TSDM: TSDM_REG_RSP_INT_RAM_RDATA_EMPTY = 0"
	# Read register TSDM_REG_RSP_PXP_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f TSDM_REG_RSP_PXP_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "TSDM: Interface from PXP is full"
	# Read register TSDM_REG_RSP_PXP_RDATA_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSDM_REG_RSP_PXP_RDATA_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "TSDM: TSDM_REG_RSP_PXP_RDATA_EMPTY = 0"
	# Read register TSDM_REG_SH_DELAY_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f TSDM_REG_SH_DELAY_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "TSDM: TSDM_REG_SH_DELAY_EMPTY = 0"
	# Read register MSDM_REG_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f MSDM_REG_INT_STS "(\$val != 0)" IDLE_CHK_ERROR "MSDM: Interrupt 0 is set"
	# Read register MSDM_REG_ASYNC_HOST_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSDM_REG_ASYNC_HOST_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "MSDM: MSDM_REG_ASYNC_HOST_EMPTY = 0"
	# Read register MSDM_REG_CCFC_LOAD_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSDM_REG_CCFC_LOAD_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "MSDM: CCFC_LOAD_PEND_EMPTY = 0"
	# Read register MSDM_REG_CM_DELAY_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSDM_REG_CM_DELAY_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "MSDM: MSDM_REG_CM_DELAY_EMPTY = 0"
	# Read register MSDM_REG_CMSG_QUE_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSDM_REG_CMSG_QUE_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "MSDM: MSDM_REG_CMSG_QUE_EMPTY = 0"
	# Read register MSDM_REG_DELAY_FIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSDM_REG_DELAY_FIFO_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "MSDM: MSDM_REG_DELAY_FIFO_EMPTY = 0"
	# Read register MSDM_REG_DST_BRB_SRC_ADDR_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSDM_REG_DST_BRB_SRC_ADDR_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "MSDM: MSDM_REG_DST_BRB_SRC_ADDR_EMPTY = 0"
	# Read register MSDM_REG_DST_BRB_SRC_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSDM_REG_DST_BRB_SRC_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "MSDM: MSDM_REG_DST_BRB_SRC_PEND_EMPTY = 0"
	# Read register MSDM_REG_DST_INT_RAM_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f MSDM_REG_DST_INT_RAM_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "MSDM: internal ram IF is full"
	# Read register MSDM_REG_DST_INT_RAM_WAIT_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSDM_REG_DST_INT_RAM_WAIT_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "MSDM: MSDM_REG_DST_INT_RAM_WAIT_EMPTY = 0"
	# Read register MSDM_REG_DST_PAS_BUF_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f MSDM_REG_DST_PAS_BUF_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "MSDM: passive buffer IF is full"
	# Read register MSDM_REG_DST_PAS_BUF_WAIT_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSDM_REG_DST_PAS_BUF_WAIT_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "MSDM: MSDM_REG_DST_PAS_BUF_WAIT_EMPTY = 0"
	# Read register MSDM_REG_DST_PXP_DST_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSDM_REG_DST_PXP_DST_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "MSDM: MSDM_REG_DST_PXP_DST_PEND_EMPTY = 0"
	# Read register MSDM_REG_DST_PXP_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f MSDM_REG_DST_PXP_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "MSDM: Interface from PXP is full"
	# Read register MSDM_REG_DST_PXP_IMMED_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSDM_REG_DST_PXP_IMMED_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "MSDM: Immidiate FIFo to PXP DST is not empty"
	# Read register MSDM_REG_DST_PXP_LINK_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSDM_REG_DST_PXP_LINK_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "MSDM: MSDM_REG_DST_PXP_LINK_EMPTY = 0"
	# Read register MSDM_REG_DST_PXP_SRC_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSDM_REG_DST_PXP_SRC_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "MSDM: MSDM_REG_DST_PXP_SRC_PEND_EMPTY = 0"
	# Read register MSDM_REG_QUEUE_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSDM_REG_QUEUE_EMPTY "(\$val != 511)" IDLE_CHK_ERROR "MSDM: MSDM input queues are not all empty"
	# Read register MSDM_REG_RSP_BRB_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f MSDM_REG_RSP_BRB_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "MSDM:  Interface to BRB is full"
	# Read register MSDM_REG_RSP_BRB_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSDM_REG_RSP_BRB_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "MSDM: MSDM_REG_RSP_BRB_PEND_EMPTY = 0"
	# Read register MSDM_REG_RSP_BRB_RDATA_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSDM_REG_RSP_BRB_RDATA_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "MSDM: MSDM_REG_RSP_BRB_RDATA_EMPTY = 0"
	# Read register MSDM_REG_RSP_INT_RAM_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSDM_REG_RSP_INT_RAM_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "MSDM: MSDM_REG_RSP_INT_RAM_PEND_EMPTY = 0"
	# Read register MSDM_REG_RSP_INT_RAM_RDATA_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSDM_REG_RSP_INT_RAM_RDATA_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "MSDM: MSDM_REG_RSP_INT_RAM_RDATA_EMPTY = 0"
	# Read register MSDM_REG_RSP_PXP_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f MSDM_REG_RSP_PXP_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "MSDM: Interface from PXP is full"
	# Read register MSDM_REG_RSP_PXP_RDATA_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSDM_REG_RSP_PXP_RDATA_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "MSDM: MSDM_REG_RSP_PXP_RDATA_EMPTY = 0"
	# Read register MSDM_REG_SH_DELAY_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f MSDM_REG_SH_DELAY_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "MSDM: MSDM_REG_SH_DELAY_EMPTY = 0"
	# Read register YSDM_REG_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f YSDM_REG_INT_STS "(\$val != 0)" IDLE_CHK_ERROR "YSDM: Interrupt 0 is set"
	# Read register YSDM_REG_ASYNC_HOST_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSDM_REG_ASYNC_HOST_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "YSDM: YSDM_REG_ASYNC_HOST_EMPTY = 0"
	# Read register YSDM_REG_CCFC_LOAD_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSDM_REG_CCFC_LOAD_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "YSDM: CCFC_LOAD_PEND_EMPTY = 0"
	# Read register YSDM_REG_CM_DELAY_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSDM_REG_CM_DELAY_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "YSDM: YSDM_REG_CM_DELAY_EMPTY = 0"
	# Read register YSDM_REG_CMSG_QUE_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSDM_REG_CMSG_QUE_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "YSDM: YSDM_REG_CMSG_QUE_EMPTY = 0"
	# Read register YSDM_REG_DELAY_FIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSDM_REG_DELAY_FIFO_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "YSDM: YSDM_REG_DELAY_FIFO_EMPTY = 0"
	# Read register YSDM_REG_DST_BRB_SRC_ADDR_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSDM_REG_DST_BRB_SRC_ADDR_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "YSDM: YSDM_REG_DST_BRB_SRC_ADDR_EMPTY = 0"
	# Read register YSDM_REG_DST_BRB_SRC_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSDM_REG_DST_BRB_SRC_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "YSDM: YSDM_REG_DST_BRB_SRC_PEND_EMPTY = 0"
	# Read register YSDM_REG_DST_INT_RAM_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f YSDM_REG_DST_INT_RAM_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "YSDM: internal ram IF is full"
	# Read register YSDM_REG_DST_INT_RAM_WAIT_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSDM_REG_DST_INT_RAM_WAIT_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "YSDM: YSDM_REG_DST_INT_RAM_WAIT_EMPTY = 0"
	# Read register YSDM_REG_DST_PAS_BUF_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f YSDM_REG_DST_PAS_BUF_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "YSDM: passive buffer IF is full"
	# Read register YSDM_REG_DST_PAS_BUF_WAIT_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSDM_REG_DST_PAS_BUF_WAIT_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "YSDM: YSDM_REG_DST_PAS_BUF_WAIT_EMPTY = 0"
	# Read register YSDM_REG_DST_PXP_DST_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSDM_REG_DST_PXP_DST_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "YSDM: YSDM_REG_DST_PXP_DST_PEND_EMPTY = 0"
	# Read register YSDM_REG_DST_PXP_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f YSDM_REG_DST_PXP_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "YSDM: Interface from PXP is full"
	# Read register YSDM_REG_DST_PXP_IMMED_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSDM_REG_DST_PXP_IMMED_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "YSDM: Immidiate FIFo to PXP DST is not empty"
	# Read register YSDM_REG_DST_PXP_LINK_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSDM_REG_DST_PXP_LINK_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "YSDM: YSDM_REG_DST_PXP_LINK_EMPTY = 0"
	# Read register YSDM_REG_DST_PXP_SRC_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSDM_REG_DST_PXP_SRC_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "YSDM: YSDM_REG_DST_PXP_SRC_PEND_EMPTY = 0"
	# Read register YSDM_REG_QUEUE_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSDM_REG_QUEUE_EMPTY "(\$val != 511)" IDLE_CHK_ERROR "YSDM: YSDM input queues are not all empty"
	# Read register YSDM_REG_RSP_BRB_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f YSDM_REG_RSP_BRB_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "YSDM:  Interface to BRB is full"
	# Read register YSDM_REG_RSP_BRB_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSDM_REG_RSP_BRB_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "YSDM: YSDM_REG_RSP_BRB_PEND_EMPTY = 0"
	# Read register YSDM_REG_RSP_BRB_RDATA_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSDM_REG_RSP_BRB_RDATA_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "YSDM: YSDM_REG_RSP_BRB_RDATA_EMPTY = 0"
	# Read register YSDM_REG_RSP_INT_RAM_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSDM_REG_RSP_INT_RAM_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "YSDM: YSDM_REG_RSP_INT_RAM_PEND_EMPTY = 0"
	# Read register YSDM_REG_RSP_INT_RAM_RDATA_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSDM_REG_RSP_INT_RAM_RDATA_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "YSDM: YSDM_REG_RSP_INT_RAM_RDATA_EMPTY = 0"
	# Read register YSDM_REG_RSP_PXP_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f YSDM_REG_RSP_PXP_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "YSDM: Interface from PXP is full"
	# Read register YSDM_REG_RSP_PXP_RDATA_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSDM_REG_RSP_PXP_RDATA_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "YSDM: YSDM_REG_RSP_PXP_RDATA_EMPTY = 0"
	# Read register YSDM_REG_SH_DELAY_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f YSDM_REG_SH_DELAY_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "YSDM: YSDM_REG_SH_DELAY_EMPTY = 0"
	# Read register PSDM_REG_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f PSDM_REG_INT_STS "(\$val != 0)" IDLE_CHK_ERROR "PSDM: Interrupt 0 is set"
	# Read register PSDM_REG_ASYNC_HOST_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSDM_REG_ASYNC_HOST_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "PSDM: PSDM_REG_ASYNC_HOST_EMPTY = 0"
	# Read register PSDM_REG_CCFC_LOAD_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSDM_REG_CCFC_LOAD_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "PSDM: CCFC_LOAD_PEND_EMPTY = 0"
	# Read register PSDM_REG_CM_DELAY_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSDM_REG_CM_DELAY_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "PSDM: PSDM_REG_CM_DELAY_EMPTY = 0"
	# Read register PSDM_REG_CMSG_QUE_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSDM_REG_CMSG_QUE_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "PSDM: PSDM_REG_CMSG_QUE_EMPTY = 0"
	# Read register PSDM_REG_DELAY_FIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSDM_REG_DELAY_FIFO_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "PSDM: PSDM_REG_DELAY_FIFO_EMPTY = 0"
	# Read register PSDM_REG_DST_BRB_SRC_ADDR_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSDM_REG_DST_BRB_SRC_ADDR_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "PSDM: PSDM_REG_DST_BRB_SRC_ADDR_EMPTY = 0"
	# Read register PSDM_REG_DST_BRB_SRC_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSDM_REG_DST_BRB_SRC_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "PSDM: PSDM_REG_DST_BRB_SRC_PEND_EMPTY = 0"
	# Read register PSDM_REG_DST_INT_RAM_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f PSDM_REG_DST_INT_RAM_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "PSDM: internal ram IF is full"
	# Read register PSDM_REG_DST_INT_RAM_WAIT_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSDM_REG_DST_INT_RAM_WAIT_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "PSDM: PSDM_REG_DST_INT_RAM_WAIT_EMPTY = 0"
	# Read register PSDM_REG_DST_PAS_BUF_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f PSDM_REG_DST_PAS_BUF_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "PSDM: passive buffer IF is full"
	# Read register PSDM_REG_DST_PAS_BUF_WAIT_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSDM_REG_DST_PAS_BUF_WAIT_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "PSDM: PSDM_REG_DST_PAS_BUF_WAIT_EMPTY = 0"
	# Read register PSDM_REG_DST_PXP_DST_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSDM_REG_DST_PXP_DST_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "PSDM: PSDM_REG_DST_PXP_DST_PEND_EMPTY = 0"
	# Read register PSDM_REG_DST_PXP_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f PSDM_REG_DST_PXP_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "PSDM: Interface from PXP is full"
	# Read register PSDM_REG_DST_PXP_IMMED_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSDM_REG_DST_PXP_IMMED_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "PSDM: Immidiate FIFo to PXP DST is not empty"
	# Read register PSDM_REG_DST_PXP_LINK_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSDM_REG_DST_PXP_LINK_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "PSDM: PSDM_REG_DST_PXP_LINK_EMPTY = 0"
	# Read register PSDM_REG_DST_PXP_SRC_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSDM_REG_DST_PXP_SRC_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "PSDM: PSDM_REG_DST_PXP_SRC_PEND_EMPTY = 0"
	# Read register PSDM_REG_QUEUE_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSDM_REG_QUEUE_EMPTY "(\$val != 511)" IDLE_CHK_ERROR "PSDM: PSDM input queues are not all empty"
	# Read register PSDM_REG_RSP_BRB_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f PSDM_REG_RSP_BRB_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "PSDM:  Interface to BRB is full"
	# Read register PSDM_REG_RSP_BRB_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSDM_REG_RSP_BRB_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "PSDM: PSDM_REG_RSP_BRB_PEND_EMPTY = 0"
	# Read register PSDM_REG_RSP_BRB_RDATA_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSDM_REG_RSP_BRB_RDATA_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "PSDM: PSDM_REG_RSP_BRB_RDATA_EMPTY = 0"
	# Read register PSDM_REG_RSP_INT_RAM_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSDM_REG_RSP_INT_RAM_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "PSDM: PSDM_REG_RSP_INT_RAM_PEND_EMPTY = 0"
	# Read register PSDM_REG_RSP_INT_RAM_RDATA_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSDM_REG_RSP_INT_RAM_RDATA_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "PSDM: PSDM_REG_RSP_INT_RAM_RDATA_EMPTY = 0"
	# Read register PSDM_REG_RSP_PXP_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f PSDM_REG_RSP_PXP_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "PSDM: Interface from PXP is full"
	# Read register PSDM_REG_RSP_PXP_RDATA_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSDM_REG_RSP_PXP_RDATA_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "PSDM: PSDM_REG_RSP_PXP_RDATA_EMPTY = 0"
	# Read register PSDM_REG_SH_DELAY_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f PSDM_REG_SH_DELAY_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "PSDM: PSDM_REG_SH_DELAY_EMPTY = 0"
	# Read register XSDM_REG_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f XSDM_REG_INT_STS "(\$val != 0)" IDLE_CHK_ERROR "XSDM: Interrupt 0 is set"
	# Read register XSDM_REG_ASYNC_HOST_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSDM_REG_ASYNC_HOST_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "XSDM: XSDM_REG_ASYNC_HOST_EMPTY = 0"
	# Read register XSDM_REG_CCFC_LOAD_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSDM_REG_CCFC_LOAD_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "XSDM: CCFC_LOAD_PEND_EMPTY = 0"
	# Read register XSDM_REG_CM_DELAY_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSDM_REG_CM_DELAY_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "XSDM: XSDM_REG_CM_DELAY_EMPTY = 0"
	# Read register XSDM_REG_CMSG_QUE_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSDM_REG_CMSG_QUE_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "XSDM: XSDM_REG_CMSG_QUE_EMPTY = 0"
	# Read register XSDM_REG_DELAY_FIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSDM_REG_DELAY_FIFO_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "XSDM: XSDM_REG_DELAY_FIFO_EMPTY = 0"
	# Read register XSDM_REG_DST_BRB_SRC_ADDR_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSDM_REG_DST_BRB_SRC_ADDR_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "XSDM: XSDM_REG_DST_BRB_SRC_ADDR_EMPTY = 0"
	# Read register XSDM_REG_DST_BRB_SRC_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSDM_REG_DST_BRB_SRC_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "XSDM: XSDM_REG_DST_BRB_SRC_PEND_EMPTY = 0"
	# Read register XSDM_REG_DST_INT_RAM_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f XSDM_REG_DST_INT_RAM_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "XSDM: internal ram IF is full"
	# Read register XSDM_REG_DST_INT_RAM_WAIT_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSDM_REG_DST_INT_RAM_WAIT_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "XSDM: XSDM_REG_DST_INT_RAM_WAIT_EMPTY = 0"
	# Read register XSDM_REG_DST_PAS_BUF_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f XSDM_REG_DST_PAS_BUF_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "XSDM: passive buffer IF is full"
	# Read register XSDM_REG_DST_PAS_BUF_WAIT_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSDM_REG_DST_PAS_BUF_WAIT_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "XSDM: XSDM_REG_DST_PAS_BUF_WAIT_EMPTY = 0"
	# Read register XSDM_REG_DST_PXP_DST_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSDM_REG_DST_PXP_DST_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "XSDM: XSDM_REG_DST_PXP_DST_PEND_EMPTY = 0"
	# Read register XSDM_REG_DST_PXP_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f XSDM_REG_DST_PXP_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "XSDM: Interface from PXP is full"
	# Read register XSDM_REG_DST_PXP_IMMED_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSDM_REG_DST_PXP_IMMED_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "XSDM: Immidiate FIFo to PXP DST is not empty"
	# Read register XSDM_REG_DST_PXP_LINK_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSDM_REG_DST_PXP_LINK_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "XSDM: XSDM_REG_DST_PXP_LINK_EMPTY = 0"
	# Read register XSDM_REG_DST_PXP_SRC_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSDM_REG_DST_PXP_SRC_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "XSDM: XSDM_REG_DST_PXP_SRC_PEND_EMPTY = 0"
	# Read register XSDM_REG_QUEUE_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSDM_REG_QUEUE_EMPTY "(\$val != 511)" IDLE_CHK_ERROR "XSDM: XSDM input queues are not all empty"
	# Read register XSDM_REG_RSP_BRB_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f XSDM_REG_RSP_BRB_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "XSDM:  Interface to BRB is full"
	# Read register XSDM_REG_RSP_BRB_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSDM_REG_RSP_BRB_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "XSDM: XSDM_REG_RSP_BRB_PEND_EMPTY = 0"
	# Read register XSDM_REG_RSP_BRB_RDATA_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSDM_REG_RSP_BRB_RDATA_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "XSDM: XSDM_REG_RSP_BRB_RDATA_EMPTY = 0"
	# Read register XSDM_REG_RSP_INT_RAM_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSDM_REG_RSP_INT_RAM_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "XSDM: XSDM_REG_RSP_INT_RAM_PEND_EMPTY = 0"
	# Read register XSDM_REG_RSP_INT_RAM_RDATA_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSDM_REG_RSP_INT_RAM_RDATA_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "XSDM: XSDM_REG_RSP_INT_RAM_RDATA_EMPTY = 0"
	# Read register XSDM_REG_RSP_PXP_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f XSDM_REG_RSP_PXP_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "XSDM: Interface from PXP is full"
	# Read register XSDM_REG_RSP_PXP_RDATA_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSDM_REG_RSP_PXP_RDATA_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "XSDM: XSDM_REG_RSP_PXP_RDATA_EMPTY = 0"
	# Read register XSDM_REG_SH_DELAY_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f XSDM_REG_SH_DELAY_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "XSDM: XSDM_REG_SH_DELAY_EMPTY = 0"
	# Read register USDM_REG_INT_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f USDM_REG_INT_STS "(\$val != 0)" IDLE_CHK_ERROR "USDM: Interrupt 0 is set"
	# Read register USDM_REG_ASYNC_HOST_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USDM_REG_ASYNC_HOST_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "USDM: USDM_REG_ASYNC_HOST_EMPTY = 0"
	# Read register USDM_REG_CCFC_LOAD_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USDM_REG_CCFC_LOAD_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "USDM: CCFC_LOAD_PEND_EMPTY = 0"
	# Read register USDM_REG_CM_DELAY_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USDM_REG_CM_DELAY_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "USDM: USDM_REG_CM_DELAY_EMPTY = 0"
	# Read register USDM_REG_CMSG_QUE_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USDM_REG_CMSG_QUE_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "USDM: USDM_REG_CMSG_QUE_EMPTY = 0"
	# Read register USDM_REG_DELAY_FIFO_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USDM_REG_DELAY_FIFO_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "USDM: USDM_REG_DELAY_FIFO_EMPTY = 0"
	# Read register USDM_REG_DST_BRB_SRC_ADDR_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USDM_REG_DST_BRB_SRC_ADDR_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "USDM: USDM_REG_DST_BRB_SRC_ADDR_EMPTY = 0"
	# Read register USDM_REG_DST_BRB_SRC_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USDM_REG_DST_BRB_SRC_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "USDM: USDM_REG_DST_BRB_SRC_PEND_EMPTY = 0"
	# Read register USDM_REG_DST_INT_RAM_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f USDM_REG_DST_INT_RAM_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "USDM: internal ram IF is full"
	# Read register USDM_REG_DST_INT_RAM_WAIT_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USDM_REG_DST_INT_RAM_WAIT_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "USDM: USDM_REG_DST_INT_RAM_WAIT_EMPTY = 0"
	# Read register USDM_REG_DST_PAS_BUF_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f USDM_REG_DST_PAS_BUF_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "USDM: passive buffer IF is full"
	# Read register USDM_REG_DST_PAS_BUF_WAIT_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USDM_REG_DST_PAS_BUF_WAIT_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "USDM: USDM_REG_DST_PAS_BUF_WAIT_EMPTY = 0"
	# Read register USDM_REG_DST_PXP_DST_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USDM_REG_DST_PXP_DST_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "USDM: USDM_REG_DST_PXP_DST_PEND_EMPTY = 0"
	# Read register USDM_REG_DST_PXP_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f USDM_REG_DST_PXP_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "USDM: Interface from PXP is full"
	# Read register USDM_REG_DST_PXP_IMMED_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USDM_REG_DST_PXP_IMMED_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "USDM: Immidiate FIFo to PXP DST is not empty"
	# Read register USDM_REG_DST_PXP_LINK_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USDM_REG_DST_PXP_LINK_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "USDM: USDM_REG_DST_PXP_LINK_EMPTY = 0"
	# Read register USDM_REG_DST_PXP_SRC_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USDM_REG_DST_PXP_SRC_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "USDM: USDM_REG_DST_PXP_SRC_PEND_EMPTY = 0"
	# Read register USDM_REG_QUEUE_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USDM_REG_QUEUE_EMPTY "(\$val != 511)" IDLE_CHK_ERROR "USDM: USDM input queues are not all empty"
	# Read register USDM_REG_RSP_BRB_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f USDM_REG_RSP_BRB_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "USDM:  Interface to BRB is full"
	# Read register USDM_REG_RSP_BRB_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USDM_REG_RSP_BRB_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "USDM: USDM_REG_RSP_BRB_PEND_EMPTY = 0"
	# Read register USDM_REG_RSP_BRB_RDATA_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USDM_REG_RSP_BRB_RDATA_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "USDM: USDM_REG_RSP_BRB_RDATA_EMPTY = 0"
	# Read register USDM_REG_RSP_INT_RAM_PEND_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USDM_REG_RSP_INT_RAM_PEND_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "USDM: USDM_REG_RSP_INT_RAM_PEND_EMPTY = 0"
	# Read register USDM_REG_RSP_INT_RAM_RDATA_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USDM_REG_RSP_INT_RAM_RDATA_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "USDM: USDM_REG_RSP_INT_RAM_RDATA_EMPTY = 0"
	# Read register USDM_REG_RSP_PXP_IF_FULL val and check if condition on val exist
	IDLE_CHK_1 0x3f USDM_REG_RSP_PXP_IF_FULL "(\$val != 0)" IDLE_CHK_ERROR "USDM: Interface from PXP is full"
	# Read register USDM_REG_RSP_PXP_RDATA_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USDM_REG_RSP_PXP_RDATA_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "USDM: USDM_REG_RSP_PXP_RDATA_EMPTY = 0"
	# Read register USDM_REG_SH_DELAY_EMPTY val and check if condition on val exist
	IDLE_CHK_1 0x3f USDM_REG_SH_DELAY_EMPTY "(\$val != 1)" IDLE_CHK_ERROR "USDM: USDM_REG_SH_DELAY_EMPTY = 0"
	# Read register MISC_REG_AEU_SYS_KILL_OCCURRED val and check if condition on val exist
	IDLE_CHK_1 0x3f MISC_REG_AEU_SYS_KILL_OCCURRED "(\$val != 0)" IDLE_CHK_ERROR "MISC: system kill occured;"
	# Read register MISC_REG_AEU_SYS_KILL_STATUS_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f MISC_REG_AEU_SYS_KILL_STATUS_0 "(\$val != 0)" IDLE_CHK_ERROR "MISC: system kill occured; status_0 register"
	# Read register MISC_REG_AEU_SYS_KILL_STATUS_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f MISC_REG_AEU_SYS_KILL_STATUS_1 "(\$val != 0)" IDLE_CHK_ERROR "MISC: system kill occured; status_1 register"
	# Read register MISC_REG_AEU_SYS_KILL_STATUS_2 val and check if condition on val exist
	IDLE_CHK_1 0x3f MISC_REG_AEU_SYS_KILL_STATUS_2 "(\$val != 0)" IDLE_CHK_ERROR "MISC: system kill occured; status_2 register"
	# Read register MISC_REG_AEU_SYS_KILL_STATUS_3 val and check if condition on val exist
	IDLE_CHK_1 0x3f MISC_REG_AEU_SYS_KILL_STATUS_3 "(\$val != 0)" IDLE_CHK_ERROR "MISC: system kill occured; status_3 register"
	# Read register MISC_REG_AEU_SYS_KILL_STATUS_4 val and check if condition on val exist
	IDLE_CHK_1 0x3f MISC_REG_AEU_SYS_KILL_STATUS_4 "(\$val != 0)" IDLE_CHK_ERROR "MISC: system kill occured; status_4 register"
	# Read register MISC_REG_AEU_SYS_KILL_STATUS_5 val and check if condition on val exist
	IDLE_CHK_1 0x3f MISC_REG_AEU_SYS_KILL_STATUS_5 "(\$val != 0)" IDLE_CHK_ERROR "MISC: system kill occured; status_5 register"
	# Read register MISC_REG_AEU_SYS_KILL_STATUS_6 val and check if condition on val exist
	IDLE_CHK_1 0x3f MISC_REG_AEU_SYS_KILL_STATUS_6 "(\$val != 0)" IDLE_CHK_ERROR "MISC: system kill occured; status_6 register"
	# Read register MISC_REG_AEU_SYS_KILL_STATUS_7 val and check if condition on val exist
	IDLE_CHK_1 0x3f MISC_REG_AEU_SYS_KILL_STATUS_7 "(\$val != 0)" IDLE_CHK_ERROR "MISC: system kill occured; status_7 register"
	# Read register MISC_REG_AEU_SYS_KILL_STATUS_8 val and check if condition on val exist
	IDLE_CHK_1 0x3f MISC_REG_AEU_SYS_KILL_STATUS_8 "(\$val != 0)" IDLE_CHK_ERROR "MISC: system kill occured; status_8 register"
	# Read register MISC_REG_AEU_AFTER_INVERT_1_IGU val and check if condition on val exist
	IDLE_CHK_1 0x3f MISC_REG_AEU_AFTER_INVERT_1_IGU "(\$val != 0)" IDLE_CHK_ERROR "AEU: IGU: AFTER_INVERT_1 is not 0"
	# Read register MISC_REG_AEU_AFTER_INVERT_2_IGU val and check if condition on val exist
	IDLE_CHK_1 0x3f MISC_REG_AEU_AFTER_INVERT_2_IGU "((\$val & ~0x10) != 0)" IDLE_CHK_ERROR "AEU: IGU: AFTER_INVERT_2 is not 0x10"
	# Read register MISC_REG_AEU_AFTER_INVERT_3_IGU val and check if condition on val exist
	IDLE_CHK_1 0x3f MISC_REG_AEU_AFTER_INVERT_3_IGU "(\$val != 0)" IDLE_CHK_ERROR "AEU: IGU: AFTER_INVERT_3 is not 0"
	# Read register MISC_REG_AEU_AFTER_INVERT_4_IGU val and check if condition on val exist
	IDLE_CHK_1 0x3f MISC_REG_AEU_AFTER_INVERT_4_IGU "(\$val != 0)" IDLE_CHK_ERROR "AEU: IGU: AFTER_INVERT_4 is not 0"
	# Read register MISC_REG_AEU_AFTER_INVERT_5_IGU val and check if condition on val exist
	IDLE_CHK_1 0x3f MISC_REG_AEU_AFTER_INVERT_5_IGU "(\$val != 0)" IDLE_CHK_ERROR "AEU: IGU: AFTER_INVERT_5 is not 0"
	# Read register MISC_REG_AEU_AFTER_INVERT_6_IGU val and check if condition on val exist
	IDLE_CHK_1 0x3f MISC_REG_AEU_AFTER_INVERT_6_IGU "(\$val != 0)" IDLE_CHK_ERROR "AEU: IGU: AFTER_INVERT_6 is not 0"
	# Read register MISC_REG_AEU_AFTER_INVERT_7_IGU val and check if condition on val exist
	IDLE_CHK_1 0x3f MISC_REG_AEU_AFTER_INVERT_7_IGU "(\$val != 0)" IDLE_CHK_ERROR "AEU: IGU: AFTER_INVERT_7 is not 0"
	# Read register MISC_REG_AEU_AFTER_INVERT_8_IGU val and check if condition on val exist
	IDLE_CHK_1 0x3f MISC_REG_AEU_AFTER_INVERT_8_IGU "(\$val != 0x20000000)" IDLE_CHK_ERROR "AEU: IGU: AFTER_INVERT_8 is not 0x20000000"
	# Read register MISC_REG_AEU_AFTER_INVERT_9_IGU val and check if condition on val exist
	IDLE_CHK_1 0x3f MISC_REG_AEU_AFTER_INVERT_9_IGU "(\$val != 0)" IDLE_CHK_ERROR "AEU: IGU: AFTER_INVERT_9 is not 0"
	# Read register MISC_REG_AEU_AFTER_INVERT_1_MCP val and check if condition on val exist
	IDLE_CHK_1 0x3f MISC_REG_AEU_AFTER_INVERT_1_MCP "(\$val != 0)" IDLE_CHK_ERROR "AEU: MCP AFTER_INVERT_1 is not 0"
	# Read register MISC_REG_AEU_AFTER_INVERT_2_MCP val and check if condition on val exist
	IDLE_CHK_1 0x3f MISC_REG_AEU_AFTER_INVERT_2_MCP "((\$val & ~0x10) != 0)" IDLE_CHK_ERROR "AEU: MCP AFTER_INVERT_2 is not 0x10"
	# Read register MISC_REG_AEU_AFTER_INVERT_3_MCP val and check if condition on val exist
	IDLE_CHK_1 0x3f MISC_REG_AEU_AFTER_INVERT_3_MCP "(\$val != 0)" IDLE_CHK_ERROR "AEU: MCP AFTER_INVERT_3 is not 0"
	# Read register MISC_REG_AEU_AFTER_INVERT_4_MCP val and check if condition on val exist
	IDLE_CHK_1 0x3f MISC_REG_AEU_AFTER_INVERT_4_MCP "(\$val != 0)" IDLE_CHK_ERROR "AEU: MCP AFTER_INVERT_4 is not 0"
	# Read register MISC_REG_AEU_AFTER_INVERT_5_MCP val and check if condition on val exist
	IDLE_CHK_1 0x3f MISC_REG_AEU_AFTER_INVERT_5_MCP "(\$val != 0)" IDLE_CHK_ERROR "AEU: MCP AFTER_INVERT_5 is not 0"
	# Read register MISC_REG_AEU_AFTER_INVERT_6_MCP val and check if condition on val exist
	IDLE_CHK_1 0x3f MISC_REG_AEU_AFTER_INVERT_6_MCP "(\$val != 0)" IDLE_CHK_ERROR "AEU: MCP AFTER_INVERT_6 is not 0"
	# Read register MISC_REG_AEU_AFTER_INVERT_7_MCP val and check if condition on val exist
	IDLE_CHK_1 0x3f MISC_REG_AEU_AFTER_INVERT_7_MCP "(\$val != 0)" IDLE_CHK_ERROR "AEU: MCP AFTER_INVERT_7 is not 0"
	# Read register MISC_REG_AEU_AFTER_INVERT_8_MCP val and check if condition on val exist
	IDLE_CHK_1 0x3f MISC_REG_AEU_AFTER_INVERT_8_MCP "(\$val != 0x20000000)" IDLE_CHK_ERROR "AEU: MCP AFTER_INVERT_8 is not 0x20000000"
	# Read register MISC_REG_AEU_AFTER_INVERT_9_MCP val and check if condition on val exist
	IDLE_CHK_1 0x3f MISC_REG_AEU_AFTER_INVERT_9_MCP "(\$val != 0)" IDLE_CHK_ERROR "AEU: MCP AFTER_INVERT_9 is not 0"
	# Read register MISCS_REG_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f MISCS_REG_INT_STS_0 "(\$val != 0)" IDLE_CHK_ERROR "MISCS: interrupt status 0 is not 0."
	# Read register MISCS_REG_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f MISCS_REG_INT_STS_1 "(\$val != 0)" IDLE_CHK_ERROR "MISCS: interrupt status 1 is not 0."
	# Read register MISCS_REG_PCIE_HOT_RESET val and check if condition on val exist
	IDLE_CHK_1 0x3f MISCS_REG_PCIE_HOT_RESET "(\$val != 0)" IDLE_CHK_WARNING "MISCS: pcie_rst_b was asserted without perst assertion"
	# Read register TM_REG_PXP_READ_DATA_FIFO_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f TM_REG_PXP_READ_DATA_FIFO_STATUS "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "TM: PXP_READ_DATA_FIFO_STATUS is not 0, FIFO is not empty"
	# Read register TM_REG_PXP_READ_CTRL_FIFO_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f TM_REG_PXP_READ_CTRL_FIFO_STATUS "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "TM: PXP_READ_CTRL_FIFO_STATUS is not 0, FIFO is not empty"
	# Read register TM_REG_CFC_LOAD_ECHO_FIFO_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f TM_REG_CFC_LOAD_ECHO_FIFO_STATUS "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "TM: CFC_LOAD_ECHO_FIFO_STATUS is not 0, FIFO is not empty"
	# Read register TM_REG_CLIENT_OUT_FIFO_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f TM_REG_CLIENT_OUT_FIFO_STATUS "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "TM: TM_REG_CLIENT_OUT_FIFO_STATUS is not 0, FIFO is not empty"
	# Read register TM_REG_CLIENT_IN_PBF_FIFO_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f TM_REG_CLIENT_IN_PBF_FIFO_STATUS "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "TM: TM_REG_CLIENT_IN_PBF_FIFO_STATUS is not 0, FIFO is not empty"
	# Read register TM_REG_CLIENT_IN_XCM_FIFO_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f TM_REG_CLIENT_IN_XCM_FIFO_STATUS "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "TM: TM_REG_CLIENT_IN_XCM_FIFO_STATUS is not 0, FIFO is not empty"
	# Read register TM_REG_CLIENT_IN_TCM_FIFO_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f TM_REG_CLIENT_IN_TCM_FIFO_STATUS "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "TM: TM_REG_CLIENT_IN_TCM_FIFO_STATUS is not 0, FIFO is not empty"
	# Read register TM_REG_CLIENT_IN_UCM_FIFO_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f TM_REG_CLIENT_IN_UCM_FIFO_STATUS "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "TM: TM_REG_CLIENT_IN_UCM_FIFO_STATUS is not 0, FIFO is not empty"
	# Read register TM_REG_EXPIRATION_CMD_FIFO_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f TM_REG_EXPIRATION_CMD_FIFO_STATUS "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "TM: TM_REG_EXPIRATION_CMD_FIFO_STATUS is not 0, FIFO is not empty"
	# Read register TM_REG_AC_COMMAND_FIFO_STATUS val and check if condition on val exist
	IDLE_CHK_1 0x3f TM_REG_AC_COMMAND_FIFO_STATUS "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "TM: TM_REG_AC_COMMAND_FIFO_STATUS is not 0, FIFO is not empty"
	# Read register TM_REG_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f TM_REG_INT_STS_0 "(\$val & 0x7FFFFF)" IDLE_CHK_ERROR "TM: TM_REG_INT_STS_0  bits 22:0 are set, one of the FIFOs overflow or underrun attentions is asserted (bit 0 - address error)"
	# Read register TM_REG_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f TM_REG_INT_STS_0 "(\$val & 0x7F800000)" IDLE_CHK_ERROR "TM: TM_REG_INT_STS_0 bits 30:23 are set, one of the errored commands attentions is asserted"
	# Read register TM_REG_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f TM_REG_INT_STS_0 "(\$val & 0x80000000)" IDLE_CHK_WARNING "TM: TM_REG_INT_STS_0 bit 31 is set, cload_res_loaderr_conn attention is asserted"
	# Read register TM_REG_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x15 TM_REG_INT_STS_1 "(\$val & 0x3E1)" IDLE_CHK_WARNING "TM: TM_REG_INT_STS_1 bits 9,8,7,6,5,0 are set, one of cload_res_loadcancel_task, cload_res_loaderr_task, pxp_rd_data_eop_error, pend_task_scan, pend_conn_scan, cload_res_loadcancel_conn attentions is asserted"
	# Read register TM_REG_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x2a TM_REG_INT_STS_1 "(\$val & 0x381)" IDLE_CHK_WARNING "TM: TM_REG_INT_STS_1 bits 9,8,7,0 are set, one of cload_res_loadcancel_task, cload_res_loaderr_task, pxp_rd_data_eop_error, cload_res_loadcancel_conn attentions is asserted"
	# Read register TM_REG_INT_STS_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f TM_REG_INT_STS_1 "(\$val & 0x41E)" IDLE_CHK_ERROR "TM: TM_REG_INT_STS_1 bits 10,4:1 are set, one of cload_res_validerr_task, pxp_rd_data_eop_bvalid, context_wr_last, context_rd_last, cload_res_validerr_conn attentions is asserted"
	# Read register GRC_REG_INT_STS_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f GRC_REG_INT_STS_0 "(\$val != 0)" IDLE_CHK_ERROR "GRC: GRC_REG_INT_STS_0, one of the interrupts is set, bits 3:0: path_isolation_error, global_reserved_address, timeout_event, address_error."
	# Read register GRC_REG_TRACE_FIFO_VALID_DATA val and check if condition on val exist
	IDLE_CHK_1 0x3f GRC_REG_TRACE_FIFO_VALID_DATA "(\$val != 0)" IDLE_CHK_WARNING "GRC: GRC_REG_TRACE_FIFO_VALID_DATA = 1, the trace FIFO contains at least one valid data."
	# Read register PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ0 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: There are allocated blocks in BTB for VOQ0"
	# Read register PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ1 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ1 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: There are allocated blocks in BTB for VOQ1"
	# Read register PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ2 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ2 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: There are allocated blocks in BTB for VOQ2"
	# Read register PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ3 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ3 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: There are allocated blocks in BTB for VOQ3"
	# Read register PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ4 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ4 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: There are allocated blocks in BTB for VOQ4"
	# Read register PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ5 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ5 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: There are allocated blocks in BTB for VOQ5"
	# Read register PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ6 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ6 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: There are allocated blocks in BTB for VOQ6"
	# Read register PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ7 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ7 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: There are allocated blocks in BTB for VOQ7"
	# Read register PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ8 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ8 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: There are allocated blocks in BTB for VOQ8"
	# Read register PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ9 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ9 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: There are allocated blocks in BTB for VOQ9"
	# Read register PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ10 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ10 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: There are allocated blocks in BTB for VOQ10"
	# Read register PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ11 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ11 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: There are allocated blocks in BTB for VOQ11"
	# Read register PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ12 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ12 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: There are allocated blocks in BTB for VOQ12"
	# Read register PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ13 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ13 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: There are allocated blocks in BTB for VOQ13"
	# Read register PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ14 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ14 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: There are allocated blocks in BTB for VOQ14"
	# Read register PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ15 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ15 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: There are allocated blocks in BTB for VOQ15"
	# Read register PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ16 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ16 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: There are allocated blocks in BTB for VOQ16"
	# Read register PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ17 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ17 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: There are allocated blocks in BTB for VOQ17"
	# Read register PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ18 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ18 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: There are allocated blocks in BTB for VOQ18"
	# Read register PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ19 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_BTB_ALLOCATED_BLOCKS_VOQ19 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: There are allocated blocks in BTB for VOQ19"
	# Read register PBF_REG_YCMD_QS_OCCUPANCY_VOQ0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_OCCUPANCY_VOQ0 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ0 is not empty"
	# Read register PBF_REG_YCMD_QS_OCCUPANCY_VOQ1 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_OCCUPANCY_VOQ1 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ1 is not empty"
	# Read register PBF_REG_YCMD_QS_OCCUPANCY_VOQ2 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_OCCUPANCY_VOQ2 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ2 is not empty"
	# Read register PBF_REG_YCMD_QS_OCCUPANCY_VOQ3 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_OCCUPANCY_VOQ3 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ3 is not empty"
	# Read register PBF_REG_YCMD_QS_OCCUPANCY_VOQ4 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_OCCUPANCY_VOQ4 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ4 is not empty"
	# Read register PBF_REG_YCMD_QS_OCCUPANCY_VOQ5 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_OCCUPANCY_VOQ5 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ5 is not empty"
	# Read register PBF_REG_YCMD_QS_OCCUPANCY_VOQ6 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_OCCUPANCY_VOQ6 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ6 is not empty"
	# Read register PBF_REG_YCMD_QS_OCCUPANCY_VOQ7 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_OCCUPANCY_VOQ7 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ7 is not empty"
	# Read register PBF_REG_YCMD_QS_OCCUPANCY_VOQ8 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_OCCUPANCY_VOQ8 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ8 is not empty"
	# Read register PBF_REG_YCMD_QS_OCCUPANCY_VOQ9 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_OCCUPANCY_VOQ9 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ9 is not empty"
	# Read register PBF_REG_YCMD_QS_OCCUPANCY_VOQ10 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_OCCUPANCY_VOQ10 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ10 is not empty"
	# Read register PBF_REG_YCMD_QS_OCCUPANCY_VOQ11 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_OCCUPANCY_VOQ11 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ11 is not empty"
	# Read register PBF_REG_YCMD_QS_OCCUPANCY_VOQ12 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_OCCUPANCY_VOQ12 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ12 is not empty"
	# Read register PBF_REG_YCMD_QS_OCCUPANCY_VOQ13 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_OCCUPANCY_VOQ13 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ13 is not empty"
	# Read register PBF_REG_YCMD_QS_OCCUPANCY_VOQ14 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_OCCUPANCY_VOQ14 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ14 is not empty"
	# Read register PBF_REG_YCMD_QS_OCCUPANCY_VOQ15 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_OCCUPANCY_VOQ15 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ15 is not empty"
	# Read register PBF_REG_YCMD_QS_OCCUPANCY_VOQ16 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_OCCUPANCY_VOQ16 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ16 is not empty"
	# Read register PBF_REG_YCMD_QS_OCCUPANCY_VOQ17 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_OCCUPANCY_VOQ17 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ17 is not empty"
	# Read register PBF_REG_YCMD_QS_OCCUPANCY_VOQ18 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_OCCUPANCY_VOQ18 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ18 is not empty"
	# Read register PBF_REG_YCMD_QS_OCCUPANCY_VOQ19 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_OCCUPANCY_VOQ19 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ19 is not empty"
	# Read register PBF_REG_YCMD_QS_CMD_CNT_VOQ0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_CMD_CNT_VOQ0 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ0 has commands which have not been processed"
	# Read register PBF_REG_YCMD_QS_CMD_CNT_VOQ1 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_CMD_CNT_VOQ1 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ1 has commands which have not been processed"
	# Read register PBF_REG_YCMD_QS_CMD_CNT_VOQ2 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_CMD_CNT_VOQ2 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ2 has commands which have not been processed"
	# Read register PBF_REG_YCMD_QS_CMD_CNT_VOQ3 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_CMD_CNT_VOQ3 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ3 has commands which have not been processed"
	# Read register PBF_REG_YCMD_QS_CMD_CNT_VOQ4 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_CMD_CNT_VOQ4 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ4 has commands which have not been processed"
	# Read register PBF_REG_YCMD_QS_CMD_CNT_VOQ5 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_CMD_CNT_VOQ5 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ5 has commands which have not been processed"
	# Read register PBF_REG_YCMD_QS_CMD_CNT_VOQ6 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_CMD_CNT_VOQ6 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ6 has commands which have not been processed"
	# Read register PBF_REG_YCMD_QS_CMD_CNT_VOQ7 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_CMD_CNT_VOQ7 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ7 has commands which have not been processed"
	# Read register PBF_REG_YCMD_QS_CMD_CNT_VOQ8 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_CMD_CNT_VOQ8 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ8 has commands which have not been processed"
	# Read register PBF_REG_YCMD_QS_CMD_CNT_VOQ9 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_CMD_CNT_VOQ9 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ9 has commands which have not been processed"
	# Read register PBF_REG_YCMD_QS_CMD_CNT_VOQ10 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_CMD_CNT_VOQ10 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ10 has commands which have not been processed"
	# Read register PBF_REG_YCMD_QS_CMD_CNT_VOQ11 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_CMD_CNT_VOQ11 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ11 has commands which have not been processed"
	# Read register PBF_REG_YCMD_QS_CMD_CNT_VOQ12 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_CMD_CNT_VOQ12 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ12 has commands which have not been processed"
	# Read register PBF_REG_YCMD_QS_CMD_CNT_VOQ13 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_CMD_CNT_VOQ13 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ13 has commands which have not been processed"
	# Read register PBF_REG_YCMD_QS_CMD_CNT_VOQ14 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_CMD_CNT_VOQ14 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ14 has commands which have not been processed"
	# Read register PBF_REG_YCMD_QS_CMD_CNT_VOQ15 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_CMD_CNT_VOQ15 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ15 has commands which have not been processed"
	# Read register PBF_REG_YCMD_QS_CMD_CNT_VOQ16 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_CMD_CNT_VOQ16 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ16 has commands which have not been processed"
	# Read register PBF_REG_YCMD_QS_CMD_CNT_VOQ17 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_CMD_CNT_VOQ17 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ17 has commands which have not been processed"
	# Read register PBF_REG_YCMD_QS_CMD_CNT_VOQ18 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_CMD_CNT_VOQ18 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ18 has commands which have not been processed"
	# Read register PBF_REG_YCMD_QS_CMD_CNT_VOQ19 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_YCMD_QS_CMD_CNT_VOQ19 "(\$val != 0)" IDLE_CHK_ERROR_NO_TRAFFIC "PBF: Y command queue of VOQ19 has commands which have not been processed"
	# Read register BRB_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f BRB_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "BRB: Parity status 0 is not 0"
	# Read register BRB_REG_PRTY_STS_H_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f BRB_REG_PRTY_STS_H_1 "(\$val != 0)" IDLE_CHK_ERROR "BRB: Parity status 1 is not 0"
	# Read register BTB_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f BTB_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "BTB Parity status is not 0"
	# Read register CAU_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f CAU_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "CAU: Parity status is not 0"
	# Read register CCFC_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f CCFC_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "CCFC: Parity status is not 0"
	# Read register CCFC_REG_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f CCFC_REG_PRTY_STS "(\$val != 0)" IDLE_CHK_ERROR "CCFC: CAM Parity status is not 0"
	# Read register CDU_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f CDU_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "CDU: Parity status is not 0"
	# Read register DBG_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f DBG_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "DBG: Parity status is not 0"
	# Read register DMAE_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f DMAE_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "DMAE: Parity status is not 0"
	# Read register DORQ_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f DORQ_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "DORQ: Parity status is not 0"
	# Read register GRC_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f GRC_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "GRC: Parity status is not 0"
	# Read register IGU_REG_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f IGU_REG_PRTY_STS "(\$val != 0)" IDLE_CHK_ERROR "IGU: CAM Parity status is not 0"
	# Read register IGU_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f IGU_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "IGU: Parity status 0 is not 0"
	# Read register IGU_REG_PRTY_STS_H_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f IGU_REG_PRTY_STS_H_1 "(\$val != 0)" IDLE_CHK_ERROR "IGU: Parity status 1 is not 0"
	# Read register IPC_REG_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f IPC_REG_PRTY_STS "(\$val != 0)" IDLE_CHK_ERROR "IPC: Parity status is not 0"
	# Read register MCM_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "MCM: Parity status 0 is not 0"
	# Read register MCM_REG_PRTY_STS_H_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f MCM_REG_PRTY_STS_H_1 "(\$val != 0)" IDLE_CHK_ERROR "MCM: Parity status 1 is not 0"
	# Read register MCP2_REG_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f MCP2_REG_PRTY_STS "(\$val != 0)" IDLE_CHK_ERROR "MCP2: ROM Parity status is not 0"
	# Read register MCP2_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f MCP2_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "MCP2: Parity status is not 0"
	# Read register MSDM_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f MSDM_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "MSDM: Parity status is not 0"
	# Read register MSEM_REG_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f MSEM_REG_PRTY_STS "(\$val != 0)" IDLE_CHK_ERROR "MSEM: SEM FAST Parity status is not 0"
	# Read register MSEM_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f MSEM_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "MSEM: Parity status is not 0"
	# Read register MULD_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f MULD_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "MULD: Parity status is not 0"
	# Read register NCSI_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f NCSI_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "NCSI: Parity status is not 0"
	# Read register NIG_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f NIG_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "NIG: Parity status 0 is not 0"
	# Read register NIG_REG_PRTY_STS_H_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f NIG_REG_PRTY_STS_H_1 "(\$val != 0)" IDLE_CHK_ERROR "NIG: Parity status 1 is not 0"
	# Read register NIG_REG_PRTY_STS_H_2 val and check if condition on val exist
	IDLE_CHK_1 0x3f NIG_REG_PRTY_STS_H_2 "(\$val != 0)" IDLE_CHK_ERROR "NIG: Parity status 2 is not 0"
	# Read register NIG_REG_PRTY_STS_H_3 val and check if condition on val exist
	IDLE_CHK_1 0x3f NIG_REG_PRTY_STS_H_3 "(\$val != 0)" IDLE_CHK_ERROR "NIG: Parity status 3 is not 0"
	# Read register PBF_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "PBF: Parity status 0 is not 0"
	# Read register PBF_REG_PRTY_STS_H_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f PBF_REG_PRTY_STS_H_1 "(\$val != 0)" IDLE_CHK_ERROR "PBF: Parity status 1 is not 0"
	# Read register PCM_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PCM_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "PCM: Parity status is not 0"
	# Read register PGLUE_B_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PGLUE_B_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "PGLUE_B: Parity status is not 0"
	# Read register PRM_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PRM_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "PRM: Parity status is not 0"
	# Read register PRS_REG_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f PRS_REG_PRTY_STS "(\$val != 0)" IDLE_CHK_ERROR "PRS: CAM Parity status is not 0"
	# Read register PRS_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PRS_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "PRS: Parity status 0 is not 0"
	# Read register PRS_REG_PRTY_STS_H_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f PRS_REG_PRTY_STS_H_1 "(\$val != 0)" IDLE_CHK_ERROR "PRS: Parity status 1 is not 0"
	# Read register PSDM_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PSDM_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "PSDM: Parity status is not 0"
	# Read register PSEM_REG_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f PSEM_REG_PRTY_STS "(\$val != 0)" IDLE_CHK_ERROR "PSEM: SEM FAST Parity status is not 0"
	# Read register PSEM_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PSEM_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "PSEM: Parity status is not 0"
	# Read register PSWHST_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWHST_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "PSWHST: Parity status is not 0"
	# Read register PSWRD2_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWRD2_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "PSWRD2: Parity status 0 is not 0"
	# Read register PSWRD2_REG_PRTY_STS_H_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWRD2_REG_PRTY_STS_H_1 "(\$val != 0)" IDLE_CHK_ERROR "PSWRD2: Parity status 1 is not 0"
	# Read register PSWRQ2_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWRQ2_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "PSWRQ2: Parity status is not 0"
	# Read register PSWWR2_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWWR2_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "PSWWR2: Parity status 0 is not 0"
	# Read register PSWWR2_REG_PRTY_STS_H_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWWR2_REG_PRTY_STS_H_1 "(\$val != 0)" IDLE_CHK_ERROR "PSWWR2: Parity status 1 is not 0"
	# Read register PSWWR2_REG_PRTY_STS_H_2 val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWWR2_REG_PRTY_STS_H_2 "(\$val != 0)" IDLE_CHK_ERROR "PSWWR2: Parity status 2 is not 0"
	# Read register PSWWR2_REG_PRTY_STS_H_3 val and check if condition on val exist
	IDLE_CHK_1 0x3f PSWWR2_REG_PRTY_STS_H_3 "(\$val != 0)" IDLE_CHK_ERROR "PSWWR2: Parity status 3 is not 0"
	# Read register PTU_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f PTU_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "PTU: Parity status is not 0"
	# Read register QM_REG_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f QM_REG_PRTY_STS "(\$val != 0)" IDLE_CHK_ERROR "QM: Parity status for specific mems is not 0"
	# Read register QM_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f QM_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "QM: Parity status 0 is not 0"
	# Read register QM_REG_PRTY_STS_H_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f QM_REG_PRTY_STS_H_1 "(\$val != 0)" IDLE_CHK_ERROR "QM: Parity status 1 is not 0"
	# Read register QM_REG_PRTY_STS_H_2 val and check if condition on val exist
	IDLE_CHK_1 0x3f QM_REG_PRTY_STS_H_2 "(\$val != 0)" IDLE_CHK_ERROR "QM: Parity status 2 is not 0"
	# Read register RSS_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f RSS_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "RSS: Parity status is not 0"
	# Read register TCFC_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f TCFC_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "TCFC: Parity status is not 0"
	# Read register TCM_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "TCM: Parity status 0 is not 0"
	# Read register TCM_REG_PRTY_STS_H_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f TCM_REG_PRTY_STS_H_1 "(\$val != 0)" IDLE_CHK_ERROR "TCM: Parity status 1 is not 0"
	# Read register TDIF_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f TDIF_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "TDIF: Parity status is not 0"
	# Read register TM_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f TM_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "TM: Parity status is not 0"
	# Read register TMLD_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f TMLD_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "TMLD: Parity status is not 0"
	# Read register TSDM_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f TSDM_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "TSDM: Parity status is not 0"
	# Read register TSEM_REG_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f TSEM_REG_PRTY_STS "(\$val != 0)" IDLE_CHK_ERROR "TSEM: SEM FAST Parity status is not 0"
	# Read register TSEM_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f TSEM_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "TSEM: Parity status is not 0"
	# Read register UCM_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "UCM: Parity status 0 is not 0"
	# Read register UCM_REG_PRTY_STS_H_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f UCM_REG_PRTY_STS_H_1 "(\$val != 0)" IDLE_CHK_ERROR "UCM: Parity status 1 is not 0"
	# Read register USDM_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f USDM_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "USDM: Parity status is not 0"
	# Read register USEM_REG_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f USEM_REG_PRTY_STS "(\$val != 0)" IDLE_CHK_ERROR "USEM: SEM FAST Parity status is not 0"
	# Read register USEM_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f USEM_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "USEM: Parity status is not 0"
	# Read register XCM_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "XCM: Parity status 0 is not 0"
	# Read register XCM_REG_PRTY_STS_H_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f XCM_REG_PRTY_STS_H_1 "(\$val != 0)" IDLE_CHK_ERROR "XCM: Parity status 1 is not 0"
	# Read register XSDM_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f XSDM_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "XSDM: Parity status is not 0"
	# Read register XSEM_REG_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f XSEM_REG_PRTY_STS "(\$val != 0)" IDLE_CHK_ERROR "XSEM: SEM FAST Parity status is not 0"
	# Read register XSEM_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f XSEM_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "XSEM: Parity status is not 0"
	# Read register XYLD_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f XYLD_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "XYLD: Parity status is not 0"
	# Read register YCM_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "YCM: Parity status 0 is not 0"
	# Read register YCM_REG_PRTY_STS_H_1 val and check if condition on val exist
	IDLE_CHK_1 0x3f YCM_REG_PRTY_STS_H_1 "(\$val != 0)" IDLE_CHK_ERROR "YCM: Parity status 1 is not 0"
	# Read register YSDM_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f YSDM_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "YSDM: Parity status is not 0"
	# Read register YSEM_REG_PRTY_STS val and check if condition on val exist
	IDLE_CHK_1 0x3f YSEM_REG_PRTY_STS "(\$val != 0)" IDLE_CHK_ERROR "YSEM: SEM FAST Parity status is not 0"
	# Read register YSEM_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f YSEM_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "YSEM: Parity status is not 0"
	# Read register YULD_REG_PRTY_STS_H_0 val and check if condition on val exist
	IDLE_CHK_1 0x3f YULD_REG_PRTY_STS_H_0 "(\$val != 0)" IDLE_CHK_ERROR "YULD: Parity status is not 0"

	# Check assertions
	if {$ignoreAssert != 1} {
	    # Check assertions only if device is not hidden
	    if {$::sys(NO_PCI) == 0} {
		if {$::current(DID)} {
		    # TBD: check assertions
		}
	    }
	}
	if {$errors == 0} {
		if {$outfileName != ""} {
		       puts $outfile "Idle_chk completed successfully. (with warnings: $warnings of $total checks)"
		} else {
		       puts "Idle_chk completed successfully. (with warnings: $warnings of $total checks)"
		}
	} else {
		if {$outfileName != ""} {
		       puts $outfile "Idle_chk failed !!! (with errors: $errors warnings: $warnings of $total checks)"
		} else {
		       puts "Idle_chk failed !!! (with errors: $errors warnings: $warnings of $total checks)"
		}
	}
	if {$outfileName != ""} {
	    close $outfile
	}
	return $errors
}
